import os, math

import Blender
from Blender import Draw
from Blender import Material as BMaterial

class TypeBucket(object):
	_t = None
	_l = []
	_title = ''
	@classmethod
	def _makeT(cls):
		if cls._t is None:
			cls._t = {}
			for i, (k, d) in enumerate(cls._l):
				cls._t[k] = i

	@classmethod
	def menuString(cls, limit=False):
		m = [cls._title+'%t']
		for i, (k,d) in enumerate(cls._l):
			if not limit or (type(limit) is list and k in limit):
				m.append('%s%%x%i' % (d, i))
		return '|'.join(m)

	@classmethod
	def isT(cls, a, b):
		cls._makeT()
		if b not in cls._t.keys(): return False
		return cls._t[b] == a

	@classmethod
	def inT(cls, a, b):
		cls._makeT()
		kc = set(cls._t.keys()).intersection(set(b))
		for k in kc:
			if cls._t[k] == a: return True
		return False

	@classmethod
	def _(cls, b):
		cls._makeT()
		if b in cls._t.keys():
			return cls._t[b]
		else:
			return None

class PathsBase(object):
	_p = {
	#	string 'name': (string path, bool auto_create_if_missing, bool fail_if_missing),
	}
	def __init__(self):
		for k in self._p.keys():
			p, create, fail = self._p[k]
			e = os.path.exists(p)
			if not e:
				if fail:
					raise Exception("Required path '%s' (%s) doesn't exist" % (k,p))
				if create:
					try:
						os.makedirs(p)
					except:
						raise Exception("Failed to create required folder '%s' (%s)" % (k,p))
	
	def __repr__(self):
		o=""
		for k in self._p.keys():
			p, create, fail = self._p[k]
			o+=('  %-20s : %s\n' %(k,p))
		return o
	
	def get(self, p):
		return self._p[p][0]
	
	def getContents(self, p, ext):
		'''
		Returns a list of strings which are filenames in the path p,
		with extension ext, sorted case-insensitively. In one line,
		it's hideous I know :P
		'''
		return sorted([i for i in os.listdir(self._p[p][0]) if os.path.splitext(i)[1].lower() == ('.%s'%ext)], key=str.lower)

### ================================================================
### convertMaterial(material)
###		converts a blender-material-to an indigo-material
### ----------------------------------------------------------------
def convertMaterial(mat):
	print("Converting Blender Material: %s" % mat.getName())
	d = {}
	transparent = (mat.mode & BMaterial.Modes.RAYTRANSP) or (mat.getAlpha() < 1.0)
	if transparent or mat.enableSSS:
		# SPECULAR
		d['type'] = 2
		d['precedence'] = 10
		d['cauchy_b_coeff'] = 0.0
		d['specular.type'] = 1
		if transparent:
			d['transparent'] = 1
			d['ior'] = mat.getIOR()
		else:
			d['transparent'] = 0
			d['ior'] = 1.0/(1.001-mat.getRayMirr())
		if d['ior'] < 1.01:  # as values equal or smaller than 1.0 gives unexpected results
			d['ior'] = 1.01
		d['abs.r'] = 1.0-mat.R  # absorption is the inverted color!
		d['abs.g'] = 1.0-mat.G
		d['abs.b'] = 1.0-mat.B
		d['abs.gain'] = mat.getFilter()*100
		if mat.getFilter() == 0:
			d['abs.gain'] = 1
		if mat.enableSSS: #SmartDen
			d['sss'] = 1
			d['sss.r'] = mat.sssR
			d['sss.g'] = mat.sssG
			d['sss.b'] = mat.sssB
		if mat.getAlpha() == 0:
			d['type'] = 4
	#elif transparent and (mat.getAlpha() == 0):
	#	d['type'] = 4
	elif (not transparent and mat.getIOR() > 1) or mat.getSpec() > 0:
		specfac = mat.getSpec()
		if specfac > 0.0001:
			# PHONG
			d['type'] = 1
			d['rgb.r'] = mat.R
			d['rgb.g'] = mat.G
			d['rgb.b'] = mat.B
			d['rgb.gain'] = mat.getRef()
			d['ior'] = 1.0+math.exp(mat.getSpec())/2.71
			d['exponent'] = mat.hard*10
	elif mat.getIOR() == 1:
		# DIFFUSE
		if mat.getDiffuseShader() == BMaterial.Shaders.DIFFUSE_ORENNAYAR:
			d['type'] = 5
			d['sigma'] = mat.getRoughness()
		else:
			d['type'] = 0
			if transparent:
				d['diffuse.transmitter'] = 1
		d['rgb.r'] = mat.R
		d['rgb.g'] = mat.G
		d['rgb.b'] = mat.B
		d['rgb.gain'] = mat.getRef()
	if mat.getEmit() > 0:
		d['emitter']=1
	enabledTextures=mat.enabledTextures
	textures=mat.getTextures()
	for mtex_index in enabledTextures:
		mtex=textures[mtex_index]
		if (mtex) and (mtex.texco & Blender.Texture.TexCo['UV']) and (mtex.tex) and (mtex.tex.type == Blender.Texture.Types.IMAGE) and (mtex.tex.image):
			if mtex.mapto & Blender.Texture.MapTo['COL']:
				uvlayer=mtex.uvlayer
				obj = Blender.Scene.GetCurrent().objects.active
				if uvlayer == '':
					try:
						uvlayer=obj.getData(mesh=1).activeUVLayer
					except:
						uvlayer='uv'
				
				d['albedo.path'] = mtex.tex.image.filename
				d['albedo.uv_set'] = uvlayer
				d['albedo.exponent'] = 2.2
				d['albedo.a'] = 0.0
				d['albedo.b'] = mtex.colfac
				d['albedo.c'] = 0.0
			if mtex.mapto & Blender.Texture.MapTo['NOR']:
				d['bump.path'] = mtex.tex.image.filename
				uvlayer=mtex.uvlayer
				if uvlayer == '':
					obj = Blender.Scene.GetCurrent().objects.active
					uvlayer=obj.getData(mesh=1).activeUVLayer
				d['bump.uv_set'] = uvlayer
				d['bump.exponent'] = 2.2
				d['bump.a'] = 0.0
				d['bump.b'] = mtex.norfac * 0.01
				d['bump.c'] = 0.0
			if mtex.mapto & Blender.Texture.MapTo['DISP']:
				d['displace.path'] = mtex.tex.image.filename
				uvlayer=mtex.uvlayer
				if uvlayer == '':
					obj = Blender.Scene.GetCurrent().objects.active
					uvlayer=obj.getData(mesh=1).activeUVLayer
				d['displace.uv_set'] = uvlayer
				d['displace.exponent'] = 2.2
				d['displace.a'] = 0.0
				d['displace.b'] = mtex.dispfac * 0.01
				d['displace.c'] = 0.0
			if mtex.mapto & Blender.Texture.MapTo['SPEC']:
				d['exponent.path'] = mtex.tex.image.filename
				uvlayer=mtex.uvlayer
				if uvlayer == '':
					obj = Blender.Scene.GetCurrent().objects.active
					uvlayer=obj.getData(mesh=1).activeUVLayer
				d['exponent.uv_set'] = uvlayer
				d['exponent.exponent'] = 2.2
				d['exponent.a'] = 0.0
				d['exponent.b'] = 1000
				d['exponent.c'] = 0.0
	mat.properties['indigo'] = d
	#drawGUI()
	Draw.Redraw()

class Material(object):
	def __init__(self, blenderMaterial):
		self.blenderMaterial = blenderMaterial

		self.properties = {}
		
		if "indigo" not in self.blenderMaterial.properties.keys():
			convertMaterial(self.blenderMaterial)
			
		self.properties = self.blenderMaterial.properties['indigo'].convert_to_pyobject()

		self.albedo = Types.Channel()
		self.albedo.enabled.val = True
		if (self.read('albedo.path') != ''):
			self.albedo.type.val = Types.Channel._('TEXTURE')
		
		self.displacement = Types.Channel()
		self.displacement.type.val = Types.Channel._('TEXTURE')
		self.displacement.enabled.val = (self.read('displacement.path') != '')

		self.bump = Types.Channel()
		self.bump.type.val = Types.Channel._('TEXTURE')
		self.bump.enabled.val = (self.read('bump.path') != '')

		self.exponent = Types.Channel()
		self.exponent.type.val = Types.Channel._('TEXTURE')
		self.exponent.enabled.val = (self.read('exponent.path') != '')
		
		#absorption_layer_transmittance
		self.alt = Types.Channel()
		self.alt.type.val = Types.Channel._('TEXTURE')
		self.alt.enabled.val = (self.read('alt.path') != '')

	def read(self, name):
		if name in self.properties:
			return self.properties[name]
		else:
			return ""

class Types(object):
	class Channel(TypeBucket):
		_title = 'Channel type'
		_l	 = [
			('COLOR',	'Colour'),
			('TEXTURE',	'Texture'),
			('SHADER',	'Shader'),
		]
		def __init__(self):
			self.enabled = Draw.Create(False)
			self.type = Draw.Create(False)
		
		def typeShader(self):
			return self.isT(self.type, 'SHADER')
		def typeTexture(self):
			return self.isT(self.type, 'TEXTURE')
		def typeColor(self):
			return self.isT(self.type, 'COLOR')
		
	
	class Medium(TypeBucket):
		_title = 'Medium type'
		_l	 = [
			('BASIC',		'Basic'),
			('DERMIS',		'Dermis'),
			('EPIDERMIS',	'Epidermis'),
			#('ATMOSPHERE',  'Atmosphere'), # not 100% working
		]

	class Phase(TypeBucket):
		_title = 'Phase type'
		_l	 = [
			('UNIFORM',	'Uniform'),
			('HG',		'Henyey Greenstein'),
		]

	class Spectrum(TypeBucket):
		_title = 'Spectrum type'
		_l	 = [
			('RGB',				'RGB'),
			('UNIFORM',			'Uniform'),
			('BLACKBODY',		'Black-body'),
			('PEAK',			'Peak'),
			('LAMPSPECTRUM',	'Lamp spectrum'),  # regular-tabulated in disguise with preset data
		]

	class Specular(TypeBucket):
		_title = 'Specular type'
		_l	 = [
			('SPECULAR',	'Specular'),
			('GLOSSY',		'Glossy Transparent'),
		]

	class Materials(TypeBucket):
		_title = 'Material type'
		_l	 = [
			#('LIGHT',	   'Light'),
			('DIFFUSE',		'DIFFUSE = Flat matte paper-like material'),
			('ORENNAYAR',	'OREN-NAYAR = Rough clay material'),
			('PHONG',		'PHONG = Shiny, reflective, metallic materials'),
			('SPECULAR',	'SPECULAR = Transparent, semi-transparent  material'),
			('BLEND',		'BLENDED = A blend between two other materials'),
			('EXITPORTAL',	'EXIT PORTAL'),
			('INCLUDE',		'EXTERNAL'),
		]

	class Filters(TypeBucket):
		_title = 'Filter type'
		_l	 = [
			('BOX',			'Box'),
			('FASTBOX',		'FastBox'),
			('GAUSSIAN',	'Gaussian'),
			('MN',			'Mitchell-Netravali'),
		]

	class Tonemapping(TypeBucket):
		_title = 'ToneMap type'
		_l	 = [
			('LINEAR',		'Linear'),
			('REINHARD',	'Reinhard'),
			('CAMERA',		'Camera'),
		]

	class Environment(TypeBucket):
		_title = 'Environment type'
		_l	 = [
			('NONE',		'None (mesh emitters only)'),
			('BACKGROUND',	'Background'),
			('SUNSKY',		'Physical Sun + Sky'),
			('MAP',			'HDRI Environment map'),
		]
		
	class EmissionMeasure(TypeBucket):
		_title = 'Emission Measure'
		_l	 = [
			('FLUX',		'Luminous Flux'),
			('INTENSITY',	'Luminous Intensity'),
			('EMITTANCE',	'Luminous Emittance'),
			('LUMINANCE',	'Luminance'),
		]
		
		@classmethod
		def units(cls, val):
			if val == cls._('FLUX'):
				return 'lumens (lm) - bulb = 1500, sun = 600000'
			elif val == cls._('INTENSITY'):
				return 'candela (lm/sr)'
			elif val == cls._('EMITTANCE'):
				return 'lux (lm/m^2)'
			elif val == cls._('LUMINANCE'):
				return 'candela/sq. metre (cd/m^2)'
			else:
				return ''
			
	class MaterialInclude(TypeBucket):
		_title = 'External materials location'
		_l	 = [
			('LOCAL',	'Local file path'),
			('MATDB',	'Materials DB Downloads'),
		]
		
	class Networking(TypeBucket):
		_title  = 'Networking type'
		_l	  = [
			('OFF',		'Off'),
			('MASTER',	'Master'),
			('WORKING',	'Working Master'),
		]

class Events(object):
	evtNoEvt				  = 0
	
	class UI(object):
		evtExport			= 1
		evtExportAnim		= 2
		evtIsNumber			= 3
		evtSetDofDist		= 4
		evtFocusSelected	= 5
		evtFocusCursor		= 6
		openCamera			= 7
		openEnv				= 8
		openRSet			= 9
		openSSet			= 10
		openTmap			= 11
		evtloadimg			= 12
		evtchangesize		= 13
		evtHStop			= 14
		openMat				= 15
		evtloadobstacle		= 16
		evtloadaperture		= 17
		openMesh			= 18
		evtSubdivMesh		= 19
		evtUpdateSubdiv		= 20
		evtSetIndigoOutPath	= 21
		evtSetIndigoPath	= 22
		
	class Materials(object):
		evtMatBase				= 100 # first event-number for material editor module
		
		evtMatsMenu				= evtMatBase + 1
		evtMatType				= evtMatBase + 2
		evtMatEmittIESPath		= evtMatBase + 3
		evtMatCopy				= evtMatBase + 4
		evtMatPaste				= evtMatBase + 5
		evtMatRgbPicker			= evtMatBase + 6
		evtMatEmittRgbPicker	= evtMatBase + 7
		evtMatRgbPicker_s		= evtMatBase + 8
		evtMatBlendMatA			= evtMatBase + 9
		evtMatBlendMatB			= evtMatBase + 10
		evtMatAlbedoPath		= evtMatBase + 11
		evtMatAlbedoISL			= evtMatBase + 12
		evtMatBumpPath			= evtMatBase + 13
		evtMatExponentPath		= evtMatBase + 14
		evtMatBlendPath			= evtMatBase + 15
		evtMatSssRgbPicker		= evtMatBase + 16
		evtMatPreview			= evtMatBase + 17
		evtMatInclude			= evtMatBase + 18
		evtMatAbsPicker			= evtMatBase + 19
		evtPreviewScene			= evtMatBase + 20
		evtMatConvert			= evtMatBase + 21
		evtMatSave				= evtMatBase + 22
		evtDisplaceImage		= evtMatBase + 23
		evtALTImage				= evtMatBase + 24
		
		evtMatDBChoose			= evtMatBase + 25
		evtMatDBSearch			= evtMatBase + 26
		evtMatDBDownload		= 9000				# Need an out-of-range number here, as we add material IDs to it
