import Blender
from Blender import BGL, Draw, Scene, Window

from indigo import Types		as IndigoTypes

# Adapted from luxblend (with thanks)
class Layout:
	def __init__(self, y=-1):
		self.x = 120 # left start position after captions
		self.xmax = 110+2*(140+4)
		
		if(y==-1):
			self.y = Window.GetAreaSize()[1] - 70
		else:
			self.y = y
		
		self.w = 140	# default element width in pixels
		self.h = 18		# default element height in pixels
		self.hmax = 0
		self.xgap = 4
		self.ygap = 4
		self.resethmax = False
		self.left = 15
		self.control = 110
		self.compact = False
		
	def units(self, unit, controlWidth=0):
		self.x += controlWidth + 5
		BGL.glColor3f(0.5,0.5,0.5);
		self.text(unit)
		BGL.glColor3f(0.9,0.9,0.9) ;

	def text(self, text):
		BGL.glRasterPos2i(self.x,self.y+5);
		Draw.Text(text, 'small')
		self.x += Draw.GetStringWidth(text) + 5

	def title(self, title):
		BGL.glColor3f(0.6,0.6,0.6) ;
		BGL.glRectf(10,self.y,Draw.GetStringWidth(title)+20,self.y-30)
		
		BGL.glColor3f(0.1,0.1,0.1) ;
		BGL.glRectf(12,self.y,Draw.GetStringWidth(title)+18,self.y-28)

		BGL.glColor3f(0.2,0.2,0.2)
		BGL.glRasterPos2i(self.left+1,self.y-21);
		Draw.Text(title)
		
		BGL.glColor3f(1,1,1)
		BGL.glRasterPos2i(self.left,self.y-20);
		Draw.Text(title)
		
		self.newline()
		self.y -= 10

	def subtitle(self, title):
		self.y -= 15
		
		BGL.glColor3f(0.4,0.4,0.4) ;
		BGL.glRectf(10,self.y,Draw.GetStringWidth(title)+20,self.y-25)
		BGL.glRectf(0,self.y-23,460,self.y-25)
		
		BGL.glColor3f(0.2,0.2,0.2)
		BGL.glRasterPos2i(self.left+1,self.y-15);
		Draw.Text(title)
		
		BGL.glColor3f(1,1,1)
		BGL.glRasterPos2i(self.left,self.y-14);
		Draw.Text(title)
		
		self.newline()

		if self.compact:
			self.y -= 8
		else:
			self.y -= 5

	def reset(self):
		BGL.glColor3f(0.9,0.9,0.9) ;
	
	def newline(self, title="", distance=0, level=0, icon=None, color=None):
		if self.compact:
			self.y -= 18
		else:
			self.y -= 21
			
		self.x = 120
		
		self.reset()

		if title!="":
			BGL.glRasterPos2i(self.left,self.y + 5)
			Draw.Text(title, 'small')
		
		self.y -= distance
		self.reset()

class scrollbar:
	def __init__(self):
		self.position = 0 # current position at top (inside 0..height-viewHeight)
		self.height = 0 # total height of the content
		self.viewHeight = 0 # height of window
		self.x = 0 # horizontal position of the scrollbar
		self.scrolling = self.over = False # start without scrolling ;)
	def calcRects(self):
		# Blender doesn't give us direct access to the window size yet, but it does set the
		# GL scissor box for it, so we can get the size from that. (thx to Daniel Dunbar)
		size = BGL.Buffer(BGL.GL_FLOAT, 4)
		BGL.glGetFloatv(BGL.GL_SCISSOR_BOX, size)
		size = size.list # [winx, winy, width, height]
		self.winrect = size[:]
		self.viewHeight = size[3]
		size[0], size[1] = size[2]-20, 0 # [scrollx1, scrolly1, scrollx2, scrolly2]
		self.rect = size[:]
		if self.position < 0: self.position = 0
		if self.height < self.viewHeight: self.height = self.viewHeight
		if self.position > self.height-self.viewHeight: self.position = self.height-self.viewHeight
		self.factor = (size[3]-size[1]-4)/self.height
		self.sliderRect = [size[0]+2, size[3]-2-(self.position+self.viewHeight)*self.factor, size[2]-2, size[3]-2-self.position*self.factor]
	def draw(self):
		self.calcRects()
		BGL.glColor3f(0.5,0.5,0.5); BGL.glRectf(self.rect[0],self.rect[1],self.rect[2],self.rect[3])
		if self.over or self.scrolling: BGL.glColor3f(1.0,1.0,0.7)
		else: BGL.glColor3f(0.7,0.7,0.7)
		BGL.glRectf(self.sliderRect[0],self.sliderRect[1],self.sliderRect[2],self.sliderRect[3])
	def getTop(self):
		return self.viewHeight+self.position
	def scroll(self, delta):
		self.position = self.position + delta
		self.calcRects()
		Draw.Redraw()
	def Mouse(self):
		self.calcRects()
		coord, buttons = Window.GetMouseCoords(), Window.GetMouseButtons()
		over = (coord[0]>=self.winrect[0]+self.rect[0]) and (coord[0]<=self.winrect[0]+self.rect[2]) and \
			   (coord[1]>=self.winrect[1]+self.rect[1]) and (coord[1]<=self.winrect[1]+self.rect[3])
		if Window.MButs.L and buttons > 0:
			if self.scrolling:
				if self.factor > 0: self.scroll((self.lastcoord[1]-coord[1])/self.factor)
				Draw.Redraw()
			elif self.over:
				self.scrolling = True
			self.lastcoord = coord
		elif self.scrolling:
			self.scrolling = False
			Draw.Redraw()
		if self.over != over: Draw.Redraw()
		self.over = over
		
#===============================================================================
# UIControls - Allocate GUI controls and set defaults
#===============================================================================
class UIControls:
	@staticmethod
	def OnOffToggleText(ctrl):
		return 'On' if ctrl.val else 'Off'
	
	@staticmethod
	def PlusMinusToggleText(ctrl):
		return '-' if ctrl.val else '+'
	
	class Materials:
		matGui = Draw.Create(0)
		matType = IndigoTypes.Materials._('PHONG')
		matTypeMenu = Draw.Create(1) #workaround for buggy GUI
		matDiffTrans = Draw.Create(0)
		
		matEmitt = Draw.Create(0)
		matEmittType = Draw.Create( IndigoTypes.Spectrum._('RGB') )
		
		matEmittIES=Draw.Create(0)
		matEmittIESPath = Draw.Create('')
		
		matEmittScaleValue = Draw.Create(1500)
		matEmittScaleMeasure = Draw.Create( IndigoTypes.EmissionMeasure._('FLUX') )
		
		matEmittGain = Draw.Create(1.0)
		matEmittMult = Draw.Create(0)
		matEmittRgb = Draw.Create(1.0,1.0,1.0)
		matEmittRgbR = Draw.Create(1.0)
		matEmittRgbG = Draw.Create(1.0)
		matEmittRgbB = Draw.Create(1.0)
		matEmittLayer = Draw.Create('default')
		matEmittTemp = Draw.Create(5000.0)
		matEmittLSPreset = Draw.Create('')
		
		matTogglePhong=Draw.Create(0)
		matRgb = Draw.Create(0.8, 0.8, 0.8)
		matRgbR = Draw.Create(0.8)
		matRgbG = Draw.Create(0.8)
		matRgbB = Draw.Create(0.8)
		matRgb_s = Draw.Create(0.8, 0.8, 0.8)
		matRgbR_s = Draw.Create(0.8)
		matRgbG_s = Draw.Create(0.8)
		matRgbB_s = Draw.Create(0.8)
		
		matRgbGain = Draw.Create(1.0)
		matIOR = Draw.Create(1.0)
		matExponent = Draw.Create(100)
		matNKData = Draw.Create('')
		matSigma = Draw.Create(0.2)
		matTransparent = Draw.Create(0)
		matSpecularType = Draw.Create( IndigoTypes.Specular._('SPECULAR') )
		matPrecedence = Draw.Create(10)
		matMediumType = Draw.Create( IndigoTypes.Medium._('BASIC') )
		matCauchyBCoeff = Draw.Create(0.0)
		matAbsorptionType = Draw.Create( IndigoTypes.Spectrum._('RGB') )
		matAbs = Draw.Create(0.5, 0.5, 0.5)
		matAbsR = Draw.Create(0.0)
		matAbsG = Draw.Create(0.0)
		matAbsB = Draw.Create(0.0)
		matAbsGain = Draw.Create(1.0)
		matSss = Draw.Create(0)
		matSssType = Draw.Create( IndigoTypes.Spectrum._('RGB') )
		matSssR = Draw.Create(0.8)
		matSssG = Draw.Create(0.8)
		matSssB = Draw.Create(0.8)
		matSssRgb = Draw.Create(0.8, 0.8, 0.8)
		matSssGain = Draw.Create(1.0)
		matSssUniform = Draw.Create(1.0)
		matSssUniformExp = Draw.Create(0.0)
		matSssPhaseType = Draw.Create( IndigoTypes.Phase._('UNIFORM') )
		matSssPhaseSpec=Draw.Create(1)
		matSssPhaseSpecUni = Draw.Create(0.5)
		matSssPhaseSpecR=Draw.Create(0.5)
		matSssPhaseSpecG=Draw.Create(0.5)
		matSssPhaseSpecB=Draw.Create(0.5)
		matAlbedoUVset = Draw.Create('UVTex')
		matAlbedoPath = Draw.Create('')
		matAlbedoISL = Draw.Create(0)
		matBumpISL=Draw.Create(0)
		matExponentISL=Draw.Create(0)
		matAlbedoShader = Draw.Create(0)
		matBumpShader=Draw.Create(0)
		matExponentShader=Draw.Create(0)
		matAlbedoExponent = Draw.Create(2.2)
		matAlbedoA = Draw.Create(0.0)
		matAlbedoB = Draw.Create(1.0)
		matAlbedoC = Draw.Create(0.0)
		matBumpUVset = Draw.Create('UVTex')
		matBumpPath = Draw.Create('')
		matBumpExponent = Draw.Create(2.2)
		matBumpA = Draw.Create(0.0)
		matBumpB = Draw.Create(0.01)
		matBumpC = Draw.Create(0.0)
		matExponentUVset = Draw.Create('UVTex')
		matExponentPath = Draw.Create('')
		matExponentExponent = Draw.Create(2.2)
		matExponentA = Draw.Create(0.0)
		matExponentB = Draw.Create(10000)
		matExponentC = Draw.Create(0.0)
		matBlendMatA = Draw.Create('')
		matBlendMatB = Draw.Create('')
		matBlendFactor = Draw.Create(0.5)
		matBlendUVset = Draw.Create('UVTex')
		matBlendPath = Draw.Create('')
		matBlendExponent = Draw.Create(1.0)
		matBlendA = Draw.Create(0.0)
		matBlendB = Draw.Create(1.0)
		matBlendC = Draw.Create(0.0)
		
		matInclude = Draw.Create('')
		
		matIncludeUseDB = Draw.Create( 0 )
		matDBInclude = Draw.Create(0)
		
		matDatabaseSearchString = Draw.Create('')
		matDatabaseResultSet = []
		
		
		PreviewScene = Draw.Create(3)
		matHemoglobin = Draw.Create(0.001)
		matMelanin = Draw.Create(0.15)
		matEumelanin = Draw.Create(0.001)
		matTurbidity=Draw.Create(2.2)
		matMediumType = Draw.Create( IndigoTypes.Medium._('BASIC') )
		
		matDisplaceShader=Draw.Create(0)
		matDisplaceISL=Draw.Create(0)
		matDisplaceType=Draw.Create(0)
		matDisplacePath=Draw.Create('')
		matDisplaceExp=Draw.Create(2.2)
		matDisplaceUVSet=Draw.Create('UVTex')
		matDisplaceA=Draw.Create(0.0)
		matDisplaceB=Draw.Create(0.01)
		matDisplaceC=Draw.Create(0.0)
		matDisplaceFBMUScale=Draw.Create(1.00)
		matDisplaceFBMVScale=Draw.Create(1.00)
		
		# Absorption Layer Transmittance
		matALTShader=Draw.Create(0)
		matALTISL=Draw.Create(0)
		matALTType=Draw.Create(0)
		matALTPath = Draw.Create('')
		matALTExp=Draw.Create(2.2)
		matALTUVSet=Draw.Create('UVTex')
		matALTA=Draw.Create(0.0)
		matALTB=Draw.Create(1.0)
		matALTC=Draw.Create(0.0)
	
		matsMenu = Draw.Create(0)
		matClipboard = None
		matPreviewFile = "IMmatdb.xml"
	
		materialGUIwidth = 440
		currentMaterial = None
		
	class Mesh:
		meshGui = Draw.Create(0)
		meshSubdivide=Draw.Create(0)
		MaxNumSubdiv = Draw.Create(2)
		SubdivPixelTreshold=Draw.Create(0.0)
		SubdivCurvatureTreshold=Draw.Create(0.1)
		SubdivSmooth = Draw.Create(1)
		#subdivMeshes={}
		
	class Camera:
		tonemapcurves = {}
		
		# Dimensions
		strScaleSize = "Render scale:%t | 200 % %x200 | 150 % %x150 | 100 % %x100 | 75 % %x75 | 50 % %x50 | 25 % %x25 | 20 % %x20 | 10 % %x10 "
		ScaleSize = Draw.Create(100)
		
		# Film setting
		FilmWidth = Draw.Create(35)
		FilmIso = Draw.Create(100)
		Shutter = Draw.Create(125)
		Autoexp = Draw.Create(0)
		FStop = Draw.Create(80)
		FocusDistance = Draw.Create(2.0)
		LensShiftUp=Draw.Create(0.0)
		LensShiftRight=Draw.Create(0.0)
		
		# Aperture Diffraction
		ApertureDiffr = Draw.Create(1)
		AperturePProc = Draw.Create(0)
		ApertureType = Draw.Create(1)
		BladesNum = Draw.Create(6)
		BladeOffset = Draw.Create(0.4)
		BladeCurvature = Draw.Create(3)
		StartAngle = Draw.Create(0.2)
		ObstacleMap = Draw.Create('')
		ApertureImg = Draw.Create('')
		
		# Camera setting
		AutoFocus = Draw.Create(1)
		SelFocus = Draw.Create(0)
		CurFocus = Draw.Create(0)
		
		# Motion Blur
		CamMoBlur = Draw.Create(0)
		ObjMoBlur = Draw.Create(0)
		PrtMoBlur = Draw.Create(0)
		
		# White balance
		strWhiteBalance = "White Balance:%t | E %x0 | D50 %x1 | D55 %x2 | D65 %x3 | D75 %x4 "
		strWhiteBalance += "| A %x5 | B %x6 | C %x7 | 9300 %x8 | F2 %x9 | F7 %x10 | F11 %x11"
		WhiteBalance = Draw.Create(0)
		whiteBalanceV = [ "E","D50","D55","D65","D75","A","B","C","9300","F2","F7","F11" ]
		
		SizeX = Draw.Create( Scene.GetCurrent().getRenderingContext().imageSizeX() )
		SizeY = Draw.Create( Scene.GetCurrent().getRenderingContext().imageSizeY() )
		
	class Environment:
		# World scale
		WorldScale = Draw.Create(1.0)
		WorldScaleExp = Draw.Create(0)
		
		## Environment Type
		EnvType = Draw.Create( IndigoTypes.Environment._('SUNSKY') )
		
		# Physical Sky+Sun
		SunLayerToggle = Draw.Create(1)
		SunLayer = Draw.Create(0)
		SkyLayerToggle = Draw.Create(1)
		SkyLayer = Draw.Create(1)
		Turbidity = Draw.Create(2.2)
		
		## Environment map
		EnvFile = Draw.Create("none")
		strEnvMapType = "Map type:%t | Spherical %x0 | LatLong %x1"
		EnvMapType = Draw.Create(1)
		EnvMapTypeV = ["spherical", "latlong"]
		
		EnvGain = Draw.Create(1.0)
		EnvWidth = Draw.Create(640)
		
		## Background color
		BGColor = Draw.Create(0.0, 0.0, 0.0)
		BGGain = Draw.Create(1.0)
		
		ExtraAtm = Draw.Create(0)			# DH not present on UI
		
	class Tonemapping:
		## Tonemapping
		ToneMapType = Draw.Create( IndigoTypes.Tonemapping._('REINHARD') )
		# Reinhard
		ToneMapPreScale = Draw.Create(3.0)
		ToneMapPostScale = Draw.Create(1.0)
		ToneMapBurn = Draw.Create(6.0)
		# Linear
		ToneMapScale = Draw.Create(1.0)
		ToneMapScaleExp = Draw.Create(0)
		# Camera
		ToneMapResponseCurve=Draw.Create(0)
		ToneMapEV=Draw.Create(0)
		
	class Renderer:
		AlphaMask=Draw.Create(0)
		ClayRender = Draw.Create(0)
		
		MLT = Draw.Create(0)
		Bidirectional = Draw.Create(1)
		
		rendererAdvanced = Draw.Create(0)
		
		SplatFilter=Draw.Create(2)
		SplatF_MN_Blur=Draw.Create(0.3)
		SplatF_MN_Ring=Draw.Create(0.3)
		DownsizeFilter=Draw.Create(2)
		DownsizeF_MN_Blur=Draw.Create(0.3)
		DownsizeF_MN_Ring=Draw.Create(0.3)
		MaxNumConsRej = Draw.Create(1000)
		SuperSample = Draw.Create(2)
		
	class System:
		ExportMeshs = Draw.Create(1)
		FlybyMode = Draw.Create(0)
		
		IndigoPath = Draw.Create('')
		
		Threads = Draw.Create(1)
		AutoThreads = Draw.Create(1)	# DH Renamed
#		DisplayPeriod = Draw.Create(20)
		ImageSave = Draw.Create(120)
		
		HaltTime = Draw.Create(-1)
		HaltSamples = Draw.Create(-1)
		CloseGUIOnHalt = Draw.Create(0)
		
		Watermark = Draw.Create(1)
		SaveUTMExr = Draw.Create(0)
		SaveTMExr = Draw.Create(0)
		SaveIgi = Draw.Create(0)
		
		NetworkWM = Draw.Create(0)
		
		Logging = Draw.Create(1)
		IOverlay=Draw.Create(0)
		IndigoOutPath = Draw.Create('')
		DefaultExport = Draw.Create(1)
		ExecuteIndigo = Draw.Create(1)
		ProcessPrio = Draw.Create(0)	# Gets reset later, platform dependent


class exportProgressBar(object):
	totalSteps		= None
	realSteps		= None
	currentStep		= None
	subMode			= False
	counterStep		= None
	
	def __init__(self, ts, other=None):
		self.realSteps = ts
		self.counterStep = 0
		if other is None:
			# Master progress indicator
			self.totalSteps  = self.realSteps
			self.currentStep = 0
		else:
			# sub-progress of another progress indicator
			self.totalSteps  = self.realSteps * other.totalSteps
			self.currentStep = self.realSteps * (other.currentStep-1)
			self.subMode = True
			
	def __repr__(self):
		return '<progress total:%i real:%i current:%i count:%i frac:%f>' % (self.totalSteps, self.realSteps, self.currentStep, self.counterStep, self.get_frac())
			
	def get_frac(self):
		return float(self.currentStep) / float(self.totalSteps)
	
	def next(self, msg):
		amt = self.get_frac()
		Window.DrawProgressBar(amt, msg)
		#print('%s %s'%(self,msg))
		# Blender.Redraw()
		self.currentStep += 1
		
	def counter(self, prefix):
		
		amt = self.get_frac()
		if self.subMode:
			msg = '%s (%i/%i)' % (prefix, self.counterStep, self.realSteps)
		else:
			msg = '%s (%i/%i)' % (prefix, self.counterStep, self.totalSteps)
		Window.DrawProgressBar(amt, msg)
		#print('%s %s'%(self,msg))
		# Blender.Redraw()
		self.currentStep += 1
		self.counterStep += 1
		
	def finished(self):
		if not self.subMode:
			Window.DrawProgressBar(1.0, 'Finished')
			Blender.Redraw()