# Utility functions

import os, sys, re, platform, getpass
import Blender

class PlatformInformation(object):
	'''
	Gather info about this machine for the <metadata> tag
	'''
	type = None
	uname = None
	python = None
	user = None
	def __init__(self):
		self.type = 'Unknown platform'
		if platform.system() == 'Linux':
			self.type = ' '.join(platform.linux_distribution())
		elif platform.system() == 'Windows':
			self.type = ' '.join(platform.win32_ver())
		elif platform.system() == 'Darwin':
			self.type = ' '.join([' '.join(i) if type(i) is tuple else i for i in platform.mac_ver()])
		self.uname = ' '.join(platform.uname())
		self.python = sys.version
		self.user = getpass.getuser()

def isLinux():
	#return not isWindows() and not isMac()
	return sys.platform[:5].lower() == 'linux'
	
def isMac():
	#return (not isWindows()) and (os.popen("uname").read().strip() == "Darwin")
	return sys.platform[:6].lower() == 'darwin'
	
def isWindows():
	return sys.platform[:3].lower() == 'win'

#def getAssetsPath():
#	return os.path.join(Blender.Get("scriptsdir"), "blendigopkg", "assets")
	
def revealFile(f):
	filename = os.path.basename(f)
	path = os.path.dirname(f)
	
	if isMac():
		os.system("osascript -e 'tell application \"Finder\"\nactivate\nopen folder POSIX file \"" + path + "\"\nselect item \"" + filename + "\" of window 1\nend tell'")
	else:
		# ... something ...
		os.system()
	
def getTempDir():
	if isWindows():
		return os.environ["TMP"]
	elif isMac():
		return "/private/tmp"
	else:
		return "/tmp"

# Return a relative version of a path
#
# From http://bugs.python.org/file6846/relpath.patchd
def getRelativePath(start, path):
	
	if not path:
		raise ValueError("no path specified")

	if not start:
		raise ValueError("no start path specified")

	start_list = os.path.abspath(start).split(os.path.sep)
	path_list = path.split(os.path.sep)

	if isWindows():
		if start_list[0].lower() != path_list[0].lower():
			return path
		
	# Work out how much of the filepath is shared by start and path.
	for i in range(min(len(start_list), len(path_list))):
		if start_list[i].lower() != path_list[i].lower():
			break
		else:
			i += 1

	rel_list = [os.path.pardir] * (len(start_list)-i-1) + path_list[i:]
	return os.path.join(*rel_list).replace('\\','/') #use only fwd slash fo portability


# Remove all the characters from a string that have special meanings in filenames, and replace with '_'.
def makeSafeFilename(s):
	regexp = re.compile(r'\W') # \W = [^a-zA-Z0-9_]
	return regexp.sub("_", s)


# Do we know where indigo is?
def configured():
	#return True
	return installed()

# Is indigo full version licenced?	
def licenced():
	return False

def getPreviewScenePath(IndigoPaths):
	return os.path.join(IndigoPaths.get('assetsPath'), "preview_material", "matdb_preview.igs")

def getResourcesPath():
	if isLinux():
		return open(os.path.join(os.path.expanduser("~"), ".indigo/location")).read().strip()
	if isMac():
		#return os.path.join(open(os.path.join(os.path.expanduser("~"), ".indigo/location")).read().strip(), "Contents", "Resources")
		from ui import UIControls as uictl
		return os.path.join(uictl.System.IndigoPath.val, "Contents", "Resources")
		
	#if isMac():
	#	return "/Applications/Indigo.app/Contents/Resources"
	elif isWindows():
		import _winreg
		try:
			hKey = _winreg.OpenKey (_winreg.HKEY_CURRENT_USER, r"Software\Glare Technologies\Indigo Renderer")
			value, type = _winreg.QueryValueEx (hKey, "InstallDirectory")
		except:
			value = ""
		return value
	
def installed():
	if isLinux():
		return os.path.exists( os.path.join(os.path.expanduser("~"), ".indigo/location") )
	if isMac():
		return True
	elif isWindows():
		try:
			import _winreg
			hKey = _winreg.OpenKey (_winreg.HKEY_CURRENT_USER, r"Software\\Glare Technologies\\Indigo Renderer")
			value, type = _winreg.QueryValueEx (hKey, "InstallDirectory")
			return True
		except:
			return False
	
def getGuiPath():
	if isLinux():
		return os.path.join(getResourcesPath(), "indigo")
	if isMac():
		return os.path.join(getResourcesPath(), "..", "MacOS", "Indigo")
	elif isWindows():
		return os.path.join(getResourcesPath(), "indigo.exe")

# def forkProcess(command):
# 	fp = open("/tmp/blendigo.sh", "w")
# 	fp.write(command + " &")
# 	fp.close
# 	os.system("chmod +x /tmp/blendigo.sh")
# #	os.system("/tmp/blendigo.sh")
# 	os.spawnl(os.P_NOWAIT, "/tmp/blendigo.sh", "")

def launchAnimation(sceneFilename, options, threads, outpath):
	if isWindows():
		launchWinAnimation(sceneFilename, options, threads, outpath)
	else:
		print('Launching animation batch job not supported on this platform')

def launchWinAnimation(sceneFilename, options, threads, outpath):
	batchScript = os.path.join(getTempDir(), 'IndigoWrapper.cmd')
	#print batchScript
 	f = open(batchScript, 'w')
 	f.write("@Echo off\n\n")
	f.write("cd /d \"" + getResourcesPath() + "\"\n")
	startF = Blender.Get('staframe')
	endF = Blender.Get('endframe')
	f.write('echo Start rendering Animation\n')
	f.write('echo Start Frame: %d\n'%startF)
	f.write('echo End Frame: %d\n'%endF)
	f.write('echo Total Frames: %d\n'%(endF-startF+1))
	for i in range (startF, endF+1):
		f.write('echo Rendering Frame %d\n'%i)
		if threads > 0:
			f.write("start /min /belownormal /wait indigo.exe \""+Blender.sys.makename(sceneFilename, '-%05d.igs'%i)+"\" "+options+" -t "+str(threads)+" -o \""+Blender.sys.makename(sceneFilename,'-%05d.png'%i)+"\"\n")
		else:
			f.write("start /min /belownormal /wait indigo.exe \""+Blender.sys.makename(sceneFilename, '-%05d.igs'%i)+"\" "+options+" -o \""+Blender.sys.makename(sceneFilename,'-%05d.png'%i)+"\"\n")
	f.close()
	os.startfile('"%s"' % batchScript)

def launchGui(sceneFilename, options, prio=1):
	##options += " --thread_priority belownormal"
	gpath = getGuiPath()
	if not os.path.exists(gpath):
		Blender.Draw.PupMenu('Could not find Indigo executable. See console for details.')
		print('Could not find Indigo executable. I looked for %s'%gpath)
		return False
	
	if isMac():
		#print("Launching")
		os.system('/usr/bin/open -a "' + getGuiPath() + '" "' + sceneFilename + '"')
	elif isWindows():
		launchGuiWindows(sceneFilename,options,prio)
	elif isLinux():
		launchGuiLinux(sceneFilename, options)
	
	return True
		
def launchGuiWindows(sceneFilename, options, prio):
	batchScript = os.path.join(getTempDir(), 'IndigoWrapper.cmd')
	#print batchScript
 	f = open(batchScript, 'w')
	f.write('cd /d "' + getResourcesPath() + '"\n')
	f.write("start /b")
	if prio:
		options+=" --thread_priority belownormal"
		f.write(" /BELOWNORMAL")
	f.write(' indigo.exe "' + sceneFilename + '" ' + options +"\n")
	f.close()
 	os.startfile('"%s"' % batchScript)

def launchGuiLinux(sceneFile, options):
	wrapperFile = os.path.join(getTempDir(), "IndigoWrapper.sh")
	
	f = open(wrapperFile, 'w')
	f.write("#!/bin/sh\n") 
	f.write('cd "' + getResourcesPath() + '"\n')
	f.write('./indigo "%s" %s' %(sceneFile, options))
	f.close()
	os.system('chmod +x "' + wrapperFile + '"')
	# call external shell script to start Indigo || linux
	cmd= '"' + wrapperFile + '" &'
	#print cmd
	os.system(cmd)
	
def menuStringFromList(title, lst):
	'''
	Given a title and a list of items,
	make a menu string for Draw.Menu
	'''
	return '%s%%t|'%title + '|'.join(['%s%%x%i'%(p, i) for i,p in enumerate(lst)])
