#!BPY
"""Registration info for Blender menus:
Name: 'Render with Indigo...'
Blender: 248
Group: 'Render'
Tooltip: 'Material editor and exporter for Indigo v2.4 Stable'
"""
#
# ***** BEGIN GPL LICENSE BLOCK *****
#
# --------------------------------------------------------------------------
# BLENDIGO
# --------------------------------------------------------------------------
#
# Authors:
# * Indigo exporter - Nick Chapman, U3dreal, Zuegs, Ewout Fernhout, Leope,
#					  psor, Wedge, SmartDen, BIGFAN(BF), DaveC, Ben Nolan,
#					  Doug Hammond
#
# Attribution:
# * Portions of the GUI layout and scrollbar code were lifted from
#   LuxRender's LuxBlend exporter.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# ***** END GPL LICENCE BLOCK *****
# --------------------------------------------------------------------------

blendigoVersion = "2.4 Stable, Update 2"

#===============================================================================
# Startup Banner
#===============================================================================
print ""
print "----------------------"
print "BLENDIGO v" + blendigoVersion
print "----------------------"

#===============================================================================
# Import modules
#===============================================================================
import math, os, sys, time, copy, array, re, string
import Blender

# Import the blendigo modules
try:
	from blendigopkg.indigo        import Material             as IndigoMaterial
	from blendigopkg.indigo        import Types                as IndigoTypes
	from blendigopkg.indigo        import Events               as IndigoEvents
	from blendigopkg.indigo        import PathsBase
	from blendigopkg.indigo        import convertMaterial
	from blendigopkg.ui            import Layout               as guiLayout
	from blendigopkg.ui            import UIControls           as IndigoUIControls
	from blendigopkg.ui            import exportProgressBar
	from blendigopkg.ui            import scrollbar
	from blendigopkg.lampspectrum  import lampspectrum
	from blendigopkg               import utils
	from blendigopkg               import indigo_mesh
	
	# this is slightly strangely coded, in that we want matdblib imported
	# on every platform except win64
	try:
		MATDBLIB_AVAILABLE = False
		if sys.platform == 'win32' and sys.version.index('AMD64') > 1:
			pass
			# index() throws an exception if string not found
	except:
		from blendigopkg import matdblib
		MATDBLIB_AVAILABLE = True
	
except Exception, err:
	Blender.Draw.PupMenu("Broken installation of Blendigo, please see console for details%t|Abort")
	print('Initialisation error: %s\n' % str(err))

PY25 = sys.version_info[0] == 2 and sys.version_info[1] == 5

class IndigoPaths(PathsBase):
	_p = {
		'assetsPath':     (os.path.join(Blender.Get(u"scriptsdir"), u"blendigopkg", u"assets"), False, True),
		'matdbDownloads': (os.path.join(Blender.Get(u"scriptsdir"), u"blendigopkg", u"matdb"),  True,  False),
		'matdbPreviews':  (os.path.join(Blender.Get(u"scriptsdir"), u"blendigopkg", u"matdb", u"previews"),  True,  False),
	}

if os.getenv('INDIGODEV') is not None:
	print "Running in Developer Mode"
	if PY25: print("Running under py 2.5; using back-compat routines")
	if sys.platform == 'win32':
		class IndigoPaths(PathsBase):
			_p = {
				'assetsPath':     (os.path.join(Blender.Get(u"uscriptsdir"), u"blendigopkg", u"assets"), False, True),
				'matdbDownloads': ('d:/render/external-mat',  True,  False),
				'matdbPreviews':  ('d:/render/external-mat/previews',  True,  False),
			}
	else:
		class IndigoPaths(PathsBase):
			_p = {
				'assetsPath':     (os.path.join(Blender.Get(u"uscriptsdir"), u"blendigopkg", u"assets"), False, True),
				'matdbDownloads': (os.path.expanduser('~/render/external-mat'),  True,  False),
				'matdbPreviews':  (os.path.expanduser('~/render/external-mat/previews'),  True,  False),
			}

# Check required file paths
IndigoPaths = IndigoPaths()

PlatformInformation = utils.PlatformInformation()

scrollbar = scrollbar()

#===============================================================================
# CODE BEGINS
#===============================================================================
def drawImage(img, x, y, dx, dy):
	"""
	img =  object loaded with Blender.Image.Load() function
	x,y =   location in the script window from the left bottom corner
	dx,dy = scaling the image size dx for width and dy for height
	"""
	
	fx = float(x)
	fy = float(y)
	
	dx = float(dx)
	dy = float(dy)
	
	sx, sy = img.getSize()
	cx = fx + float(sx)*dx
	cy = fy + float(sy)*dy
	
	Blender.BGL.glEnable(Blender.BGL.GL_TEXTURE_2D)
	Blender.BGL.glColor4f(1,1,1,0)
	
	Blender.BGL.glBegin(Blender.BGL.GL_QUADS)
	Blender.BGL.glBindTexture(Blender.BGL.GL_TEXTURE_2D, img.getBindCode())
	Blender.BGL.glTexCoord2f(0.0,0.0)
	Blender.BGL.glVertex3f(fx, fy, 0.0)
	Blender.BGL.glTexCoord2f(1.0,0.0)
	Blender.BGL.glVertex3f(cx, fy, 0.0)
	Blender.BGL.glTexCoord2f(1.0,1.0)
	Blender.BGL.glVertex3f(cx, cy, 0.0)
	Blender.BGL.glTexCoord2f(0.0,1.0)
	Blender.BGL.glVertex3f(fx, cy, 0.0 )
	Blender.BGL.glEnd()
	
	Blender.BGL.glDisable(Blender.BGL.GL_TEXTURE_2D)

def setCurrentMaterial(material):
	if material not in IndigoGlobals.material_cache.keys():
		IndigoGlobals.material_cache[material] = IndigoMaterial(material)
	
	IndigoUIControls.Materials.currentMaterial = IndigoGlobals.material_cache[material]


class IndigoGlobals:
	# Load the indigo logo..
	indigoLogo = Blender.Image.Load(os.path.join(IndigoPaths.get('assetsPath'), "indigo_logo.png"))
	
	mat_preview_cache = {}
	
	# 'Globals'
	activeObject = None
	activeMaterial = None
	activeObjectMat0 = None
	
	ShaderList = []
	
	linux = 0
	
	selectedSection = "camera"
	
	material_cache = {}
	
	layer_names = ['Environment']				# named light layers.
	layer_name_counts = { 'Environment': 0 }	# number of lights using each layer
	
	###  **** OBSOLETE **** GUI Globals
		#FrameUp = Blender.Draw.Create(20)
		#LMP = Blender.Draw.Create(0.4)
		#MaxChange = Blender.Draw.Create(0.02)
		#MaxDepth = Blender.Draw.Create(1000)
		#UseGUI = Blender.Draw.Create(1)
		#NetworkSL = Blender.Draw.Create(0)
		## Separate materials
		# SepMatFile = Blender.Draw.Create(0)
		## Separate meshs
		#SepMeshFile = Blender.Draw.Create(0)
		# Half fstops
		#HStop = Blender.Draw.Create(0)
		## Overall color gain
		#DiffuseGain = Blender.Draw.Create(1.00)
		#SpecularGain = Blender.Draw.Create(1.00)
		# Pathtrace/MLT Hybrid
		#Hybrid = Blender.Draw.Create(0)
		#BIH = Blender.Draw.Create(500000)
		#RayNudgeMant = Blender.Draw.Create(1)
		#RayNudgeExp = Blender.Draw.Create(-5)
		#Override = Blender.Draw.Create(0)
		# Cache KD-trees to disk
		#CacheTree = Blender.Draw.Create(1)
	
	# text color fix
	textcol = [0, 0, 0]
	
	Screen = 0
	lastScreen = 0

### ================================================================
### readWriteMaterial(material, write)
###		reads or writes material-data from or to blender material property
### ----------------------------------------------------------------
def readWriteValue(dict, name, draw, write, default):
	if write == 0:
		try:
			try:
				draw.val = dict[name]
			except KeyError:
				draw.val = default
		except ValueError:
			print "ValueError: on property \"%s\""%name
	else:
		dict[name] = draw.val

def readWriteMaterial(m, write=0):
	if m != None:
		
		try:
			d = m.properties['indigo'].convert_to_pyobject()
		except KeyError:
			d = {}
			if write == 0:
				convertMaterial(m)
				d = m.properties['indigo'].convert_to_pyobject()
		
		if write == 0:
			try:
		 		IndigoUIControls.Materials.matType = d['type']
			except KeyError:
				IndigoUIControls.Materials.matType = 0
		else:
			d['type'] = IndigoUIControls.Materials.matType
		
		readWriteValue(d, 'emitter', IndigoUIControls.Materials.matEmitt, write, 0)
		readWriteValue(d, 'emitter.type', IndigoUIControls.Materials.matEmittType, write, IndigoTypes.Spectrum._('RGB'))
		readWriteValue(d, 'emitter.ies', IndigoUIControls.Materials.matEmittIES, write, 0)
		readWriteValue(d, 'emitter.ies_path', IndigoUIControls.Materials.matEmittIESPath, write, '')
		readWriteValue(d, 'emitter.temperature', IndigoUIControls.Materials.matEmittTemp, write, 5000)
		readWriteValue(d, 'emitter.scale_value', IndigoUIControls.Materials.matEmittScaleValue, write, 1500)
		readWriteValue(d, 'emitter.scale_measure', IndigoUIControls.Materials.matEmittScaleMeasure, write, IndigoTypes.EmissionMeasure._('FLUX'))
		readWriteValue(d, 'emitter.gain', IndigoUIControls.Materials.matEmittGain, write, 1)
		readWriteValue(d, 'emitter.mult', IndigoUIControls.Materials.matEmittMult, write, 0)
		readWriteValue(d, 'rgb.r_e', IndigoUIControls.Materials.matEmittRgbR, write, 1)
		readWriteValue(d, 'rgb.g_e', IndigoUIControls.Materials.matEmittRgbG, write, 1)
		readWriteValue(d, 'rgb.b_e', IndigoUIControls.Materials.matEmittRgbB, write, 1)
		readWriteValue(d, 'emitter.layer', IndigoUIControls.Materials.matEmittLayer, write, 'default')
		readWriteValue(d, 'emitter.lspreset', IndigoUIControls.Materials.matEmittLSPreset, write, '')
		
		readWriteValue(d, 'diffuse.transmitter', IndigoUIControls.Materials.matDiffTrans, write, 0)
		readWriteValue(d, 'toggle_phong', IndigoUIControls.Materials.matTogglePhong, write, 0)
		readWriteValue(d, 'rgb.r', IndigoUIControls.Materials.matRgbR, write, 0.8)
		readWriteValue(d, 'rgb.g', IndigoUIControls.Materials.matRgbG, write, 0.8)
		readWriteValue(d, 'rgb.b', IndigoUIControls.Materials.matRgbB, write, 0.8)
		readWriteValue(d, 'rgb.r_s', IndigoUIControls.Materials.matRgbR_s, write, 0.8)
		readWriteValue(d, 'rgb.g_s', IndigoUIControls.Materials.matRgbG_s, write, 0.8)
		readWriteValue(d, 'rgb.b_s', IndigoUIControls.Materials.matRgbB_s, write, 0.8)
		readWriteValue(d, 'rgb.gain', IndigoUIControls.Materials.matRgbGain, write, 1.0)
		readWriteValue(d, 'ior', IndigoUIControls.Materials.matIOR, write, 1.0)
		readWriteValue(d, 'exponent', IndigoUIControls.Materials.matExponent, write, 100)
		readWriteValue(d, 'nk_data', IndigoUIControls.Materials.matNKData, write, '')
		readWriteValue(d, 'sigma', IndigoUIControls.Materials.matSigma, write, 0.2)
		readWriteValue(d, 'transparent', IndigoUIControls.Materials.matTransparent, write, 0)
		readWriteValue(d, 'specular.type', IndigoUIControls.Materials.matSpecularType, write, 1)
		readWriteValue(d, 'precedence', IndigoUIControls.Materials.matPrecedence, write, 10)
		readWriteValue(d, 'medium.type', IndigoUIControls.Materials.matMediumType, write, 1)
		readWriteValue(d, 'hemoglobin', IndigoUIControls.Materials.matHemoglobin, write, 0.001)
		readWriteValue(d, 'turbidity', IndigoUIControls.Materials.matTurbidity, write, 2.2)
		readWriteValue(d, 'melanin', IndigoUIControls.Materials.matMelanin, write, 0.15)
		readWriteValue(d, 'eumelanin', IndigoUIControls.Materials.matEumelanin, write, 0.001)
		readWriteValue(d, 'cauchy_b_coeff', IndigoUIControls.Materials.matCauchyBCoeff, write, 0.0)
		readWriteValue(d, 'absorption.type', IndigoUIControls.Materials.matAbsorptionType, write, 1)
		readWriteValue(d, 'abs.r', IndigoUIControls.Materials.matAbsR, write, 0.0)
		readWriteValue(d, 'abs.g', IndigoUIControls.Materials.matAbsG, write, 0.0)
		readWriteValue(d, 'abs.b', IndigoUIControls.Materials.matAbsB, write, 0.0)
		readWriteValue(d, 'abs.gain', IndigoUIControls.Materials.matAbsGain, write, 1)
		readWriteValue(d, 'sss', IndigoUIControls.Materials.matSss, write, 0)
		readWriteValue(d, 'sss.type', IndigoUIControls.Materials.matSssType, write, 2)
		readWriteValue(d, 'sss.r', IndigoUIControls.Materials.matSssR, write, 1)
		readWriteValue(d, 'sss.g', IndigoUIControls.Materials.matSssG, write, 1)
		readWriteValue(d, 'sss.b', IndigoUIControls.Materials.matSssB, write, 1)
		readWriteValue(d, 'sss.gain', IndigoUIControls.Materials.matSssGain, write, 1.0)
		readWriteValue(d, 'sss.uniform', IndigoUIControls.Materials.matSssUniform, write, 1.0)
		readWriteValue(d, 'sss.uniformExp', IndigoUIControls.Materials.matSssUniformExp, write, 0.0)
		readWriteValue(d, 'sss.phasespec', IndigoUIControls.Materials.matSssPhaseSpec,write,1)
		readWriteValue(d, 'sss.phasespec_uni', IndigoUIControls.Materials.matSssPhaseSpecUni,write,0.5)
		readWriteValue(d, 'sss.phasespec_r', IndigoUIControls.Materials.matSssPhaseSpecR,write,0.5)
		readWriteValue(d, 'sss.phasespec_g', IndigoUIControls.Materials.matSssPhaseSpecG,write,0.5)
		readWriteValue(d, 'sss.phasespec_b', IndigoUIControls.Materials.matSssPhaseSpecB,write,0.5)
		readWriteValue(d, 'sss.phasetype' , IndigoUIControls.Materials.matSssPhaseType, write, 1)
		
		readWriteValue(d, 'albedo.uv_set', IndigoUIControls.Materials.matAlbedoUVset, write, 'default')
		readWriteValue(d, 'albedo.path', IndigoUIControls.Materials.matAlbedoPath, write, '')
		readWriteValue(d, 'albedo.isl',IndigoUIControls.Materials.matAlbedoISL, write, 0)
		readWriteValue(d, 'albedo.shader', IndigoUIControls.Materials.matAlbedoShader, write, -1)
		readWriteValue(d, 'albedo.exponent', IndigoUIControls.Materials.matAlbedoExponent, write, 2.2)
		readWriteValue(d, 'albedo.a', IndigoUIControls.Materials.matAlbedoA, write, 0.0)
		readWriteValue(d, 'albedo.b', IndigoUIControls.Materials.matAlbedoB, write, 1.0)
		readWriteValue(d, 'albedo.c', IndigoUIControls.Materials.matAlbedoC, write, 0.0)
		
		readWriteValue(d, 'bump.isl', IndigoUIControls.Materials.matBumpISL,write, 0)
		readWriteValue(d, 'bump.shader', IndigoUIControls.Materials.matBumpShader,write, -1)
		readWriteValue(d, 'bump.uv_set', IndigoUIControls.Materials.matBumpUVset, write, 'default')
		readWriteValue(d, 'bump.path', IndigoUIControls.Materials.matBumpPath, write, '')
		readWriteValue(d, 'bump.exponent', IndigoUIControls.Materials.matBumpExponent, write, 2.2)
		readWriteValue(d, 'bump.a', IndigoUIControls.Materials.matBumpA, write, 0.0)
		readWriteValue(d, 'bump.b', IndigoUIControls.Materials.matBumpB, write, 1.0)
		readWriteValue(d, 'bump.c', IndigoUIControls.Materials.matBumpC, write, 0.0)
		
		readWriteValue(d, 'exponent.isl', IndigoUIControls.Materials.matExponentISL,write, 0)
		readWriteValue(d, 'exponent.shader', IndigoUIControls.Materials.matExponentShader,write, -1)
		readWriteValue(d, 'exponent.uv_set', IndigoUIControls.Materials.matExponentUVset, write, 'default')
		readWriteValue(d, 'exponent.path', IndigoUIControls.Materials.matExponentPath, write, '')
		readWriteValue(d, 'exponent.exponent', IndigoUIControls.Materials.matExponentExponent, write, 2.2)
		readWriteValue(d, 'exponent.a', IndigoUIControls.Materials.matExponentA, write, 0.0)
		readWriteValue(d, 'exponent.b', IndigoUIControls.Materials.matExponentB, write, 10000)
		readWriteValue(d, 'exponent.c', IndigoUIControls.Materials.matExponentC, write, 1.0)
		
		readWriteValue(d, 'blend.a_name', IndigoUIControls.Materials.matBlendMatA, write, '')
		readWriteValue(d, 'blend.b_name', IndigoUIControls.Materials.matBlendMatB, write, '')
		readWriteValue(d, 'blend.factor', IndigoUIControls.Materials.matBlendFactor, write, 0.5)
		readWriteValue(d, 'blend.uv_set', IndigoUIControls.Materials.matBlendUVset, write, 'default')
		readWriteValue(d, 'blend.path', IndigoUIControls.Materials.matBlendPath, write, '')
		readWriteValue(d, 'blend.exponent', IndigoUIControls.Materials.matBlendExponent, write, 1.0)
		readWriteValue(d, 'blend.a', IndigoUIControls.Materials.matBlendA, write, 0.0)
		readWriteValue(d, 'blend.b', IndigoUIControls.Materials.matBlendB, write, 1.0)
		readWriteValue(d, 'blend.c', IndigoUIControls.Materials.matBlendC, write, 0.0)
		
		readWriteValue(d, 'include.matdb', IndigoUIControls.Materials.matIncludeUseDB, write, 0)
		readWriteValue(d, 'include', IndigoUIControls.Materials.matInclude, write, '')
		
		readWriteValue(d, 'displace.isl', IndigoUIControls.Materials.matDisplaceISL,write, 0)
		readWriteValue(d, 'displace.shader', IndigoUIControls.Materials.matDisplaceShader,write, -1)
		readWriteValue(d, 'displace.path', IndigoUIControls.Materials.matDisplacePath,write,'')
		readWriteValue(d, 'displace.exp', IndigoUIControls.Materials.matDisplaceExp,write,2.2)
		readWriteValue(d, 'displace.uv_set', IndigoUIControls.Materials.matDisplaceUVSet,write,'default')
		readWriteValue(d, 'displace.a', IndigoUIControls.Materials.matDisplaceA,write,0.0)
		readWriteValue(d, 'displace.b', IndigoUIControls.Materials.matDisplaceB,write,0.01)
		readWriteValue(d, 'displace.c', IndigoUIControls.Materials.matDisplaceC,write,0)
		
		# Absorption Layer Transmittance
		readWriteValue(d, 'alt.isl', IndigoUIControls.Materials.matALTISL,write, 0)
		readWriteValue(d, 'alt.shader', IndigoUIControls.Materials.matALTShader,write, -1)
		readWriteValue(d, 'alt.path', IndigoUIControls.Materials.matALTPath, write, '')
		readWriteValue(d, 'alt.exp', IndigoUIControls.Materials.matALTExp,write,2.2)
		readWriteValue(d, 'alt.uv_set', IndigoUIControls.Materials.matALTUVSet,write,'default')
		readWriteValue(d, 'alt.a', IndigoUIControls.Materials.matALTA,write,0.0)
		readWriteValue(d, 'alt.b', IndigoUIControls.Materials.matALTB,write,0.01)
		readWriteValue(d, 'alt.c', IndigoUIControls.Materials.matALTC,write,0)
		
		if write != 0:
			m.properties['indigo'] = d

def check_uvtext(str):
	if str.lower() == 'uvtex':
		return 'default'
	else:
		return str 

### ================================================================
### exportMaterial(
###		material		-> string				name of material
###		file			-> open file object		output file to write to
###		exportedmatlist	-> list					list of already exported mats
###		newmatname		-> string				??
###		uvset			-> string				??
### )
### returns:
###		includelist		-> list
###		matname			-> string
### ----------------------------------------------------------------
def exportMaterial(mat, file, exportedmatlist, newmatname='', uvset=''):
	includelist=[]
	
	texture_index=0
	if mat != None:
		if 'indigo' not in mat.properties.keys():
			convertMaterial(mat)
		
		d = mat.properties['indigo'].convert_to_pyobject()
		
		matname = mat.getName()
		if newmatname != '':
			matname = newmatname
		exportedmatlist.append(matname)
		m = []
		if not IndigoUIControls.Renderer.ClayRender.val:
			matType = readValue(d, 'type', 0)
		else:
			matType = 0
		matMediumType = readValue(d, 'medium.type', 1)
		if not IndigoTypes.Materials.inT(matType, ['LIGHT', 'INCLUDE', 'EXITPORTAL']):
			# specular + media
			if IndigoTypes.Materials.isT(matType, 'SPECULAR'):
				if IndigoTypes.Medium.isT(matMediumType, 'BASIC'):
					file.write("\t<medium>\n")
					file.write("\t\t<name>%s</name>\n"%(matname))
					file.write("\t\t<precedence>%g</precedence>\n"%(readValue(d, 'precedence', 1.0)))
					file.write("\t\t<basic>\n")
					file.write("\t\t\t<ior>%g</ior>\n"%(readValue(d, 'ior', 1.0)))
					file.write("\t\t\t<cauchy_b_coeff>%g</cauchy_b_coeff>\n"%(readValue(d, 'cauchy_b_coeff', 1.0)))
					file.write("\t\t\t<absorption_coefficient_spectrum>\n")
					if readValue(d, 'absorption.type', 1) == IndigoTypes.Spectrum._('RGB'):
						gain = readValue(d, 'abs.gain', 1.0)
						file.write("\t\t\t\t<rgb>\n")
						file.write("\t\t\t\t\t<rgb>%g %g %g</rgb>\n\t\t\t\t\t<gamma>2.2</gamma>\n"%((readValue(d, 'abs.r', 0.0))*gain, (readValue(d, 'abs.g', 0.0))*gain, (readValue(d, 'abs.b', 0.0))*gain))
						file.write("\t\t\t\t</rgb>\n")
					elif readValue(d, 'absorption.type', 1) == IndigoTypes.Spectrum._('UNIFORM'):
						file.write("\t\t\t\t<uniform>\n")
						file.write("\t\t\t\t\t<value>%f</value>\n" % readValue(d, 'abs.gain', 1.0))
						file.write("\t\t\t\t</uniform>\n")
					file.write("\t\t\t</absorption_coefficient_spectrum>\n")
					if (readValue(d, 'sss', 0) != 0):
						file.write("\t\t\t<subsurface_scattering>\n")
						file.write("\t\t\t\t<scattering_coefficient_spectrum>\n")
						if readValue(d, 'sss.type', 1) == IndigoTypes.Spectrum._('RGB'):
							gain = readValue(d, 'sss.gain', 1.0)
							file.write("\t\t\t\t\t<rgb>\n")
							file.write("\t\t\t\t\t\t<rgb>%g %g %g</rgb>\n"%(readValue(d, 'sss.r', 0.0)*gain, readValue(d, 'sss.g', 0.0)*gain, readValue(d, 'sss.b', 0.0)*gain))
							file.write("\t\t\t\t\t\t<gamma>2.2</gamma>\n")
							file.write("\t\t\t\t\t</rgb>\n")
						if readValue(d,'sss.type',1) == IndigoTypes.Spectrum._('UNIFORM'):
							file.write("\t\t\t\t\t<uniform>\n")
							file.write("\t\t\t\t\t\t<value>%g</value>\n"%(readValue(d,'sss.uniform',0.0)*(10**readValue(d, 'sss.uniformExp',0.0))))
							file.write("\t\t\t\t\t</uniform>\n")
						file.write("\t\t\t\t</scattering_coefficient_spectrum>\n")
						if readValue(d, 'sss.phasetype', 1) == IndigoTypes.Phase._('UNIFORM'):
							file.write("\t\t\t\t<phase_function>\n")
							file.write("\t\t\t\t\t<uniform/>\n")
							file.write("\t\t\t\t</phase_function>\n")
						if readValue(d, 'sss.phasetype', 1) == IndigoTypes.Phase._('HG'):
							file.write("\t\t\t\t<phase_function>\n")
							file.write("\t\t\t\t\t<henyey_greenstein>\n")
							file.write("\t\t\t\t\t\t<g_spectrum>\n")
							if readValue(d,'sss.phasespec',1) == IndigoTypes.Spectrum._('UNIFORM'):
								file.write("\t\t\t\t\t\t\t<uniform>\n")
								file.write("\t\t\t\t\t\t\t\t<value>%g</value>\n"%(readValue(d, 'sss.phasespec_uni', 0.5)))
								file.write("\t\t\t\t\t\t\t</uniform>\n")
							elif readValue(d,'sss.phasespec',1) == IndigoTypes.Spectrum._('RGB'):
								file.write("\t\t\t\t\t\t\t<rgb>\n")
								file.write("\t\t\t\t\t\t\t\t<rgb>%g %g %g</rgb>\n"%(readValue(d,'sss.phasespec_r',0.5),readValue(d,'sss.phasespec_g',0.5),readValue(d,'sss.phasespec_b',0.5)))
								file.write("\t\t\t\t\t\t\t\t<gamma>2.2</gamma>\n")
								file.write("\t\t\t\t\t\t\t</rgb>\n")
							file.write("\t\t\t\t\t\t</g_spectrum>\n")
							file.write("\t\t\t\t\t</henyey_greenstein>\n")
							file.write("\t\t\t\t</phase_function>\n")
						file.write("\t\t\t</subsurface_scattering>\n")
					file.write("\t\t</basic>\n")
					file.write("\t</medium>\n")
				if IndigoTypes.Medium.isT(matMediumType, 'DERMIS'):
					file.write("\t<medium\n>")
					file.write("\t\t<name>%s</name>\n"%(matname))
					file.write("\t\t<precedence>%g</precedence>\n"%(readValue(d, 'precedence', 1.0)))
					file.write("\t\t<dermis>\n")
					file.write("\t\t\t<hemoglobin_fraction>%g</hemoglobin_fraction>\n"%(readValue(d, 'hemoglobin', 0.15)))
					file.write("\t\t</dermis>\n")
					file.write("\t</medium>\n")
				if IndigoTypes.Medium.isT(matMediumType, 'EPIDERMIS'):
					file.write("\t<medium>\n")
					file.write("\t\t<name>%s</name>\n"%(matname))
					file.write("\t\t<precedence>%g</precedence>"%(readValue(d, 'precedence', 1.0)))
					file.write("\t\t<epidermis>")
					file.write("\t\t\t<melanin_fraction>%g</melanin_fraction>"%(readValue(d, 'melanin', 0.001)))
					file.write("\t\t\t<melanin_type_blend>%g</melanin_type_blend>"%(readValue(d, 'eumelanin', 0.001)))
					file.write("\t\t</epidermis>")
					file.write("\t</medium>")
				if IndigoTypes.Medium.isT(matMediumType, 'ATMOSPHERE'):
					file.write("\t<medium>")
					file.write("\t\t<name>%s</name>"%(matname))
					file.write("\t\t<precedence>%g</precedence>"%(readValue(d, 'precedence', 1.0)))
					file.write("\t\t\t<atmosphere>")
					file.write("\t\t\t\t<turbidity>%s</turbidity>"%(readValue(d,'turbidity',2.2)))
					file.write("\t\t\t</atmosphere>")
					file.write("\t</medium>")
			
			#check if blended material
			if IndigoTypes.Materials.isT(matType, 'BLEND'):
				matAName = readValue(d, 'blend.a_name', '')
				matBName = readValue(d, 'blend.b_name', '')
				# check if linked materials have already been exported -
				# if not do it now and add it infront of this material.
				il = []
				if matAName not in exportedmatlist:
					mm = Blender.Material.Get(matAName)
					exportedmatlist.append(matAName)
					inc, pigmname = exportMaterial(mm, file, exportedmatlist,'',uvset)
					il += inc
					matAName = pigmname
				if matBName not in exportedmatlist:
					mm = Blender.Material.Get(matBName)
					exportedmatlist.append(matBName)
					inc, pigmname = exportMaterial(mm, file, exportedmatlist,'',uvset)
					il += inc
					matBName = pigmname
				
				writeIncludes(file, il)
			
			file.write("\t<material>\n")
			file.write("\t\t<name>%s</name>\n"%(matname))
			
			if IndigoTypes.Materials.isT(matType, 'DIFFUSE'):
				transparent=readValue(d,'diffuse.transmitter',0)
				if not transparent or IndigoUIControls.Renderer.ClayRender.val:
					file.write("\t\t<diffuse>\n")
				else:
					file.write("\t\t<diffuse_transmitter>\n")
			elif IndigoTypes.Materials.isT(matType, 'PHONG'):
				file.write("\t\t<phong>\n")
				#export exponent only if no exponent map
				if readValue(d,'exponent.path','')  == '':
					file.write("\t\t\t<exponent>\n\t\t\t\t<constant>%g</constant>\n\t\t\t</exponent>\n"%(readValue(d, 'exponent', 1000.0)))
				file.write("\t\t\t<ior>%g</ior>\n"%(readValue(d, 'ior', 1.0)))
				if not readValue(d,'toggle_phong',0) and (readValue(d, 'nk_data', '').strip() != ''):
						file.write("\t\t\t<nk_data>nkdata%s%s</nk_data>\n"%(os.sep,readValue(d, 'nk_data', '')))
			elif IndigoTypes.Materials.isT(matType, 'SPECULAR'):
				speculartype = readValue(d, 'specular.type', 1)
				if speculartype == IndigoTypes.Specular._('SPECULAR'):
					file.write("\t\t<specular>\n")
				if speculartype == IndigoTypes.Specular._('GLOSSY'):
					file.write("\t\t<glossy_transparent>\n")
					file.write("\t\t\t<exponent>%g</exponent>\n"%(readValue(d, 'exponent', 1000.0)))
				else:
					if (readValue(d, 'transparent', 0) != 0):
						file.write("\t\t\t<transparent>true</transparent>\n")
					else:
						file.write("\t\t\t<transparent>false</transparent>\n")
				file.write("\t\t\t<internal_medium_name>%s</internal_medium_name>\n"%(matname))
			elif IndigoTypes.Materials.isT(matType, 'BLEND'):
				# don't re-read blend mat names, they might get re-written above
				# if one or both of them are external mats
				#matAName = readValue(d, 'blend.a_name', '')
				#matBName = readValue(d, 'blend.b_name', '')
				
				file.write("\t\t<blend>\n")
				file.write("\t\t\t<a_name>%s</a_name>\n"%(matAName))
				file.write("\t\t\t<b_name>%s</b_name>\n"%(matBName))
				file.write("\t\t\t<step_blend>false</step_blend>\n")
			elif IndigoTypes.Materials.isT(matType, 'ORENNAYAR'):
				file.write("\t\t<oren_nayar>\n")
				file.write("\t\t\t<sigma><constant>%f</constant></sigma>\n"%readValue(d,'sigma',0.2))
			
			#---TEXTURES EXPORT---
			albedo=0
			exponent=0
			bump=0
			displace=0
			blend=0
			alt=0
			if not IndigoUIControls.Renderer.ClayRender.val and not IndigoTypes.Materials.isT(matType, 'BLEND') and readValue(d,'albedo.path','') != '' :
				albedo=1
				file.write("\t\t\t<texture>\n")
				currpath=Blender.sys.dirname(Blender.Get('filename'))
				target=readValue(d, 'albedo.path', '')
				if target[0:2] == "\\\\" or target[0:2] == "//":
					target=currpath+os.sep+target[2:]
				#file.write("\t\t\t\t<path>%s</path>\n"%(utils.getRelativePath(file.name, readValue(d, 'albedo.path', ''))))
				file.write("\t\t\t\t<path>%s</path>\n"%(utils.getRelativePath(file.name, target)))
				if uvset != '':
					file.write("\t\t\t\t<uv_set>%s</uv_set>\n" % check_uvtext(uvset))
				else:
					#Check if any uv sets exists
					file.write("\t\t\t\t<uv_set>%s</uv_set>\n" % check_uvtext(readValue(d, 'albedo.uv_set', '')))
				file.write("\t\t\t\t<exponent>%f</exponent>\n"%(readValue(d, 'albedo.exponent', 1.0)))
				file.write("\t\t\t\t<a>%g</a>\n"%(readValue(d, 'albedo.a', 0.0)))
				file.write("\t\t\t\t<b>%g</b>\n"%(readValue(d, 'albedo.b', 1.0)))
				file.write("\t\t\t\t<c>%g</c>\n"%(readValue(d, 'albedo.c', 0.0)))
				file.write("\t\t\t</texture>\n")
			
			if not IndigoUIControls.Renderer.ClayRender.val and IndigoTypes.Materials.isT(matType, 'PHONG') and readValue(d,'exponent.path','')  != '':
				exponent=1
				file.write("\t\t\t<texture>\n");
				currpath=Blender.sys.dirname(Blender.Get('filename'))
				target=readValue(d, 'exponent.path', '')
				if target[0:2] == "\\\\" or target[0:2] == "//":
					target=currpath+os.sep+target[2:]
				file.write("\t\t\t\t<path>%s</path>\n"%(utils.getRelativePath(file.name, target)))
				if uvset != '':
					file.write("\t\t\t\t<uv_set>%s</uv_set>\n"%check_uvtext(uvset))
				else:
					file.write("\t\t\t\t<uv_set>%s</uv_set>\n" % check_uvtext(readValue(d, 'exponent.uv_set', '')))
				file.write("\t\t\t\t<exponent>%f</exponent>\n"%(readValue(d, 'exponent.exponent', 2.2)))
				file.write("\t\t\t\t<a>%g</a>\n"%(readValue(d, 'exponent.a', 0.0)))
				file.write("\t\t\t\t<b>%g</b>\n"%(readValue(d, 'exponent.b', 1.0)))
				file.write("\t\t\t\t<c>%g</c>\n"%(readValue(d, 'exponent.c', 0.0)))
				file.write("\t\t\t</texture>\n");
			
			if not IndigoUIControls.Renderer.ClayRender.val and not IndigoTypes.Materials.isT(matType, 'BLEND') and readValue(d,'bump.path','') != '':
				bump=1
				file.write("\t\t\t<texture>\n")
				currpath=Blender.sys.dirname(Blender.Get('filename'))
				target=readValue(d, 'bump.path', '')
				if target[0:2] == "\\\\" or target[0:2] == "//":
					target=currpath+os.sep+target[2:]
				file.write("\t\t\t\t<path>%s</path>\n"%(utils.getRelativePath(file.name, target)))
				if uvset != '':
					file.write("\t\t\t\t<uv_set>%s</uv_set>\n" % check_uvtext(uvset))
				else:
					file.write("\t\t\t\t<uv_set>%s</uv_set>\n" % check_uvtext(readValue(d, 'bump.uv_set', '')))
				print("BUMP UV SET: %s"%(readValue(d, 'bump.uv_set', '')))
				file.write("\t\t\t\t<exponent>%f</exponent>\n"%(readValue(d, 'bump.exponent', 1.0)))
				file.write("\t\t\t\t<a>%g</a>\n"%(readValue(d, 'bump.a', 0.0)))
				file.write("\t\t\t\t<b>%g</b>\n"%(readValue(d, 'bump.b', 1.0)/1000.0 )) # convert mm to m
				file.write("\t\t\t\t<c>%g</c>\n"%(readValue(d, 'bump.c', 0.0)/1000.0 ))
				file.write("\t\t\t</texture>\n")
			
			if not IndigoUIControls.Renderer.ClayRender.val and not IndigoTypes.Materials.isT(matType, 'BLEND') and readValue(d, 'displace.path', '') != '':
				displace=1
				file.write("\t\t\t<texture>\n")
				currpath=Blender.sys.dirname(Blender.Get('filename'))
				target=readValue(d, 'displace.path', '')
				if target[0:2] == "\\\\" or target[0:2] == "//":
					target=currpath+os.sep+target[2:]
				file.write("\t\t\t\t<path>%s</path>\n"%(utils.getRelativePath(file.name, target)))
				file.write("\t\t\t\t<exponent>%f</exponent>\n"%(readValue(d, 'displace.exp', 2.2)))
				file.write("\t\t\t\t<uv_set>%s</uv_set>\n" % check_uvtext(readValue(d, 'displace.uv_set', 'uv')))
				file.write("\t\t\t\t<a>%g</a>\n"%(readValue(d, 'displace.a', 0.0)))
				file.write("\t\t\t\t<b>%g</b>\n"%(readValue(d, 'displace.b', 1.0)))
				file.write("\t\t\t\t<c>%g</c>\n"%(readValue(d, 'displace.c', 0.0)))
				file.write("\t\t\t</texture>\n")
			
			if not IndigoUIControls.Renderer.ClayRender.val and IndigoTypes.Materials.isT(matType, 'BLEND') and readValue(d, 'blend.path','') != '':
				blend=1
				file.write("\t\t\t<texture>\n")
				currpath=Blender.sys.dirname(Blender.Get('filename'))
				target=readValue(d, 'blend.path', '')
				if target[0:2] == "\\\\" or target[0:2] == "//":
					target=currpath+os.sep+target[2:]
				file.write("\t\t\t\t<path>%s</path>\n"%(utils.getRelativePath(file.name, target)))
				file.write("\t\t\t\t<exponent>%f</exponent>\n"%(readValue(d, 'blend.exp', 2.2)))
				if uvset != '':
					file.write("\t\t\t\t<uv_set>%s</uv_set>\n" % check_uvtext(uvset))
				else:
					file.write("\t\t\t\t<uv_set>%s</uv_set>\n" % check_uvtext(readValue(d, 'blend.uv_set', '')))
				file.write("\t\t\t\t<a>%g</a>\n"%(readValue(d, 'blend.a', 1.0)))
				file.write("\t\t\t\t<b>%g</b>\n"%(readValue(d, 'blend.b', 1.0)))
				file.write("\t\t\t\t<c>%g</c>\n"%(readValue(d, 'blend.c', 1.0)))
				file.write("\t\t\t</texture>\n")
			
			if not IndigoUIControls.Renderer.ClayRender.val and IndigoTypes.Materials.isT(matType, 'SPECULAR') and readValue(d, 'alt.path','') != '':
				alt=1
				file.write("\t\t\t<texture>\n")
				currpath=Blender.sys.dirname(Blender.Get('filename'))
				target=readValue(d, 'alt.path', '')
				if target[0:2] == "\\\\" or target[0:2] == "//":
					target=currpath+os.sep+target[2:]
				file.write("\t\t\t\t<path>%s</path>\n"%(utils.getRelativePath(file.name, target)))
				file.write("\t\t\t\t<exponent>%f</exponent>\n"%(readValue(d, 'alt.exp', 2.2)))
				if uvset != '':
					file.write("\t\t\t\t<uv_set>%s</uv_set>\n" % check_uvtext(uvset))
				else:
					file.write("\t\t\t\t<uv_set>%s</uv_set>\n" % check_uvtext(readValue(d, 'alt.uv_set', '')))
				file.write("\t\t\t\t<a>%g</a>\n"%(readValue(d, 'alt.a', 1.0)))
				file.write("\t\t\t\t<b>%g</b>\n"%(readValue(d, 'alt.b', 1.0)))
				file.write("\t\t\t\t<c>%g</c>\n"%(readValue(d, 'alt.c', 1.0)))
				file.write("\t\t\t</texture>\n")
			
			#---ALBEDO TEXTURE OR SHADER---
			if IndigoTypes.Materials.inT(matType, ['DIFFUSE', 'PHONG' ,'ORENNAYAR']):
				if albedo and not readValue(d,'albedo.isl',0):
					if readValue(d,'emitter',0):
						file.write("\t\t\t<emission>\n")
					elif matType == IndigoTypes.Materials._('PHONG'):
						file.write("\t\t\t<diffuse_albedo>\n")
					else:
						file.write("\t\t\t<albedo>\n")
					file.write("\t\t\t\t<texture>\n\t\t\t\t\t<texture_index>%d</texture_index>\n\t\t\t\t</texture>\n"%texture_index)
					if readValue(d,'emitter',0):
						file.write("\t\t\t</emission>\n")
					elif matType == IndigoTypes.Materials._('PHONG'):
						file.write("\t\t\t</diffuse_albedo>\n")
					else:
						file.write("\t\t\t</albedo>\n")
					texture_index+=1
				elif readValue(d,'albedo.isl',0):
					if readValue(d,'emitter',0):
						file.write("\t\t\t<emission>\n")
					elif matType == IndigoTypes.Materials._('PHONG'):
						file.write("\t\t\t<diffuse_albedo>\n")
					else:
						file.write("\t\t\t<albedo>\n")
					file.write("\t\t\t\t<shader>\n")
					file.write("\t\t\t\t\t<shader>\n\t\t\t\t\t\t<![CDATA[\n")
					shadername=ShaderList[readValue(d,'albedo.shader',-1)]
					text=Blender.Text.Get(shadername)
					for l in text.asLines():
						file.write("\t\t\t\t\t%s\n"%l);
					file.write("\t\t\t\t\t\t]]>\n\t\t\t\t\t</shader>\n\t\t\t\t</shader>\n")
					if readValue(d,'emitter',0):
						file.write("\t\t\t<emission>\n")
					elif matType == IndigoTypes.Materials._('PHONG'):
						file.write("\t\t\t</diffuse_albedo>\n")
					else:
						file.write("\t\t\t</albedo>\n")
				else:
					if matType == IndigoTypes.Materials._('PHONG') and readValue(d,'toggle_phong',0):
						file.write("\t\t\t<specular_reflectivity>\n")
						file.write("\t\t\t\t<constant>\n\t\t\t\t\t<rgb>\n\t\t\t\t\t\t<rgb>%g %g %g</rgb>\n\t\t\t\t\t\t<gamma>1</gamma>\n\t\t\t\t\t</rgb>\n\t\t\t\t</constant>\n"%(readValue(d, 'rgb.r_s', 0.0), readValue(d, 'rgb.g_s', 0.0), readValue(d, 'rgb.b_s', 0.0)))
					else:
						if matType == IndigoTypes.Materials._('PHONG'):
							file.write("\t\t\t<diffuse_albedo>\n")
						else:
							file.write("\t\t\t<albedo>\n")
						if not IndigoUIControls.Renderer.ClayRender.val:
							file.write("\t\t\t\t<constant><rgb>\n\t\t\t\t\t<rgb>%g %g %g</rgb>\n\t\t\t\t\t<gamma>2.2</gamma>\n\t\t\t\t</rgb></constant>\n"%(readValue(d, 'rgb.r', 0.0), readValue(d, 'rgb.g', 0.0), readValue(d, 'rgb.b', 0.0)))
						else:
							file.write("\t\t\t\t<constant><rgb>\n\t\t\t\t\t<rgb>0.85 0.85 0.85</rgb>\n\t\t\t\t\t<gamma>2.2</gamma>\n\t\t\t\t</rgb></constant>\n")
					if matType == IndigoTypes.Materials._('PHONG') and readValue(d,'toggle_phong',0):
						file.write("\t\t\t</specular_reflectivity>\n")
					else:
						if matType == IndigoTypes.Materials._('PHONG'):
							file.write("\t\t\t</diffuse_albedo>\n")
						else:
							file.write("\t\t\t</albedo>\n")
			
			#---EXPONENT TEXTURE OR SHADER
			if exponent and not readValue(d,'exponent.isl',0):
				file.write("\t\t\t<exponent>\n\t\t\t\t<texture>\n\t\t\t\t\t<texture_index>%d</texture_index>\n\t\t\t\t</texture>\n\t\t\t</exponent>\n"%texture_index)
				texture_index+=1
			elif readValue(d,'exponent.isl',0):
				file.write("\t\t\t<exponent>\n")
				file.write("\t\t\t\t<shader>\n")
				file.write("\t\t\t\t\t<shader>\n\t\t\t\t\t\t<![CDATA[\n")
				shadername=ShaderList[readValue(d,'exponent.shader',-1)]
				text=Blender.Text.Get(shadername)
				for l in text.asLines():
					file.write("\t\t\t\t\t%s\n"%l);
				file.write("\t\t\t\t\t\t]]>\n\t\t\t\t\t</shader>\n\t\t\t\t</shader>\n\t\t\t</exponent>\n")
			#---BUMP TEXTURE OR SHADER---
			if bump and not readValue(d,'bump.isl',0):
				file.write("\t\t\t<bump>\n\t\t\t\t<texture>\n\t\t\t\t\t<texture_index>%d</texture_index>\n\t\t\t\t</texture>\n\t\t\t</bump>\n"%texture_index)
				texture_index+=1
			elif readValue(d,'bump.isl',0):
				file.write("\t\t\t<bump>\n")
				file.write("\t\t\t\t<shader>\n")
				file.write("\t\t\t\t\t<shader>\n\t\t\t\t\t\t<![CDATA[\n")
				shadername=ShaderList[readValue(d,'bump.shader',-1)]
				text=Blender.Text.Get(shadername)
				for l in text.asLines():
					file.write("\t\t\t\t\t%s\n"%l);
				file.write("\t\t\t\t\t\t]]>\n\t\t\t\t\t</shader>\n\t\t\t\t</shader>\n\t\t\t</bump>\n")
			#---DISPLACEMNET TEXTURE OR SHADER
			if displace and not readValue(d,'displace.isl',0):
				file.write("\t\t\t<displacement>\n\t\t\t\t<texture>\n\t\t\t\t\t<texture_index>%d</texture_index>\n\t\t\t\t</texture>\n\t\t\t</displacement>\n"%texture_index)
				texture_index+=1
			elif readValue(d,'displace.isl',0):
				file.write("\t\t\t<displacement>\n")
				file.write("\t\t\t\t<shader>\n")
				file.write("\t\t\t\t\t<shader>\n\t\t\t\t\t\t<![CDATA[\n")
				shadername=ShaderList[readValue(d,'displace.shader',-1)]
				text=Blender.Text.Get(shadername)
				for l in text.asLines():
					file.write("\t\t\t\t\t%s\n"%l);
				file.write("\t\t\t\t\t\t]]>\n\t\t\t\t\t</shader>\n\t\t\t\t</shader>\n\t\t\t</displacement>\n")
			#---Absorption Layer Transmittance TEXTURE OR SHADER
			if IndigoTypes.Materials.isT(matType, 'SPECULAR'):
				if alt and not readValue(d,'alt.isl',0):
					file.write("\t\t\t<absorption_layer_transmittance>\n\t\t\t\t<texture>\n\t\t\t\t\t<texture_index>%d</texture_index>\n\t\t\t\t</texture>\n\t\t\t</absorption_layer_transmittance>\n"%texture_index)
					texture_index+=1
				elif readValue(d,'alt.isl',0):
					file.write("\t\t\t<absorption_layer_transmittance>\n")
					file.write("\t\t\t\t<shader>\n")
					file.write("\t\t\t\t\t<shader>\n\t\t\t\t\t\t<![CDATA[\n")
					shadername=ShaderList[readValue(d,'alt.shader',-1)]
					text=Blender.Text.Get(shadername)
					for l in text.asLines():
						file.write("\t\t\t\t\t%s\n"%l);
					file.write("\t\t\t\t\t\t]]>\n\t\t\t\t\t</shader>\n\t\t\t\t</shader>\n\t\t\t</absorption_layer_transmittance>\n")
			
			#---BLEND TEXTURE OR SHADER
			if IndigoTypes.Materials.isT(matType, 'BLEND'):
				if blend and not readValue(d,'blend.isl',0):
					file.write("\t\t\t<blend>\n\t\t\t\t<texture>\n\t\t\t\t\t<texture_index>%d</texture_index>\n\t\t\t\t</texture>\n\t\t\t</blend>\n"%texture_index)
					texture_index+=1
				elif readValue(d,'blend.isl',0):
					file.write("\t\t\t<blend>\n")
					file.write("\t\t\t\t<shader>\n")
					file.write("\t\t\t\t\t<shader>\n\t\t\t\t\t\t<![CDATA[\n")
					shadername=ShaderList[readValue(d,'blend.shader',-1)]
					text=Blender.Text.Get(shadername)
					for l in text.asLines():
						file.write("\t\t\t\t\t%s\n"%l);
					file.write("\t\t\t\t\t\t]]>\n\t\t\t\t\t</shader>\n\t\t\t\t</shader>\n\t\t\t</blend>\n")
				else:
					file.write("\t\t\t<blend>\n")
					file.write("\t\t\t\t<constant>%g</constant>\n"%(readValue(d, 'blend.factor', 0.0)))
					file.write("\t\t\t</blend>\n")
			#^^^END TEXTURES^^^
			
			#can Emitts
			if IndigoTypes.Materials.inT(matType, ['DIFFUSE', 'ORENNAYAR', 'PHONG', 'SPECULAR']):
				emitter=readValue(d, 'emitter',0)
				if(emitter):
					gain=readValue(d, 'emitter.gain',0)
					mult=readValue(d, 'emitter.mult',0)
					file.write("\t\t\t<base_emission>\n")
					file.write("\t\t\t\t<constant>\n")
					etype = readValue(d,'emitter.type', IndigoTypes.Spectrum._('RGB'))
					if etype == IndigoTypes.Spectrum._('RGB'):
						file.write("\t\t\t\t\t<rgb>\n")
						file.write("\t\t\t\t\t\t<rgb>%f %f %f</rgb>\n"%(readValue(d, 'rgb.r_e',0)*gain*10**mult,readValue(d, 'rgb.g_e',0)*gain*10**mult,readValue(d, 'rgb.b_e',0)*gain*10**mult))
						file.write("\t\t\t\t\t\t<gamma>1</gamma>\n")
						file.write("\t\t\t\t\t</rgb>\n")
					elif etype == IndigoTypes.Spectrum._('BLACKBODY'):
						file.write("\t\t\t\t\t<blackbody>\n")
						file.write("\t\t\t\t\t\t<temperature>%d</temperature>\n"%readValue(d,'emitter.temperature',5000))
						file.write("\t\t\t\t\t\t<gain>%f</gain>\n"%float(gain*10**mult))
						file.write("\t\t\t\t\t</blackbody>\n")
					elif etype == IndigoTypes.Spectrum._('LAMPSPECTRUM'):
						lsn = readValue(d,'emitter.lspreset','')
						lsdata = lampspectrum.get_data(lsn)
						file.write("\t\t\t\t\t<regular_tabulated>\n")
						file.write("\t\t\t\t\t\t<start_wavelength>0.4e-6</start_wavelength>\n")
						file.write("\t\t\t\t\t\t<end_wavelength>0.7e-6</end_wavelength>\n")
						file.write("\t\t\t\t\t\t<num_values>300</num_values>\n")
						file.write("\t\t\t\t\t\t<values>%s</values>\n" % ' '.join(['%g'%i for i in lsdata]))
						file.write("\t\t\t\t\t</regular_tabulated>\n")
						 
					file.write("\t\t\t\t</constant>\n")
					file.write("\t\t\t</base_emission>\n")
					if readValue(d,'emitter',0):
						lname = readValue(d,'emitter.layer','default')
						if lname not in IndigoGlobals.layer_names:
							IndigoGlobals.layer_names.append(lname)
							if lname in IndigoGlobals.layer_name_counts.keys():
								IndigoGlobals.layer_name_counts[lname] += 1
							else:
								IndigoGlobals.layer_name_counts[lname] = 1
						lindex = IndigoGlobals.layer_names.index(lname)
						file.write("\t\t\t<layer>%d</layer>\n"%lindex)
			
			if IndigoTypes.Materials.isT(matType, 'DIFFUSE'):
				transparent=readValue(d,'diffuse.transmitter',0)
				if not transparent or IndigoUIControls.Renderer.ClayRender.val:
					file.write("\t\t</diffuse>\n")
				else:
					file.write("\t\t</diffuse_transmitter>\n")
			
			if IndigoTypes.Materials.isT(matType, 'ORENNAYAR'):
				file.write("\t\t</oren_nayar>\n")
			
			if IndigoTypes.Materials.isT(matType, 'PHONG'):
				file.write("\t\t</phong>\n")
			
			if IndigoTypes.Materials.isT(matType, 'SPECULAR'):
				speculartype = readValue(d, 'specular.type', 1)
				if speculartype == IndigoTypes.Specular._('SPECULAR'):
					file.write("\t\t</specular>\n")
				if speculartype == IndigoTypes.Specular._('GLOSSY'):
					file.write("\t\t</glossy_transparent>\n")
			
			if IndigoTypes.Materials.isT(matType, 'BLEND'):
				file.write("\t\t</blend>\n")
			
			file.write("\t</material>\n")
			file.write("\n")
		
		if IndigoTypes.Materials.isT(matType, 'INCLUDE'):
			includefile = readValue(d, 'include', '').strip()
			if includefile != '':
				includefile = utils.getRelativePath(file.name, checkAndUnpackPigm(includefile))
				print("Include file: %s"%includefile)
				if includefile not in includelist:
					includelist.append(includefile)
				matname = os.path.splitext(os.path.basename(includefile))[0]
		
		if IndigoTypes.Materials.inT(matType, ['LIGHT', 'EXITPORTAL']):
			file.write("\t<material> <!-- light or exit portal dummy material -->\n")
			file.write("\t\t<name>%s</name>\n"%(matname))
			file.write("\t\t<null_material/>\n")
			file.write("\t</material>\n\n")
					
	return includelist, matname

def python25_zip_extractall(zf, destination):
	for f in zf.namelist():
		fn = os.path.join(destination, f)
		print(' py25-zip-extract ->  file: '+fn)
		if not os.path.exists( os.path.dirname(fn) ):
			print(' py25-zip-extract -> mkdir: '+fn)
			os.makedirs( fn )
		
		fh=open(fn, 'wb')
		fh.write( zf.read(f) )
		fh.close()
			

def checkAndUnpackPigm(filename):
	fn, ext = os.path.splitext(filename)
	base = os.path.basename(fn)
	
	#print('checkAndUnpackPigm filename:' + filename)
	
	if ext.lower() not in ['.igm', '.pigm']:
		print('Warning: trying to include non-material file: %s' % filename)
	
	# potentially extract files from pigm file
	if ext.lower() == '.pigm':
		newpath = os.path.join(IndigoPaths.get('matdbDownloads'), 'unpacked', base)
		#print('checkAndUnpackPigm newpath:' + newpath)
		try:
			os.makedirs( newpath )
			unpack = True
		except: # likely exception is that path exists
			# assume pigm is already unpacked
			unpack = False
			
		# unpack pigm if needed
		if unpack:
			print('Unpacking %s to %s' % (filename, newpath))
			try:
				import zipfile
			except:
				raise Exception('Python zipfile module required for working with pigm files. Install python 2.6 or greater.')
			
			zf = zipfile.ZipFile(filename, 'r')
			if PY25:
				python25_zip_extractall(zf, newpath)
			else:
				zf.extractall(newpath)
			zf.close()
			
		# find the real igm file and change filename
		igmfiles = [i for i in os.listdir(newpath) if os.path.splitext(i)[1].lower()=='.igm' ]
		#print('checkAndUnpackPigm igmfiles: %s' % igmfiles)
		if len(igmfiles) == 0:
			raise Exception('No materials found in %s' % filename)
		if len(igmfiles) > 1:
			print('Warning: More than one material file found in %s, using the 1st only' % filename)
			
		filename = os.path.join(newpath, igmfiles[0])
		
		#print('checkAndUnpackPigm filename:' + filename)
		
	return filename 

### ================================================================
### exportLight(material, pathname gain) -> string
###		returns indigo XML code for light material
### ----------------------------------------------------------------
def exportLight(mat, extgain, file):
	
	if mat != None:
		if 'indigo' not in mat.properties.keys():
			convertMaterial(mat)	
			
		m = []
		matType = readValue(d, 'type', 0)
		if matType == IndigoTypes.Materials.LIGHT:
			lighttype = readValue(d, 'light.type', 1)
			gain = readValue(d, 'light.gain', 1.0) * extgain
			file.write("\t\t<spectrum>\n")
			if lighttype == IndigoTypes.Spectrum.RGB:
				file.write("\t\t\t<rgb>\n")
				file.write("\t\t\t\t<rgb>%g %g %g</rgb>\n"%(readValue(d, 'rgb.r', 0.0)*gain, readValue(d, 'rgb.g', 0.0)*gain, readValue(d, 'rgb.b', 0.0)*gain))
				file.write("\t\t\t\t<gamma>2.2</gamma>\n");
				file.write("\t\t\t</rgb>\n")
			#if (lighttype == 2):
			#	file.write("\t\t\t<peak>\n")
			#	file.write("\t\t\t\t<base_value>%g</base_value>\n"%(readValue(d, 'light.base', 0.0)))
			#	file.write("\t\t\t\t<peak_min>%g</peak_min>\n"%(readValue(d, 'light.peak_min', 0.0)))
			#	file.write("\t\t\t\t<peak_width>%g</peak_width>\n"%(readValue(d, 'light.peak_width', 0.0)))
			#	file.write("\t\t\t\t<peak_value>%g</peak_value>\n"%(gain))
			#	file.write("\t\t\t</peak>\n")
			if lighttype == IndigoTypes.Spectrum.BLACKBODY:
				file.write("\t\t\t<blackbody>\n")
				temp = readValue(d, 'light.temperature', 1.0)
				file.write("\t\t\t\t<temperature>%g</temperature>\n"%(temp))
				# blackbody correction removed as indigo v0.7t3 has "efficacy scale" feature now
				# blackbody gain correction depending on temperature (see Stephan Boltzmann Law)
				file.write("\t\t\t\t<gain>%g</gain>\n"%(gain)) # *100/(5.6705e-8*pow(temp, 4))))
				file.write("\t\t\t</blackbody>\n")
			file.write("\t\t</spectrum>\n")
			
			if (readValue(d, 'light.efficacy_scale', 0) != 0):
				file.write("\t\t<efficacy_scale>\n")
				file.write("\t\t\t<power_drawn>%g</power_drawn>\n"%(readValue(d, 'light.scale_value', 0.0)))
				file.write("\t\t\t<overall_luminous_efficacy>%g</overall_luminous_efficacy>\n"%(readValue(d, 'light.overall_luminous_efficacy', 0.0)))
				file.write("\t\t</efficacy_scale>\n")
			elif (readValue(d,'light.ies',0) != 0):
				file.write("\t\t<ies_profile>\n")
				file.write("\t\t\t<path>%s</path>\n"%(utils.getRelativePath(file.name, readValue(d,'light.ies_path', ''))))
				file.write("\t\t</ies_profile>\n")
			
			if (readValue(d, 'albedo.path', '').strip() != ''):
				file.write("\t\t<texture>\n")
				file.write("\t\t\t<path>%s</path>\n"%(utils.getRelativePath(file.name, readValue(d, 'albedo.path', ''))))
				file.write("\t\t\t<uv_set>%s</uv_set>\n" % check_uvtext(readValue(d, 'albedo.uv_set', 'default')))
				file.write("\t\t\t<exponent>%f</exponent>\n"%(readValue(d, 'albedo.exponent', 1.0)))
				file.write("\t\t</texture>\n")

### ================================================================
### matIsExitPortal(material) -> boolean
###		returns true if material is an exit portal
### ----------------------------------------------------------------
def matIsExitPortal(mat):
	if mat != None:
		
		if 'indigo' not in mat.properties.keys():
			convertMaterial(mat)
		
		return readValue(mat.properties['indigo'], 'type', 0) == IndigoTypes.Materials._('EXITPORTAL')

### ================================================================
### matXXXXPathCB(filename)
###		GUI file-selector callbacks
### ----------------------------------------------------------------
def matEmittIESPathCB(filename):
	IndigoUIControls.Materials.matEmittIESPath.val = filename
	readWriteMaterial(IndigoGlobals.activeMaterial, 1)

def matAlbedoPathCB(filename):
	IndigoUIControls.Materials.matAlbedoPath.val = filename
	readWriteMaterial(IndigoGlobals.activeMaterial, 1)

def matBumpPathCB(filename):
	IndigoUIControls.Materials.matBumpPath.val = filename
	readWriteMaterial(IndigoGlobals.activeMaterial, 1)

def matExponentPathCB(filename):
	IndigoUIControls.Materials.matExponentPath.val = filename
	readWriteMaterial(IndigoGlobals.activeMaterial, 1)

def matBlendPathCB(filename):
	IndigoUIControls.Materials.matBlendPath.val = filename
	readWriteMaterial(IndigoGlobals.activeMaterial, 1)

def matIncludeCB(filename):
	IndigoUIControls.Materials.matInclude.val = filename
	readWriteMaterial(IndigoGlobals.activeMaterial, 1)

def matDisplaceCB(filename):
	IndigoUIControls.Materials.matDisplacePath.val = filename
	readWriteMaterial(IndigoGlobals.activeMaterial, 1)
	
def matALTCB(filename):
	IndigoUIControls.Materials.matALTPath.val = filename
	readWriteMaterial(IndigoGlobals.activeMaterial, 1)

def matSaveCB(filename):
	igmfile=open(filename,'w')
	igmfile.write("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n\n")
	igmfile.write('<scenedata>\n')
	exportedmatlist=[]
	includelist, matname = exportMaterial(IndigoGlobals.activeMaterial, igmfile, exportedmatlist)
	igmfile.write('</scenedata>\n')
	igmfile.close()

### ================================================================
### materialGUI()
###		draws the material GUI
### ----------------------------------------------------------------
def materialGUI():
	drawScreenButtons()
	
	def xd(pos, div):
		return (IndigoUIControls.Materials.materialGUIwidth-10)/div*pos+10
	def wd(div):
		return (IndigoUIControls.Materials.materialGUIwidth-10)/div-10
	
	gui = guiLayout()
	gui.y = int(scrollbar.getTop()) - 70
	
	gui.title("Material editor")
	gui.newline()
	
	mats = Blender.Material.Get()
	matsmenustr = "Materials:%t|"
	
	for m in mats:
		matsmenustr += m.getName()+"|"
	matsmenustr = matsmenustr[:-1]
	gui.newline("Blender material")
	IndigoUIControls.Materials.matsMenu = Blender.Draw.Menu(matsmenustr, IndigoEvents.Materials.evtMatsMenu, gui.x, gui.y, wd(2)-10, 18, IndigoUIControls.Materials.matsMenu.val, "select Blender material")
	
	gui.newline()
	
	Blender.Draw.Button("Reset", IndigoEvents.Materials.evtMatConvert, gui.x, gui.y, 60, 18, "Reset to the original blender material")
	gui.x += 65
	gui.text("or")
	
	# Blender.Draw.Button("Import", evtMatSave, gui.x, gui.y, 60, 18, "Load an indigo material from an igm file")
	# gui.x += 65
	
	Blender.Draw.Button("Export", IndigoEvents.Materials.evtMatSave, gui.x, gui.y, 60, 18, "Save current indigo material into separate igm file")
	gui.x += 65
	
	if IndigoGlobals.activeMaterial != None:
		gui.text("or")
		Blender.Draw.Button("Copy", IndigoEvents.Materials.evtMatCopy, gui.x, gui.y, 40, 18, "the present editor material")
		gui.x += 45
		if IndigoUIControls.Materials.matClipboard != None:
			Blender.Draw.Button("Paste", IndigoEvents.Materials.evtMatPaste, gui.x, gui.y, 45, 18, "the  previously copied editor material")
		
		gui.subtitle("Indigo Material")
		
		gui.newline("Material type")
		
		IndigoUIControls.Materials.matTypeMenu = Blender.Draw.Menu(IndigoTypes.Materials.menuString(), IndigoEvents.Materials.evtMatType, gui.x, gui.y, 320, 18, IndigoUIControls.Materials.matType, "select the desired material type")
		IndigoUIControls.Materials.matType = IndigoUIControls.Materials.matTypeMenu.val
		
		gui.newline()
		Blender.Draw.Button("Preview", IndigoEvents.Materials.evtMatPreview, gui.x, gui.y, 70, 18, "render a preview of the material in simple scene")
		
		if IndigoTypes.Materials.isT(IndigoUIControls.Materials.matType, 'DIFFUSE'):
			gui.subtitle("Diffuse material")
			gui.newline("Transparent")
			IndigoUIControls.Materials.matDiffTrans = Blender.Draw.Toggle( IndigoUIControls.OnOffToggleText(IndigoUIControls.Materials.matDiffTrans), IndigoEvents.evtNoEvt, gui.x, gui.y, 30, 18, IndigoUIControls.Materials.matDiffTrans.val, "enable transparency")
		
		if IndigoTypes.Materials.isT(IndigoUIControls.Materials.matType, 'ORENNAYAR'):
			gui.subtitle("Oren-nayar material")
			gui.newline("Sigma")
			IndigoUIControls.Materials.matSigma = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y,wd(4)+5,18, IndigoUIControls.Materials.matSigma.val,0.0,1.0,"material roughness 0.0. - 1.0")
		
		if IndigoTypes.Materials.isT(IndigoUIControls.Materials.matType, 'PHONG'):
			gui.subtitle("Phong material")
			
			gui.newline("NK Data")
			IndigoUIControls.Materials.matNKData = Blender.Draw.String("", IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Materials.matNKData.val, 20, "lab measured mat. parameters called as e.g. Au.nk = gold")
			
			gui.newline("Exponent")
			IndigoUIControls.Materials.matExponent = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Materials.matExponent.val, 1.0, 1e+9, "surface shininess - rubber 50 , mirror 1 million")
			gui.units("dimensionless", 100)
			
			gui.newline("Index of Refract")
			IndigoUIControls.Materials.matIOR = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Materials.matIOR.val, 1e-4, 1000.0, "index of refraction,water 1.33 glass 1.5")
			gui.units("dimensionless", 100)
			
			gui.newline("Option")
			IndigoUIControls.Materials.matTogglePhong = Blender.Draw.Toggle("Specular Reflectivity", IndigoEvents.evtNoEvt, gui.x, gui.y,wd(2),18, IndigoUIControls.Materials.matTogglePhong.val,"enable specular reflectivity color")
			
			y = gui.y
			
			if IndigoUIControls.Materials.matTogglePhong.val:
				gui.subtitle("Specular reflectivity")
				
				gui.newline("Color")
				IndigoUIControls.Materials.matRgb_s = Blender.Draw.ColorPicker(IndigoEvents.Materials.evtMatRgbPicker_s, gui.x, gui.y - 40, 30, 58, (IndigoUIControls.Materials.matRgbR_s.val, IndigoUIControls.Materials.matRgbG_s.val, IndigoUIControls.Materials.matRgbB_s.val), "colour and picker (LMB)")
				gui.x += 32
				IndigoUIControls.Materials.matRgbR_s = Blender.Draw.Slider("R ", IndigoEvents.evtNoEvt, gui.x, gui.y, 120, 18, IndigoUIControls.Materials.matRgbR_s.val, 0.0, 1.0, 1, "red component of colour")
				gui.y -= 20
				IndigoUIControls.Materials.matRgbG_s = Blender.Draw.Slider("G ", IndigoEvents.evtNoEvt, gui.x, gui.y, 120, 18, IndigoUIControls.Materials.matRgbG_s.val, 0.0, 1.0, 1, "green component of colour")
				gui.y -= 20
				IndigoUIControls.Materials.matRgbB_s = Blender.Draw.Slider("B ", IndigoEvents.evtNoEvt, gui.x, gui.y, 120, 18, IndigoUIControls.Materials.matRgbB_s.val, 0.0, 1.0, 1, "blue component of colour")
				gui.y -= 20
		
		if IndigoTypes.Materials.isT(IndigoUIControls.Materials.matType, 'SPECULAR'):
			gui.subtitle("Specular material")
			gui.newline("Spec type:")

			IndigoUIControls.Materials.matSpecularType = Blender.Draw.Menu(IndigoTypes.Specular.menuString(), IndigoEvents.evtNoEvt, gui.x, gui.y, wd(3), 18, IndigoUIControls.Materials.matSpecularType.val, "select specular type")
			
			if IndigoTypes.Specular.isT(IndigoUIControls.Materials.matSpecularType.val, 'SPECULAR'):
				gui.newline("Transparency")
				IndigoUIControls.Materials.matTransparent = Blender.Draw.Toggle(IndigoUIControls.OnOffToggleText(IndigoUIControls.Materials.matTransparent), IndigoEvents.evtNoEvt, gui.x, gui.y, 30, 18, IndigoUIControls.Materials.matTransparent.val, "enable transparency")
			
			if IndigoTypes.Specular.isT(IndigoUIControls.Materials.matSpecularType.val, 'GLOSSY'):
				gui.newline("Exponent")
				IndigoUIControls.Materials.matExponent = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Materials.matExponent.val, 1.0, 1e+9, "surface shininess - rubber 50 , mirror 1 million")
				gui.units("dimensionless", 100)

			gui.newline("Precedence")
			IndigoUIControls.Materials.matPrecedence = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 50, 18, IndigoUIControls.Materials.matPrecedence.val, 1, 100, "set precendence of mediums when sharing same space")
			gui.units("integer", 50)
			
			gui.newline("Medium type:")
			IndigoUIControls.Materials.matMediumType = Blender.Draw.Menu(IndigoTypes.Medium.menuString(), IndigoEvents.evtNoEvt, gui.x, gui.y, wd(4), 18, IndigoUIControls.Materials.matMediumType.val, "select medium type")
			
			if IndigoTypes.Medium.isT(IndigoUIControls.Materials.matMediumType.val,'DERMIS'):
				gui.newline("  Haemoglobin")
				IndigoUIControls.Materials.matHemoglobin = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Materials.matHemoglobin.val, 0.0, 1.0, "amount of hemoglobin, typ range .001-.01")
				gui.units("fraction", 100)
				
			if IndigoTypes.Medium.isT(IndigoUIControls.Materials.matMediumType.val,'EPIDERMIS'):
				gui.newline("  Melanin")
				IndigoUIControls.Materials.matMelanin = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Materials.matMelanin.val, 0.0, 1.0, "amount of melanin, typ range 0-.05")
				gui.units("fraction", 100)

				gui.newline("  Eumelanin")
				IndigoUIControls.Materials.matEumelanin = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Materials.matEumelanin.val, 0.0, 1.0, "amount of eumelanin relative to pheomelanin")
				gui.units("fraction", 100)
				
			if IndigoTypes.Medium.isT(IndigoUIControls.Materials.matMediumType.val,'ATMOSPHERE'):
				gui.newline("  Turbidity")
				IndigoUIControls.Materials.matTurbidity = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Materials.matTurbidity.val, 2, 10, "Atmosphere turbidity. Values 2-10")
				gui.units("real", 100)
				
			if IndigoTypes.Medium.isT(IndigoUIControls.Materials.matMediumType.val,'BASIC'):
				gui.newline("  Index of refrac.")
				IndigoUIControls.Materials.matIOR = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Materials.matIOR.val, 1e-4, 1000.0, "index of refraction,water 1.33 glass 1.5")
				
				gui.newline("  Cauchy B coeff")
				IndigoUIControls.Materials.matCauchyBCoeff = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Materials.matCauchyBCoeff.val, 0.0, 1.0, "dispersive refr, range .003-.01, 0 disables ")
				gui.units("coefficient", 100)
				
				gui.newline("  Gain")
				IndigoUIControls.Materials.matAbsGain = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Materials.matAbsGain.val, 1e-3, 1e3, "RGB gain")
				gui.units("dimensionless", 100)
				
				gui.newline('  Absorption type')
				IndigoUIControls.Materials.matAbsorptionType = Blender.Draw.Menu(
					IndigoTypes.Spectrum.menuString(limit=['RGB', 'UNIFORM']),
					IndigoEvents.evtNoEvt, gui.x, gui.y, wd(4), 18, IndigoUIControls.Materials.matAbsorptionType.val, "select absorption type"
				)
				
				if IndigoTypes.Spectrum.isT(IndigoUIControls.Materials.matAbsorptionType, 'RGB'):
					gui.newline("  Color")
					IndigoUIControls.Materials.matAbs = Blender.Draw.ColorPicker(IndigoEvents.Materials.evtMatAbsPicker, gui.x, gui.y - 40, 30, 58, (1.0-IndigoUIControls.Materials.matAbsR.val, 1.0-IndigoUIControls.Materials.matAbsG.val, 1.0-IndigoUIControls.Materials.matAbsB.val), "absorption color and picker (LMB)")
					gui.x += 32
					IndigoUIControls.Materials.matAbsR = Blender.Draw.Slider("R ", IndigoEvents.evtNoEvt, gui.x, gui.y, wd(2)-30, 18, IndigoUIControls.Materials.matAbsR.val, 0.0, 1.0, 1, "red component of absorption colour")
					gui.y -= 20
					IndigoUIControls.Materials.matAbsG = Blender.Draw.Slider("G ", IndigoEvents.evtNoEvt, gui.x, gui.y, wd(2)-30, 18, IndigoUIControls.Materials.matAbsG.val, 0.0, 1.0, 1, "green component of absorption colour")
					gui.y -= 20
					IndigoUIControls.Materials.matAbsB = Blender.Draw.Slider("B ", IndigoEvents.evtNoEvt, gui.x, gui.y, wd(2)-30, 18, IndigoUIControls.Materials.matAbsB.val, 0.0, 1.0, 1, "blue component of absorption colour")
				elif IndigoTypes.Spectrum.isT(IndigoUIControls.Materials.matAbsorptionType, 'UNIFORM'):
					pass
				
				gui.subtitle("       Sub surface scattering")
				IndigoUIControls.Materials.matSss = Blender.Draw.Toggle(IndigoUIControls.PlusMinusToggleText(IndigoUIControls.Materials.matSss), IndigoEvents.evtNoEvt, 16, gui.y+5, 20, 18, IndigoUIControls.Materials.matSss.val, "enable subsurface scattering")
				
				if IndigoUIControls.Materials.matSss.val:
					gui.newline("Scatter type:")
					IndigoUIControls.Materials.matSssType = Blender.Draw.Menu(IndigoTypes.Spectrum.menuString(limit=['UNIFORM','RGB']), IndigoEvents.evtNoEvt, gui.x, gui.y, wd(3), 18, IndigoUIControls.Materials.matSssType.val, "select scattering type")

					if IndigoTypes.Spectrum.isT(IndigoUIControls.Materials.matSssType.val, 'RGB'):
						gui.newline("  Gain")
						IndigoUIControls.Materials.matSssGain = Blender.Draw.Number("Gain ", IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Materials.matSssGain.val, 1e-3, 1e3, "RGB gain")
						gui.units("dimensionless", 100)

						gui.newline("  Color")
						IndigoUIControls.Materials.matSssRgb = Blender.Draw.ColorPicker(IndigoEvents.Materials.evtMatSssRgbPicker, gui.x, gui.y - 40, 30, 58, (IndigoUIControls.Materials.matSssR.val, IndigoUIControls.Materials.matSssG.val, IndigoUIControls.Materials.matSssB.val), "SSS color and picker (LMB)")
						gui.x += 32
						IndigoUIControls.Materials.matSssR = Blender.Draw.Slider("R ", IndigoEvents.evtNoEvt, gui.x, gui.y, wd(2)-30, 18, IndigoUIControls.Materials.matSssR.val, 0.0, 1.0, 1, "red component of SSS color")
						gui.y -= 20
						IndigoUIControls.Materials.matSssG = Blender.Draw.Slider("G ", IndigoEvents.evtNoEvt, gui.x, gui.y, wd(2)-30, 18, IndigoUIControls.Materials.matSssG.val, 0.0, 1.0, 1, "green component of SSS color")
						gui.y -= 20
						IndigoUIControls.Materials.matSssB = Blender.Draw.Slider("B ", IndigoEvents.evtNoEvt, gui.x, gui.y, wd(2)-30, 18, IndigoUIControls.Materials.matSssB.val, 0.0, 1.0, 1, "blue component of SSS color")
						
					if IndigoTypes.Spectrum.isT(IndigoUIControls.Materials.matSssType.val, 'UNIFORM'):
						gui.newline("   Value")
						IndigoUIControls.Materials.matSssUniform = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 80, 18, IndigoUIControls.Materials.matSssUniform.val,0,10,"scattering value mantissa")
						gui.units("x10^", 80)
						IndigoUIControls.Materials.matSssUniformExp = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, wd(4), 18, IndigoUIControls.Materials.matSssUniformExp.val,-5,5,"scattering value exponent 10^+-5")

					gui.newline("Phase function")
					IndigoUIControls.Materials.matSssPhaseType = Blender.Draw.Menu(IndigoTypes.Phase.menuString(), IndigoEvents.evtNoEvt, gui.x, gui.y, wd(3), 18, IndigoUIControls.Materials.matSssPhaseType.val, "select phase type")

					if IndigoTypes.Phase.isT(IndigoUIControls.Materials.matSssPhaseType.val, 'HG'):
						gui.newline("   Spectrum")
						IndigoUIControls.Materials.matSssPhaseSpec = Blender.Draw.Menu(IndigoTypes.Spectrum.menuString(limit=['UNIFORM','RGB']), IndigoEvents.evtNoEvt, gui.x, gui.y,wd(4),18, IndigoUIControls.Materials.matSssPhaseSpec.val,"Select phase spectrum")

						if IndigoTypes.Spectrum.isT(IndigoUIControls.Materials.matSssPhaseSpec.val, 'UNIFORM'):
							gui.newline("   Value")
							IndigoUIControls.Materials.matSssPhaseSpecUni=Blender.Draw.Number("", IndigoEvents.evtNoEvt,gui.x, gui.y,wd(4),18, IndigoUIControls.Materials.matSssPhaseSpecUni.val,-1,1,"Phase Function Uniform Value")

						if IndigoTypes.Spectrum.isT(IndigoUIControls.Materials.matSssPhaseSpec.val, 'RGB'):
							gui.newline("     Red")
							IndigoUIControls.Materials.matSssPhaseSpecR=Blender.Draw.Number("R", IndigoEvents.evtNoEvt, gui.x, gui.y,wd(4),18, IndigoUIControls.Materials.matSssPhaseSpecR.val,0,1,"Phase Function Red Value")
							gui.newline("     Green")
							IndigoUIControls.Materials.matSssPhaseSpecG=Blender.Draw.Number("G", IndigoEvents.evtNoEvt, gui.x, gui.y,wd(4),18, IndigoUIControls.Materials.matSssPhaseSpecG.val,0,1,"Phase Function Green Value")
							gui.newline("     Blue")
							IndigoUIControls.Materials.matSssPhaseSpecB=Blender.Draw.Number("B", IndigoEvents.evtNoEvt, gui.x, gui.y,wd(4),18, IndigoUIControls.Materials.matSssPhaseSpecB.val,0,1,"Phase Function Blue Value")
		
		if IndigoTypes.Materials.isT(IndigoUIControls.Materials.matType, 'BLEND'):
			gui.newline('Material A:')
			Blender.Draw.Button("...", IndigoEvents.Materials.evtMatBlendMatA, gui.x, gui.y, 40, 18, "select a material from list") #xd(0,2)+wd(2)-40
			gui.units(IndigoUIControls.Materials.matBlendMatA.val, 60)
			
			gui.newline('Material B:')
			Blender.Draw.Button("...", IndigoEvents.Materials.evtMatBlendMatB, gui.x, gui.y, 40, 18, "select a material from list")
			gui.units(IndigoUIControls.Materials.matBlendMatB.val, 60)
			
			gui.newline('Blend Factor:')
			IndigoUIControls.Materials.matBlendFactor = Blender.Draw.Slider("", IndigoEvents.evtNoEvt, gui.x, gui.y, 240, 18, IndigoUIControls.Materials.matBlendFactor.val, 0.0, 1.0, 0, "blend ratio of the materials")
			
			gui.newline('Blend Map:')
			IndigoUIControls.Materials.matBlendPath = Blender.Draw.String("filename: ", IndigoEvents.evtNoEvt, gui.x, gui.y, 240, 18, IndigoUIControls.Materials.matBlendPath.val, 250, "blend texture filename")
			kk = Blender.Draw.Button("...", IndigoEvents.Materials.evtMatBlendPath, gui.x+245, gui.y, 20, 18, "open file selector")
			
			if IndigoUIControls.Materials.matBlendPath.val is not "":
				gui.newline('Gamma:')
				IndigoUIControls.Materials.matBlendExponent = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 80, 18, IndigoUIControls.Materials.matBlendExponent.val, 0.0, 10.0, "texture gammma")
				gui.units('UV Set:', 118)
				IndigoUIControls.Materials.matBlendUVset = Blender.Draw.String("", IndigoEvents.evtNoEvt, gui.x, gui.y, 80, 18, IndigoUIControls.Materials.matBlendUVset.val, 20, "blend texture UV-set")
				#obj = Blender.Scene.GetCurrent().objects.active
				#mesh=obj.getData(mesh=True)
				#uvLayerNames=mesh.getUVLayerNames()
				#uvsets="UVsets:%t"
				#index=0
				#for l in uvLayerNames:
				#	uvsets+="|%s%%x%d"%(l,index)
				#	index+=1
				#matBlendUVset = Blender.Draw.Menu(uvsets, IndigoEvents.evtNoEvt, 107,y-205,95,18,matBlendUVset.val,"UV sets");
				gui.newline('(A) Gradient')
				IndigoUIControls.Materials.matBlendA = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 80, 18, IndigoUIControls.Materials.matBlendA.val, -1.0, 1.0, "")
				
				gui.newline('(B) Scale')
				IndigoUIControls.Materials.matBlendB = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 80, 18, IndigoUIControls.Materials.matBlendB.val, 0.0, 10.0, "")
				
				gui.newline('(C) Base Value / Offset')
				IndigoUIControls.Materials.matBlendC = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 80, 18, IndigoUIControls.Materials.matBlendC.val, 0.0, 10.0, "")
				
		##
		## Get textures and shaders...
		##
		texts=Blender.Text.Get()
		shader="NONE%x-1"
		IndigoGlobals.ShaderList[:]=[]
		for s in texts:
			shadername=s.getName()
			if(shadername[:4] == "isl_"):
				IndigoGlobals.ShaderList.append(s.getName())
		
		for index,l in enumerate(IndigoGlobals.ShaderList):
			shader+="|%s%%x%d"%(l, index)

		##
		##	Albedo channel
		##
		if (IndigoTypes.Materials.isT(IndigoUIControls.Materials.matType, 'PHONG') and not IndigoUIControls.Materials.matTogglePhong.val) \
		or  IndigoTypes.Materials.inT(IndigoUIControls.Materials.matType, ['DIFFUSE', 'ORENNAYAR', 'LIGHT']):
			gui.subtitle("Color")
			#currentMaterial.albedo.enabled = Blender.Draw.Toggle(" X ", IndigoEvents.evtNoEvt, 16, gui.y + 5, 20, 18, currentMaterial.albedo.enabled.val, "enable color channel")
			
			if IndigoUIControls.Materials.currentMaterial.albedo.enabled.val:
				gui.newline("Type")
				IndigoUIControls.Materials.currentMaterial.albedo.type = Blender.Draw.Menu(IndigoTypes.Channel.menuString(), IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Materials.currentMaterial.albedo.type.val, "select the desired material type")

				if IndigoUIControls.Materials.currentMaterial.albedo.typeColor():
					gui.newline("Color")
					IndigoUIControls.Materials.matRgb = Blender.Draw.ColorPicker(IndigoEvents.Materials.evtMatRgbPicker, gui.x, gui.y - 40, 30, 58, (IndigoUIControls.Materials.matRgbR.val, IndigoUIControls.Materials.matRgbG.val, IndigoUIControls.Materials.matRgbB.val), "colour and picker (LMB)")
					gui.x += 32
					IndigoUIControls.Materials.matRgbR = Blender.Draw.Slider("R ", IndigoEvents.evtNoEvt, gui.x, gui.y, 120, 18, IndigoUIControls.Materials.matRgbR.val, 0.0, 1.0, 1, "red component of colour")
					gui.y -= 20
					IndigoUIControls.Materials.matRgbG = Blender.Draw.Slider("G ", IndigoEvents.evtNoEvt, gui.x, gui.y, 120, 18, IndigoUIControls.Materials.matRgbG.val, 0.0, 1.0, 1, "green component of colour")
					gui.y -= 20
					IndigoUIControls.Materials.matRgbB = Blender.Draw.Slider("B ", IndigoEvents.evtNoEvt, gui.x, gui.y, 120, 18, IndigoUIControls.Materials.matRgbB.val, 0.0, 1.0, 1, "blue component of colour")

				if (IndigoUIControls.Materials.currentMaterial.albedo.typeShader()) or (IndigoUIControls.Materials.currentMaterial.albedo.typeTexture()):
					gui.newline("UV Texture")
					IndigoUIControls.Materials.matAlbedoUVset = Blender.Draw.String("", IndigoEvents.evtNoEvt, gui.x, gui.y, 95, 18, IndigoUIControls.Materials.matAlbedoUVset.val, 20, "albedo texture UV-set")

				if IndigoUIControls.Materials.currentMaterial.albedo.typeTexture():
					gui.newline("Filename")
					IndigoUIControls.Materials.matAlbedoPath = Blender.Draw.String("", IndigoEvents.evtNoEvt, gui.x, gui.y, 250, 18, IndigoUIControls.Materials.matAlbedoPath.val, 250, "albedo texture filename (referencing Blender UV co-ords)")
					Blender.Draw.Button("...", IndigoEvents.Materials.evtMatAlbedoPath, gui.x + 255, gui.y, 20, 18, "open file selector")

					gui.newline("Gamma")
					IndigoUIControls.Materials.matAlbedoExponent = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Materials.matAlbedoExponent.val, 0.0, 10.0, "texture gammma")
					gui.units("dimensionless", 100)

					gui.newline("(A) Brightness")
					IndigoUIControls.Materials.matAlbedoA = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Materials.matAlbedoA.val, 0.0, 10.0, "")
					gui.units("dimensionless", 100)
					
					gui.newline("(B) Contrast")
					IndigoUIControls.Materials.matAlbedoB = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Materials.matAlbedoB.val, 0.0, 10.0, "")
					gui.units("dimensionless", 100)

					gui.newline("(C) Saturation")
					IndigoUIControls.Materials.matAlbedoC = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Materials.matAlbedoC.val, 0.0, 10.0, "")
					gui.units("dimensionless", 100)

				if IndigoUIControls.Materials.currentMaterial.albedo.typeShader():
					# matAlbedoISL = Blender.Draw.Toggle("Shader", IndigoEvents.evtNoEvt, 10, y-280, 70, 18, matAlbedoISL.val, "Use Indigo Shader")
					gui.newline("Indigo Shader")
					IndigoUIControls.Materials.matAlbedoShader=Blender.Draw.Menu(shader, IndigoEvents.evtNoEvt, gui.x, gui.y, 250, 18, IndigoUIControls.Materials.matAlbedoShader.val, "Select Albedo Shader")
	
		##
		##	Emission channel
		##
		if not IndigoTypes.Materials.inT(IndigoUIControls.Materials.matType, ['LIGHT', 'BLEND', 'EXITPORTAL', 'INCLUDE']):
			gui.subtitle("       Light emission")
			IndigoUIControls.Materials.matEmitt = Blender.Draw.Toggle(IndigoUIControls.PlusMinusToggleText(IndigoUIControls.Materials.matEmitt), IndigoEvents.evtNoEvt, 16, gui.y + 6, 20, 18, IndigoUIControls.Materials.matEmitt.val, "enable emission")

			if IndigoUIControls.Materials.matEmitt.val:
				gui.newline("Light layer")
				IndigoUIControls.Materials.matEmittLayer=Blender.Draw.String("", IndigoEvents.evtNoEvt, gui.x, gui.y, 120, 18, IndigoUIControls.Materials.matEmittLayer.val, 250, "layer that light emitted from this material will be drawn to")

				gui.newline("Emission Units")
				IndigoUIControls.Materials.matEmittScaleMeasure = Blender.Draw.Menu(
					IndigoTypes.EmissionMeasure.menuString(),
					IndigoEvents.evtNoEvt,
					gui.x, gui.y,
					250, 18,
					IndigoUIControls.Materials.matEmittScaleMeasure.val,
					"Choose emission units"
				)
				
				gui.newline("Emission Value")
				IndigoUIControls.Materials.matEmittScaleValue = Blender.Draw.Number("", IndigoEvents.evtNoEvt , gui.x, gui.y, 80, 18, IndigoUIControls.Materials.matEmittScaleValue.val, 0.0, 1000000, "")
				gui.units(IndigoTypes.EmissionMeasure.units(IndigoUIControls.Materials.matEmittScaleMeasure.val), 80)
				
				gui.newline("Gain")
				IndigoUIControls.Materials.matEmittGain=Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 80, 18, IndigoUIControls.Materials.matEmittGain.val, 0.0, 1.0, "Light Gain value")
				gui.units("x 10^", 80)
				IndigoUIControls.Materials.matEmittMult=Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 20, 18, IndigoUIControls.Materials.matEmittMult.val, -5, 5, "Light Gain exponent")
				gui.units("(= %f)" % (IndigoUIControls.Materials.matEmittGain.val * 10**IndigoUIControls.Materials.matEmittMult.val), 35)
				
				gui.newline('Spectrum type')
				IndigoUIControls.Materials.matEmittType = Blender.Draw.Menu( IndigoTypes.Spectrum.menuString(limit=['RGB','BLACKBODY', 'LAMPSPECTRUM']), IndigoEvents.evtNoEvt, gui.x, gui.y, 250, 18, IndigoUIControls.Materials.matEmittType.val, "Select emission spectrum type")
				
				if IndigoTypes.Spectrum.isT(IndigoUIControls.Materials.matEmittType.val, 'RGB'):
					gui.newline("RGB Color")
					IndigoUIControls.Materials.matEmittRgb = Blender.Draw.ColorPicker(IndigoEvents.Materials.evtMatEmittRgbPicker, gui.x, gui.y - 40, 30, 58, (IndigoUIControls.Materials.matEmittRgbR.val, IndigoUIControls.Materials.matEmittRgbG.val, IndigoUIControls.Materials.matEmittRgbB.val), "colour and picker (LMB)")
					gui.x += 32
					IndigoUIControls.Materials.matEmittRgbR = Blender.Draw.Slider("R ", IndigoEvents.evtNoEvt, gui.x, gui.y, 120, 18, IndigoUIControls.Materials.matEmittRgbR.val, 0.0, 1.0, 1, "red component of colour")
					gui.y -= 20
					IndigoUIControls.Materials.matEmittRgbG = Blender.Draw.Slider("G ", IndigoEvents.evtNoEvt, gui.x, gui.y, 120, 18, IndigoUIControls.Materials.matEmittRgbG.val, 0.0, 1.0, 1, "green component of colour")
					gui.y -= 20
					IndigoUIControls.Materials.matEmittRgbB = Blender.Draw.Slider("B ", IndigoEvents.evtNoEvt, gui.x, gui.y, 120, 18, IndigoUIControls.Materials.matEmittRgbB.val, 0.0, 1.0, 1, "blue component of colour")
					gui.y -= 20
					gui.y += 19
				elif IndigoTypes.Spectrum.isT(IndigoUIControls.Materials.matEmittType.val, 'BLACKBODY'):
					gui.newline("Blackbody temperature")
					IndigoUIControls.Materials.matEmittTemp = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 80, 18, IndigoUIControls.Materials.matEmittTemp.val, 500.0, 10000.0, "Blackbody emitter temperature")
				elif IndigoTypes.Spectrum.isT(IndigoUIControls.Materials.matEmittType.val, 'LAMPSPECTRUM'):
					gui.newline("Measured lamp preset")
					#mnu = utils.menuStringFromList("Spectrum", lampspectrum.get_names())
					#IndigoUIControls.Materials.matEmittLSPreset = Blender.Draw.Menu(
					#	mnu,
					#	IndigoEvents.evtNoEvt,
					#	gui.x, gui.y,
					#	250, 18,
					#	IndigoUIControls.Materials.matEmittLSPreset.val,
					#	"Choose lamp preset"
					#)
					def setLampSpectrum(i, ctl):
						if i>=0:
							ctl.val = lampspectrum.get_names()[i]
					
					Blender.Draw.Button(
						IndigoUIControls.Materials.matEmittLSPreset.val,
						IndigoEvents.evtNoEvt,
						gui.x, gui.y,
						230, 18,
						"Choose lamp preset",
						lambda e,v: setLampSpectrum(
							Blender.Draw.PupTreeMenu( lampspectrum.get_tree() ),
							IndigoUIControls.Materials.matEmittLSPreset
						)
					)
					# Show prevew RGB
					Blender.Draw.ColorPicker(
						IndigoEvents.evtNoEvt,
						gui.x+230, gui.y,
						20, 18,
						lampspectrum.get_rgb(IndigoUIControls.Materials.matEmittLSPreset.val),
						'Lamp spectrum RGB preview (D65 whitepoint)'
					)
					
				
				gui.newline("IES Profile")
				IndigoUIControls.Materials.matEmittIES = Blender.Draw.Toggle("Enable", IndigoEvents.evtNoEvt, gui.x, gui.y, 60, 18, IndigoUIControls.Materials.matEmittIES.val, "Enable IES Profile")

				if IndigoUIControls.Materials.matEmittIES.val:
					gui.newline("Filename")
					IndigoUIControls.Materials.matEmittIESPath = Blender.Draw.String("filename: ", IndigoEvents.evtNoEvt, gui.x, gui.y, 300, 18, IndigoUIControls.Materials.matEmittIESPath.val, 250, "Path to file with IES Profile ")
					Blender.Draw.Button("...", IndigoEvents.Materials.evtMatEmittIESPath, gui.x + 310, gui.y, 20, 18, "open file selector")

		##
		##	Bump channel
		##
		if (IndigoTypes.Materials.isT(IndigoUIControls.Materials.matType, 'DIFFUSE') and not IndigoUIControls.Materials.matDiffTrans.val) \
		or  IndigoTypes.Materials.inT(IndigoUIControls.Materials.matType, ['PHONG', 'SPECULAR', 'ORENNAYAR']):
			gui.subtitle("       Bump")
			IndigoUIControls.Materials.currentMaterial.bump.enabled = Blender.Draw.Toggle(IndigoUIControls.PlusMinusToggleText(IndigoUIControls.Materials.currentMaterial.bump.enabled), IndigoEvents.evtNoEvt, 16, gui.y + 6, 20, 18, IndigoUIControls.Materials.currentMaterial.bump.enabled.val, "enable bump channel")
		
			if IndigoUIControls.Materials.currentMaterial.bump.enabled.val:
				gui.newline("Type")
				IndigoUIControls.Materials.currentMaterial.bump.type = Blender.Draw.Menu(IndigoTypes.Channel.menuString(limit=['TEXTURE','SHADER']), IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Materials.currentMaterial.bump.type.val, "select the desired channel type")

				if (IndigoUIControls.Materials.currentMaterial.bump.typeShader()) or (IndigoUIControls.Materials.currentMaterial.bump.typeTexture()):
					gui.newline("UV Texture")
					IndigoUIControls.Materials.matBumpUVset = Blender.Draw.String("", IndigoEvents.evtNoEvt, gui.x, gui.y, 95, 18, IndigoUIControls.Materials.matBumpUVset.val, 20, "bump texture UV-set")

				if IndigoUIControls.Materials.currentMaterial.bump.typeTexture():
					gui.newline("Filename")
					IndigoUIControls.Materials.matBumpPath = Blender.Draw.String("", IndigoEvents.evtNoEvt, gui.x, gui.y, 250, 18, IndigoUIControls.Materials.matBumpPath.val, 250, "texture filename (referencing Blender UV co-ords)")
					Blender.Draw.Button("...", IndigoEvents.Materials.evtMatBumpPath, gui.x + 255, gui.y, 20, 18, "open file selector")

					gui.newline("Gamma")
					IndigoUIControls.Materials.matBumpExponent = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Materials.matBumpExponent.val, 0.0, 10.0, "texture gammma")
					gui.units("dimensionless", 100)
					
					gui.newline("(A) Gradient")
					IndigoUIControls.Materials.matBumpA = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Materials.matBumpA.val, -1.0, 1.0, "")
					
					gui.newline("(B) Height / Scale")
					IndigoUIControls.Materials.matBumpB = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Materials.matBumpB.val, -100.0, 100.0, "")
					gui.units("mm", 100)
					
					gui.newline("(C) Base Value / Offset")
					IndigoUIControls.Materials.matBumpC = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Materials.matBumpC.val, -100.0, 100.0, "")
					gui.units("mm", 100)
			
				if IndigoUIControls.Materials.currentMaterial.bump.typeShader():
					gui.newline("Indigo Shader")
					IndigoUIControls.Materials.matBumpShader=Blender.Draw.Menu(shader, IndigoEvents.evtNoEvt, gui.x, gui.y, 250, 18, IndigoUIControls.Materials.matBumpShader.val, "Select Bump Shader")
		
		if IndigoTypes.Materials.isT(IndigoUIControls.Materials.matType, 'PHONG'):
			gui.subtitle("       Exponent")
			IndigoUIControls.Materials.currentMaterial.exponent.enabled = Blender.Draw.Toggle(IndigoUIControls.PlusMinusToggleText(IndigoUIControls.Materials.currentMaterial.exponent.enabled), IndigoEvents.evtNoEvt, 16, gui.y + 6, 20, 18, IndigoUIControls.Materials.currentMaterial.exponent.enabled.val, "enable exponent channel")
		
			if IndigoUIControls.Materials.currentMaterial.exponent.enabled.val:
				gui.newline("Type")
				IndigoUIControls.Materials.currentMaterial.exponent.type = Blender.Draw.Menu(IndigoTypes.Channel.menuString(limit=['TEXTURE','SHADER']), IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Materials.currentMaterial.exponent.type.val, "select the desired channel type")

				if (IndigoUIControls.Materials.currentMaterial.exponent.typeShader()) or (IndigoUIControls.Materials.currentMaterial.exponent.typeTexture()):
					gui.newline("UV Texture")
					IndigoUIControls.Materials.matExponentUVset = Blender.Draw.String("", IndigoEvents.evtNoEvt, gui.x, gui.y, 95, 18, IndigoUIControls.Materials.matExponentUVset.val, 20, "exponent texture UV-set")

				if IndigoUIControls.Materials.currentMaterial.exponent.typeTexture():
					gui.newline("Filename")
					IndigoUIControls.Materials.matExponentPath = Blender.Draw.String("", IndigoEvents.evtNoEvt, gui.x, gui.y, 250, 18, IndigoUIControls.Materials.matExponentPath.val, 250, "texture filename (referencing Blender UV co-ords)")
					Blender.Draw.Button("...", IndigoEvents.Materials.evtMatExponentPath, gui.x + 255, gui.y, 20, 18, "open file selector")

					gui.newline("Gamma")
					IndigoUIControls.Materials.matExponentExponent = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Materials.matExponentExponent.val, 0.0, 10.0, "texture gammma")
					gui.units("dimensionless", 100)
					
					gui.newline("(A) Gradient")
					IndigoUIControls.Materials.matExponentA = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Materials.matExponentA.val, -1.0, 1.0, "")
					
					gui.newline("(B) Intensity")
					IndigoUIControls.Materials.matExponentB = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Materials.matExponentB.val, 0.0, 1e6, "")
					
					gui.newline("(C} Base Value / Offset")
					IndigoUIControls.Materials.matExponentC = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Materials.matExponentC.val, -1e6, 1e6, "")
			
				if IndigoUIControls.Materials.currentMaterial.exponent.typeShader():
					gui.newline("Indigo Shader")
					IndigoUIControls.Materials.matExponentShader=Blender.Draw.Menu(shader, IndigoEvents.evtNoEvt, gui.x, gui.y, 250, 18, IndigoUIControls.Materials.matExponentShader.val, "Select Exponent Shader")
		
		##	
		##	displacement channel
		##
		if (IndigoTypes.Materials.isT(IndigoUIControls.Materials.matType, 'DIFFUSE') and not IndigoUIControls.Materials.matDiffTrans.val) \
		or  IndigoTypes.Materials.inT(IndigoUIControls.Materials.matType, ['PHONG', 'SPECULAR', 'ORENNAYAR']):
			gui.subtitle("       Displacement")
			IndigoUIControls.Materials.currentMaterial.displacement.enabled = Blender.Draw.Toggle(IndigoUIControls.PlusMinusToggleText(IndigoUIControls.Materials.currentMaterial.displacement.enabled), IndigoEvents.evtNoEvt, 16, gui.y + 6, 20, 18, IndigoUIControls.Materials.currentMaterial.displacement.enabled.val, "enable displacement channel")

			if IndigoUIControls.Materials.currentMaterial.displacement.enabled.val:
				gui.newline("Type")
				IndigoUIControls.Materials.currentMaterial.displacement.type = Blender.Draw.Menu(IndigoTypes.Channel.menuString(limit=['TEXTURE','SHADER']), IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Materials.currentMaterial.displacement.type.val, "select the desired channel type")

				if (IndigoUIControls.Materials.currentMaterial.displacement.typeShader()) or (IndigoUIControls.Materials.currentMaterial.displacement.typeTexture()):
					gui.newline("UV Texture")
					IndigoUIControls.Materials.matDisplaceUVSet = Blender.Draw.String("", IndigoEvents.evtNoEvt, gui.x, gui.y, 95, 18, IndigoUIControls.Materials.matDisplaceUVSet.val, 20, "displacement texture UV-set")

				if IndigoUIControls.Materials.currentMaterial.displacement.typeTexture():
					gui.newline("Filename")
					IndigoUIControls.Materials.matDisplacePath = Blender.Draw.String("", IndigoEvents.evtNoEvt, gui.x, gui.y, 250, 18, IndigoUIControls.Materials.matDisplacePath.val, 250, "texture filename (referencing Blender UV co-ords)")
					Blender.Draw.Button("...", IndigoEvents.Materials.evtDisplaceImage, gui.x + 255, gui.y, 20, 18, "open file selector")

					gui.newline("Gamma")
					IndigoUIControls.Materials.matDisplaceExp = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Materials.matDisplaceExp.val, 0.0, 10.0, "texture gammma")
					gui.units("dimensionless", 100)
					
					gui.newline("(A) Gradient")
					IndigoUIControls.Materials.matDisplaceA = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Materials.matDisplaceA.val, -1.0, 1.0, "")
					
					gui.newline("(B) Height")
					IndigoUIControls.Materials.matDisplaceB = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Materials.matDisplaceB.val, -1.0, 1.0, "")
					gui.units("metres", 100)
					
					gui.newline("(C) Base Value / Offset")
					IndigoUIControls.Materials.matDisplaceC = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Materials.matDisplaceC.val, -1.0, 1.0, "")
					gui.units("metres", 100)

				if IndigoUIControls.Materials.currentMaterial.displacement.typeShader():
					gui.newline("Indigo Shader")
					IndigoUIControls.Materials.matDisplaceShader=Blender.Draw.Menu(shader, IndigoEvents.evtNoEvt, gui.x, gui.y, 250, 18, IndigoUIControls.Materials.matDisplaceShader.val, "Select Displace Shader")
		
		
		##	
		##	absorption layer transmittance channel
		##
		if IndigoTypes.Materials.inT(IndigoUIControls.Materials.matType, ['SPECULAR']):
			gui.subtitle("       Absorption Layer Transmittance")
			IndigoUIControls.Materials.currentMaterial.alt.enabled = Blender.Draw.Toggle(IndigoUIControls.PlusMinusToggleText(IndigoUIControls.Materials.currentMaterial.alt.enabled), IndigoEvents.evtNoEvt, 16, gui.y + 6, 20, 18, IndigoUIControls.Materials.currentMaterial.alt.enabled.val, "enable absorption layer transmittance channel")

			if IndigoUIControls.Materials.currentMaterial.alt.enabled.val:
				gui.newline("Type")
				IndigoUIControls.Materials.currentMaterial.alt.type = Blender.Draw.Menu(IndigoTypes.Channel.menuString(limit=['TEXTURE','SHADER']), IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Materials.currentMaterial.alt.type.val, "select the desired channel type")

				if (IndigoUIControls.Materials.currentMaterial.alt.typeShader()) or (IndigoUIControls.Materials.currentMaterial.alt.typeTexture()):
					gui.newline("UV Texture")
					IndigoUIControls.Materials.matALTUVSet = Blender.Draw.String("", IndigoEvents.evtNoEvt, gui.x, gui.y, 95, 18, IndigoUIControls.Materials.matALTUVSet.val, 20, "absorption layer transmittance texture UV-set")

				if IndigoUIControls.Materials.currentMaterial.alt.typeTexture():
					gui.newline("Filename")
					IndigoUIControls.Materials.matALTPath = Blender.Draw.String("", IndigoEvents.evtNoEvt, gui.x, gui.y, 250, 18, IndigoUIControls.Materials.matALTPath.val, 250, "texture filename (referencing Blender UV co-ords)")
					Blender.Draw.Button("...", IndigoEvents.Materials.evtALTImage, gui.x + 255, gui.y, 20, 18, "open file selector")

					gui.newline("Gamma")
					IndigoUIControls.Materials.matALTExp = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Materials.matALTExp.val, 0.0, 10.0, "texture gammma")
					gui.units("dimensionless", 100)
					
					gui.newline("(A) Gradient")
					IndigoUIControls.Materials.matALTA = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Materials.matALTA.val, -1.0, 1.0, "")
					
					gui.newline("(B) Scale")
					IndigoUIControls.Materials.matALTB = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Materials.matALTB.val, -1.0, 1.0, "")
					gui.units("metres", 100)
					
					gui.newline("(C) Base Value / Offset")
					IndigoUIControls.Materials.matALTC = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Materials.matALTC.val, -1.0, 1.0, "")
					gui.units("metres", 100)

				if IndigoUIControls.Materials.currentMaterial.alt.typeShader():
					gui.newline("Indigo Shader")
					IndigoUIControls.Materials.matALTShader=Blender.Draw.Menu(shader, IndigoEvents.evtNoEvt, gui.x, gui.y, 250, 18, IndigoUIControls.Materials.matALTShader.val, "Select absorption layer transmittance Shader")
		
		
		if IndigoTypes.Materials.isT(IndigoUIControls.Materials.matType, 'INCLUDE'):
			gui.subtitle("External material")
			
			#if IndigoTypes.MaterialInclude.isT(IndigoUIControls.Materials.matIncludeType.val, 'LOCAL'):
			gui.newline("Filename")
			IndigoUIControls.Materials.matInclude = Blender.Draw.String("", IndigoEvents.evtNoEvt, gui.x, gui.y, 250, 18, IndigoUIControls.Materials.matInclude.val, 250, "include filename")
			Blender.Draw.Button("...", IndigoEvents.Materials.evtMatInclude, gui.x+255, gui.y, 20, 18, "open file selector")
			
			if MATDBLIB_AVAILABLE:
				
				gui.subtitle('       Materials Database')
				IndigoUIControls.Materials.matIncludeUseDB = Blender.Draw.Toggle(
					IndigoUIControls.PlusMinusToggleText(IndigoUIControls.Materials.matIncludeUseDB),
					IndigoEvents.evtNoEvt,
					16, gui.y + 6,
					20, 18,
					IndigoUIControls.Materials.matIncludeUseDB.val,
					"Material DB downloads"
				)
				
				if IndigoUIControls.Materials.matIncludeUseDB.val:
					gui.newline('Downloads')
					IndigoUIControls.Materials.matDBInclude = Blender.Draw.Menu(
						utils.menuStringFromList('Downloads', IndigoPaths.getContents('matdbDownloads', 'pigm')),
						IndigoEvents.Materials.evtMatDBChoose,
						gui.x, gui.y,
						250, 18,
						IndigoUIControls.Materials.matDBInclude.val,
						"Choose file from Material DB downloads"
					)
					
					gui.newline('DB Search')
					IndigoUIControls.Materials.matDatabaseSearchString = Blender.Draw.String("Search term: ", IndigoEvents.evtNoEvt, gui.x, gui.y, 250, 18, IndigoUIControls.Materials.matDatabaseSearchString.val, 250, 'Search string for materials database')
					Blender.Draw.Button("Go!", IndigoEvents.Materials.evtMatDBSearch, gui.x+255, gui.y, 20, 18, "Search database")
					if len(IndigoUIControls.Materials.matDatabaseResultSet) > 0:
						# show result names and download buttons
						for r in IndigoUIControls.Materials.matDatabaseResultSet:
							matname = r['name']
							gui.newline('  %s' % matname)
							
							if 'preview_image' in r.keys():
								# 35x35px preview
								if r['name'] not in IndigoGlobals.mat_preview_cache.keys():
									IndigoGlobals.mat_preview_cache[r['name']] = Blender.Image.Load(r['preview_image'])
								
								IndigoGlobals.mat_preview_cache[r['name']].reload()
								IndigoGlobals.mat_preview_cache[r['name']].glLoad()
								drawImage(IndigoGlobals.mat_preview_cache[r['name']], gui.x-90, gui.y-68, 1.0, 1.0)
							else:
								if r['name'] not in IndigoGlobals.mat_preview_cache.keys():
									r['preview_image'] = r.download_image( IndigoPaths.get('matdbPreviews') )
							
							if not r.is_downloaded( IndigoPaths.get('matdbDownloads') ):
								if 'pc' in r.keys():
									gui.units('Downloading... %i%%' % r['pc'], gui.x)
								else:
									gui.y-=18
									Blender.Draw.Button('Download', IndigoEvents.Materials.evtMatDBDownload+int(r['id']), gui.x, gui.y, 65, 18, "")
									sz = int(r['size'])/1024
									if sz<1: sz = '<1'
									else: sz='%i'%sz
									gui.units('(%sk)'%sz, 65)
							else:
								gui.y-=18
								gui.units('Downloaded OK')
							
							# add extra vertical space for preview image
							gui.y-=58
	
	drawExportButtons(gui)
	return gui.y


### ================================================================
### materialEvent(evt, val)
###		material GUI event routine
### ----------------------------------------------------------------
def materialEvent(evt, val):
	obj = Blender.Scene.GetCurrent().objects.active
	if(obj != None):
		try:
			try:
				if obj.getType() == "Text":
					cu=Blender.Curve.Get(obj.getData().getName())
					mat0=cu.materials[0]
				else:
					mat0 = (obj.getData(0,1).materials)[obj.activeMaterial - 1]
			except IndexError:
				mat0 = None
		except AttributeError:
			mat0 = None
	else: mat0 = None
	
	if obj != None and ((IndigoGlobals.activeObject != obj) or (IndigoGlobals.activeObjectMat0 != mat0)):
		IndigoGlobals.activeObject = obj
		IndigoGlobals.activeObjectMat0 = mat0
		if (mat0 != None):
			mats = Blender.Material.Get()
			IndigoUIControls.Materials.matsMenu.val = 0
			matindex = IndigoUIControls.Materials.matsMenu.val
			for m in mats:
				if m.getName() == mat0.getName():
					matindex = mats.index(m)+1
					IndigoGlobals.activeMaterial = m
					setCurrentMaterial(IndigoGlobals.activeMaterial)
			IndigoUIControls.Materials.matsMenu.val = matindex
			readWriteMaterial(IndigoGlobals.activeMaterial, 0)
			#drawGUI()
			Blender.Draw.Redraw(1)


### ================================================================
### materialButtonEvt(evt)
###		material GUI button-event routine
### ----------------------------------------------------------------
def materialButtonEvt(evt):
	
	if evt == IndigoEvents.Materials.evtMatsMenu:
		mats = Blender.Material.Get()
		IndigoGlobals.activeMaterial = mats[IndigoUIControls.Materials.matsMenu.val-1]
		setCurrentMaterial(IndigoGlobals.activeMaterial)
		readWriteMaterial(IndigoGlobals.activeMaterial, 0)
		
	if evt == IndigoEvents.Materials.evtMatType:
		IndigoUIControls.Materials.matType = IndigoUIControls.Materials.matTypeMenu.val
		
	if evt == IndigoEvents.Materials.evtMatAbsPicker:
		IndigoUIControls.Materials.matAbsR.val = 1.0-IndigoUIControls.Materials.matAbs.val[0]
		IndigoUIControls.Materials.matAbsG.val = 1.0-IndigoUIControls.Materials.matAbs.val[1]
		IndigoUIControls.Materials.matAbsB.val = 1.0-IndigoUIControls.Materials.matAbs.val[2]
		
	if evt == IndigoEvents.Materials.evtMatRgbPicker:
		IndigoUIControls.Materials.matRgbR.val = IndigoUIControls.Materials.matRgb.val[0]
		IndigoUIControls.Materials.matRgbG.val = IndigoUIControls.Materials.matRgb.val[1]
		IndigoUIControls.Materials.matRgbB.val = IndigoUIControls.Materials.matRgb.val[2]
		
	if evt == IndigoEvents.Materials.evtMatEmittRgbPicker:
		IndigoUIControls.Materials.matEmittRgbR.val=IndigoUIControls.Materials.matEmittRgb.val[0]
		IndigoUIControls.Materials.matEmittRgbG.val=IndigoUIControls.Materials.matEmittRgb.val[1]
		IndigoUIControls.Materials.matEmittRgbB.val=IndigoUIControls.Materials.matEmittRgb.val[2]
		
	if evt == IndigoEvents.Materials.evtMatRgbPicker_s:
		IndigoUIControls.Materials.matRgbR_s.val = IndigoUIControls.Materials.matRgb_s.val[0]
		IndigoUIControls.Materials.matRgbG_s.val = IndigoUIControls.Materials.matRgb_s.val[1]
		IndigoUIControls.Materials.matRgbB_s.val = IndigoUIControls.Materials.matRgb_s.val[2]
		
	if evt == IndigoEvents.Materials.evtMatSssRgbPicker:
		IndigoUIControls.Materials.matSssR.val = IndigoUIControls.Materials.matSssRgb.val[0]
		IndigoUIControls.Materials.matSssG.val = IndigoUIControls.Materials.matSssRgb.val[1]
		IndigoUIControls.Materials.matSssB.val = IndigoUIControls.Materials.matSssRgb.val[2]
		
	if (evt == IndigoEvents.Materials.evtMatBlendMatA) or (evt == IndigoEvents.Materials.evtMatBlendMatB):
		mats = Blender.Material.Get()
		matsmenustr = "materials:%t|"
		for m in mats:
			matsmenustr += m.getName()+"|"
		matsmenustr += "Default|null"
		i = Blender.Draw.PupMenu(matsmenustr,20)
		if i > 0:
			if (i <= len(mats)):
				matname = mats[i-1].getName()
			else:
				if (i == len(mats)+1):
					matname = "Default"
				else:
					matname = "null"
			if (evt == IndigoEvents.Materials.evtMatBlendMatA):
				IndigoUIControls.Materials.matBlendMatA.val = matname
			if (evt == IndigoEvents.Materials.evtMatBlendMatB):
				IndigoUIControls.Materials.matBlendMatB.val = matname
				
	if evt == IndigoEvents.Materials.evtMatEmittIESPath:
		Blender.Window.FileSelector(matEmittIESPathCB,'select IES file')
		
	if evt == IndigoEvents.Materials.evtMatAlbedoPath:
		Blender.Window.FileSelector(matAlbedoPathCB, 'select texture')
		
	if evt == IndigoEvents.Materials.evtMatBumpPath:
		Blender.Window.FileSelector(matBumpPathCB, 'select texture')
		
	if evt == IndigoEvents.Materials.evtMatExponentPath:
		Blender.Window.FileSelector(matExponentPathCB, 'select texture')
		
	if evt == IndigoEvents.Materials.evtMatBlendPath:
		Blender.Window.FileSelector(matBlendPathCB, 'select texture')
		
	if evt == IndigoEvents.Materials.evtMatInclude:
		Blender.Window.FileSelector(matIncludeCB, 'select include-file')
		
	if evt == IndigoEvents.Materials.evtDisplaceImage:
		Blender.Window.FileSelector(matDisplaceCB, 'select image')
		
	if evt == IndigoEvents.Materials.evtALTImage:
		Blender.Window.FileSelector(matALTCB, 'select image')
		
	if evt == IndigoEvents.Materials.evtMatPreview:
		if not IndigoTypes.Materials.inT(IndigoUIControls.Materials.matType, ['INCLUDE', 'EXITPORTAL']):
			exportMatPreview(IndigoGlobals.activeMaterial)
		else:
			Blender.Draw.PupMenu("Can't preview exit portals or external materials, sorry%t|OK%x1")
	
	if evt == IndigoEvents.Materials.evtMatCopy:
		if (IndigoGlobals.activeMaterial != None):
			if 'indigo' not in IndigoGlobals.activeMaterial.properties.keys():
				convertMaterial(IndigoGlobals.activeMaterial)
			
			readWriteMaterial(IndigoGlobals.activeMaterial, 1)
			IndigoUIControls.Materials.matClipboard = IndigoGlobals.activeMaterial.properties['indigo'].convert_to_pyobject()
			
				
	if (evt == IndigoEvents.Materials.evtMatPaste) and (IndigoUIControls.Materials.matClipboard != None):
		if (IndigoGlobals.activeMaterial != None):
			if 'indigo' not in IndigoGlobals.activeMaterial.properties.keys():
				convertMaterial(IndigoGlobals.activeMaterial)
			
			IndigoGlobals.activeMaterial.properties['indigo'] = IndigoUIControls.Materials.matClipboard
			readWriteMaterial(IndigoGlobals.activeMaterial, 0)
			
				
	if evt == IndigoEvents.Materials.evtMatConvert:
		if (IndigoGlobals.activeMaterial != None):
			convertMaterial(IndigoGlobals.activeMaterial);
			readWriteMaterial(IndigoGlobals.activeMaterial, 0);
			
	if evt == IndigoEvents.Materials.evtMatSave:
		if (IndigoGlobals.activeMaterial != None):
			Blender.Window.FileSelector(matSaveCB, 'select igm file')
			
	if evt == IndigoEvents.Materials.evtMatDBChoose:
		files  = IndigoPaths.getContents('matdbDownloads', 'pigm')
		chosen = IndigoUIControls.Materials.matDBInclude.val
		IndigoUIControls.Materials.matInclude.val = os.path.join(IndigoPaths.get('matdbDownloads'), files[chosen])
		
	if evt == IndigoEvents.Materials.evtMatDBSearch:
		IndigoUIControls.Materials.matDatabaseResultSet = []
		
		searchterm = IndigoUIControls.Materials.matDatabaseSearchString.val.strip()
		
		if len(searchterm) < 1:
			Blender.Draw.PupMenu('Search term cannot be blank!%t|')
		else:
			Blender.Draw.PupMenu('Note:|\
			 User Interface will freeze while|\
			 results and preview images are downloaded|\
			 and also while pigm files are downloaded.|\
			 See console for progress.')
			
			pb = exportProgressBar(2)
			
			pb.next('Searching database')
			
			q = matdblib.query( matdblib.connector() )
			rs = q.fulltext( searchterm )
			
			IndigoUIControls.Materials.matDatabaseResultSet = rs
			
			pb.finished()
		
	if evt >= IndigoEvents.Materials.evtMatDBDownload:
		matID = evt - IndigoEvents.Materials.evtMatDBDownload
		#print(evt, matID)
		
		for r in IndigoUIControls.Materials.matDatabaseResultSet:
			if int(r['id']) == matID:
				# download the material in a background thread
				#threading.Thread(
				#	target=r.download_pigm,
				#	kwargs={
				#		'folder': IndigoPaths.get('matdbDownloads'),
				#		'cb': Blender.Draw.Redraw,
				#		'cba': (1,)
				#	}
				#).start()
				r.download_pigm(folder=IndigoPaths.get('matdbDownloads')) #, cb=Blender.Draw.Redraw, cba=(1,))
	
	readWriteMaterial(IndigoGlobals.activeMaterial, 1)
	Blender.Draw.Redraw(1)

# New name based on old with a different extension
def newFName(ext):
	return Blender.Get('filename')[: -len(Blender.Get('filename').split('.', -1)[-1]) ] + ext

def IndigoBoolean(val):
	if val != 0:
		val = "true"
	else:
		val = "false"
	return val

def exportMeshBinary(mesh, meshname, mesh_save_dir, current_mesh_index, objmatrix, file, animation_mode, flyby_mode, subdiv = 0, subdiv_values={}):
	
	verbose = False
	
	#if verbose: print("exportMeshBinary()")
	if verbose: start_time = Blender.sys.time()
	
	# Use IndigoMeshDH because it writes correct data on 64bit OSs
	igmesh = indigo_mesh.IndigoMeshDH()
	
	filename = utils.makeSafeFilename(meshname) + "_%d.igmesh" % current_mesh_index
	path = os.path.join(mesh_save_dir, filename)
	
	# If we're doing a flyby, and the static mesh file exists, don't re-write it.
	if animation_mode and flyby_mode and os.path.exists(path):
		print('\tRe-using static mesh file %s'%path)
		return
	
	# Write used materials
	if len(mesh.materials) > 0 and not IndigoUIControls.Renderer.ClayRender.val:
		for mat in mesh.materials:
			use_mat_name = ""
			if mat != None:
				use_mat_name = mat.name
				if 'indigo' not in mat.properties.keys():
					convertMaterial(mat)
					
				d = mat.properties['indigo'].convert_to_pyobject()
				mType = readValue(d, 'type', 0)
				if (mType == IndigoTypes.Materials._('INCLUDE')):
					if verbose: print "\tMesh use external material"
					fname= checkAndUnpackPigm(readValue(d,'include',''))
					if (os.path.isfile(fname)):
						mTag = re.compile('\s*<material>')
						mName = re.compile("\s*<name>([\w\s]*)</name>")
						#print mName
						f=open(fname,'r')
						line=f.readline()
						insideMat = 0
						while line != "":
							if mTag.match(line):
								insideMat = 1
							m=mName.match(line)
							if (insideMat and m):
								use_mat_name=m.group(1)
							line=f.readline()
						f.close()
			else:
				use_mat_name = "Default"
			
			igmesh.addUsedMaterial(use_mat_name)
	else:
		igmesh.addUsedMaterial("Default")
	
	
	pos = objmatrix.translationPart()
	scl = objmatrix.scalePart()
	# the blender euler rotation is wrong in some cases and don't match indigo rotation so here a new one:
	rot = Blender.Mathutils.Matrix(objmatrix[0]/scl[0],objmatrix[1]/scl[1],objmatrix[2]/scl[2])
	
	mat = None
	if (abs(scl[0]-scl[1])>1e-4) or (abs(scl[0]-scl[2])>1e-4):
		mat = Blender.Mathutils.Matrix()
		for v in range(3):
			for z in range(3):
				mat[z][v] = objmatrix[z][v]
		rot = Blender.Mathutils.Matrix()
		scalefact = 1.0
	else:
		scalefact = (scl[0] + scl[1] + scl[2]) / 3
	
	if mat:
		transpose_m = mat.copy()
		transpose_m.invert().transpose()
	
	# Write vertices
	for v in mesh.verts:
		if mat and mesh.users == 1:
			pos = Blender.Mathutils.Vector(v.co) * mat
			nor = Blender.Mathutils.Vector(v.no) * transpose_m
		else:
			pos = v.co
			nor = v.no
		igmesh.addVertPosition(pos[0], pos[1], pos[2])
		igmesh.addVertNormal(nor[0], nor[1], nor[2])

	
	if verbose: print("getUVLayerNames(): " + str(mesh.getUVLayerNames()))
	
	# Build up a list of arrays of UV coords.
	# Each layer gets its own array.
	# These layers will then be interleaved and saved in the igmesh.
	
	if mesh.faceUV:
		layer_uv_arrays = [] # A list of UV arrays, one array per layer.
		
		for name in mesh.getUVLayerNames(): # For Each UV Layer
			# Set current active layer.
			mesh.activeUVLayer = name
			
			layer_uv_arrays.append(array.array('f')) # Construct an array to hold the UVs, add to list
			uv_array = layer_uv_arrays[len(layer_uv_arrays) - 1] # Get last elem of list.
			face_index = 0
			for face in mesh.faces: # For each face in the mesh
				for vert_index in range(len(face.verts)): # For each vert in the face
					uv_array.append(face.uv[vert_index][0]) # Add U
					uv_array.append(face.uv[vert_index][1]) # Add V
		
		# Add UVs to IGMESH
		num_uv_layers = len(mesh.getUVLayerNames())
		
		if num_uv_layers > 0:
			num_uv_pairs = len(layer_uv_arrays[0]) / 2
			for i in range(num_uv_pairs): # For each UV pair
				for layer_i in range(num_uv_layers): # For each UV layer
					igmesh.addUVPair(layer_uv_arrays[layer_i][i*2], layer_uv_arrays[layer_i][i*2 + 1])
					
		
		igmesh.setNumUVMappings(num_uv_layers)
		
		# Write UV exports
		uv_index = 0
		for name in mesh.getUVLayerNames():
			igmesh.addUVSetExposition(check_uvtext(name), uv_index)
			uv_index += 1
	
	else: # Else if no UVs
		igmesh.setNumUVMappings(0)
		
	
	# Write triangles
	uv_index = 0
	face_index = 0
	for face in mesh.faces:
		igmesh.addTriangle(
			face.verts[0].index, face.verts[1].index, face.verts[2].index,
			uv_index, uv_index+1, uv_index+2,
			face.mat
			)
		
		if len(face.verts) > 3:
			igmesh.addTriangle(
				face.verts[0].index, face.verts[2].index, face.verts[3].index,
				uv_index, uv_index+2, uv_index+3,
				face.mat
				)
		
		face_index += 1
		uv_index += len(face.verts)
	
	if verbose: print("\tBuilding IGMESH object Done.  (Took %f s)" % (Blender.sys.time() - start_time))
	
	# Write IGMESH to disk.
	#if verbose:
	print("\tSaving igmesh to '" + path + "...")
	if verbose: start_time = Blender.sys.time()
	
	igmesh.writeToDisk(path)
	
	if verbose: print("\tDone.  (Took %f s)" % (Blender.sys.time() - start_time))

def exportMeshElement(meshname, mesh, current_mesh_index, file, animation_mode, flyby_mode):
	if animation_mode and flyby_mode:
		filename = 'static_meshes' + os.sep + utils.makeSafeFilename(meshname) + "_%d.igmesh" % current_mesh_index
	else:
		filename = ('%05i'%Blender.Get('curframe')) + os.sep + utils.makeSafeFilename(meshname) + "_%d.igmesh" % current_mesh_index
	# Write the Mesh XML element, this will refer to the IGMESH stored on disk.
	
	try:
		d=mesh.properties['subdivision'].convert_to_pyobject()
	except:
		d = {}
	
	file.write("\t<mesh>\n");
	file.write("\t\t<name>" + meshname + "</name>\n");
	file.write("\t\t<normal_smoothing>" + IndigoBoolean(mesh.faces[0].smooth) + "</normal_smoothing>\n");
	
	if meshname in d:
		subdiv_props=d[meshname]
		file.write("\t\t<max_num_subdivisions>%d</max_num_subdivisions>\n"%subdiv_props[0])
		file.write("\t\t<subdivide_pixel_threshold>%f</subdivide_pixel_threshold>\n"%subdiv_props[1])
		file.write("\t\t<subdivide_curvature_threshold>%f</subdivide_curvature_threshold>\n"%float(subdiv_props[2]))
		file.write("\t\t<subdivision_smoothing>%s</subdivision_smoothing>\n"%(IndigoBoolean(subdiv_props[3])))
	
	file.write("\t\t<external>\n");
	file.write("\t\t\t<path>" + filename + "</path>\n");
	file.write("\t\t</external>\n");
	file.write("\t</mesh>\n");

def exportParticleModel(matrix, meshname, mesh_string, file, keyframes=None):
	ws = IndigoUIControls.Environment.WorldScale.val*(10**IndigoUIControls.Environment.WorldScaleExp.val)
	pos = matrix.translationPart()
	rot = matrix.rotationPart()
	
	file.write("\t<model>\n")
	
	if type(keyframes) is list:
		scln = matrix.scalePart().length
		file.write("\t\t<scale>%s</scale>\n"%(scln*ws))
		for kf in keyframes:
			writeKeyFrame(**kf)
	else:
		file.write("\t\t<scale>%s</scale>\n"%ws) # The scale is encoded into rot matrix
		file.write("\t\t<pos>%f %f %f</pos>\n" %(pos[0]*ws, pos[1]*ws, pos[2]*ws))
		file.write("\t\t<rotation>\n")
		file.write("\t\t\t<matrix>\n")
		file.write("\t\t\t\t")
		for v in range(3):
				for z in range(3):
					file.write("%f " % (rot[z][v]))
		file.write("\n")
		file.write("\t\t\t</matrix>\n")
		file.write("\t\t</rotation>\n")
	
	file.write("\t\t<mesh_name>%s</mesh_name>\n" %(meshname))
	file.write(mesh_string)
	file.write("\t</model>\n\n")
	return True

def readValue(dict, name, default):
	try:
		r = dict[name]
	except KeyError:
		r = default
	return r

def writeKeyFrame(file, time, posn=None, rotn=None, matrix=None):
	t, mbframe = time
	file.write("\t\t<keyframe>\n")
	file.write("\t\t\t<time>%f</time> <!-- frame %f -->\n"%(t, mbframe))
	
	if matrix is not None:
		posn = matrix.translationPart()
		rotn = matrix.rotationPart().toEuler()
	
	if posn is not None:
		file.write("\t\t\t<pos>%f %f %f</pos>\n"%tuple(posn))
	if rotn is not None:
		rotnquat = rotn.toQuat()
		# negate angle to change handedness
		axis, angle = rotnquat.axis, -math.radians(rotnquat.angle)
		file.write("\t\t\t<rotation_quaternion> <!-- %f %f %f -->\n" % tuple(rotn))
		file.write("\t\t\t\t<axis>%f %f %f</axis>\n"%(axis.x,axis.y,axis.z))
		file.write("\t\t\t\t<angle>%f</angle>\n"%angle)
		file.write("\t\t\t</rotation_quaternion>\n")
	
	file.write("\t\t</keyframe>\n")

def exportObject(obj, exportedobjlist, file, mat=0, particles_file = None, write_particles = True):
	
	ws = IndigoUIControls.Environment.WorldScale.val*(10**IndigoUIControls.Environment.WorldScaleExp.val)
	
	matrix = obj.getMatrix()
	objectname = obj.getName()
	objecttype = obj.getType()
	objectdup = obj.enableDupGroup# or obj.enableDupVerts
	
	if not obj.enableDupVerts and ((objecttype == "Mesh") or (objecttype == "Curve") or (objecttype == "Text")):
		meshname=obj.getData(name_only=1)
		if(objecttype == "Curve"):
			meshname +='_cu'
		imeshname = meshname.replace("<","-").replace(">","-")
		
		pObj=obj.getParent()
		if pObj != None:
			if pObj.enableDupGroup or pObj.enableDupVerts or pObj.enableDupFaces:
				if len(pObj.DupObjects) > 0:
					print "%d Instances of %s found"%(len(pObj.DupObjects), objectname)
					
					try:
						matlist=obj.getData().materials
					except:
						matlist=[]
					
					emission_tag = ''
					if len(matlist) > 0:
						emitter_idx=0
						for mat_i in matlist:
							if mat_i != None:
								try:
									d = mat_i.properties['indigo'].convert_to_pyobject()
								except KeyError:
									d = {}
									
							if readValue(d,'emitter', 0):
							#if matIsEmitter(mat_i):
								emission_tag+="\t\t<emission_scale>\n"
								emission_tag+="\t\t\t<material_name>%s</material_name>\n"%matlist[emitter_idx].getName()
								emission_tag+="\t\t\t<measure>luminous_flux</measure>"
								emission_tag+="\t\t\t<value>%f</value>\n"%readValue(d,'emitter.scale_value',0)
								emission_tag+="\t\t</emission_scale>\n"
								
								if readValue(d,'emitter.ies',0) and readValue(d,'emitter.ies_path','') != '':
									emission_tag+="\t\t<ies_profile>\n"
									emission_tag+="\t\t\t<material_name>%s</material_name>\n"%matlists[emitter_idx].getName()
									emission_tag+="\t\t\t<path>%s</path>\n"%readValue(d,'emitter.ies_path','')
									emission_tag+="\t\t</ies_profile>\n"
							emitter_idx+=1
							
					dup_instances_file = file
					
					for o, m in pObj.DupObjects:
						exportParticleModel(m, imeshname, emission_tag, dup_instances_file)
					
					print "%d Instances of %s exported"%(len(pObj.DupObjects), objectname)
		
		render_emitter = True
		psystems = obj.getParticleSystems()
		if write_particles:
			p_count = 0
			
			p_moblur = IndigoUIControls.Camera.PrtMoBlur.val
			
			startF = Blender.Get('curframe')
			
			# Get MoBlur frame data for t=0.0 and t=1.0 only
			keyframe_times = getMoBlurTiming(startF) #, first_and_last_only=True)
			
			def export_particle(dup_obj, i, psys, loc, rot, scale, particle_file):
				
				def get_particle_matrix_for_this_frame(dobj_matrix, loc, rot, scale):
					dobj_translation_vec = dobj_matrix.translationPart()
					dobj_rotation_scale_mat = dobj_matrix.rotationPart()	# This gets a 3D submatrix with the rotation AND scale parts.
					
					part_rotation_quat = Blender.Mathutils.Quaternion(rot)
					part_rotation_mat = part_rotation_quat.toMatrix()
					rotation_scale_mat =  dobj_rotation_scale_mat * part_rotation_mat * scale
					
					# Skip this particle if it has zero size
					#if rotation_scale_mat.determinant() == 0:
					#	return False
					
					# If dup_obj is translated, the particles are translated by the same amount but
					# the direction is rotated by the particle rotation. If dup_obj is rotated, that rotation
					# does not affect the translation. I know it's a bit odd, but that's the way Blender does it
					# and so that's why the order of the matrix multiplications is like this.
					translation_vec = Blender.Mathutils.Vector(loc) + part_rotation_quat*dobj_translation_vec
					translation_mat = Blender.Mathutils.TranslationMatrix(translation_vec)
					rotation_scale_mat.resize4x4()
					# Translation must be last because of the way the rotations and translations are encoded in 4D matrices.
					#combined_matrix = scale_matrix*rotation_mat*translation_mat
					combined_matrix = rotation_scale_mat*translation_mat
					#print "combined_matrix = ", combined_matrix
					return combined_matrix
				
				start_matrix = get_particle_matrix_for_this_frame(dup_obj.getMatrix(), loc, rot, scale)
				
				keyframes = None
				if IndigoUIControls.Camera.PrtMoBlur.val:
					
					if len(keyframe_times) > 1:
						keyframes = []
						
						times_written = set()
						# in order to reduce the amount of data written, just
						# use the first and last KF
						for t, mbframe in keyframe_times:
							# Prevent writing duplicate keyframes
							# Convert to string to force rounded float comparison
							str_t = '%f'%t 
							if str_t not in times_written:
								times_written.add(str_t)
							else:
								continue
							
							# setting mbframe to int is going to cause inaccurate particle keyframes,
							# but it is the only way to get the MB data from particles
							Blender.Set('curframe', int(mbframe))
							Blender.Scene.GetCurrent().update(1)
							Blender.Draw.Redraw(1)
							
							# This is going to suck for large data sets :(
							try:
								loc = psys.getLoc()[i]
								scale = psys.getSize()[i]
								rot = psys.getRot()[i]
							except:
								# particle has died
								continue
							
							kf_matrix = get_particle_matrix_for_this_frame(dup_obj.getMatrix(), loc, rot, scale)
							
							kf = {
								'file': file,
								'time': (t, mbframe),
								'matrix': kf_matrix
							}
							keyframes.append(kf)
						
						Blender.Set('curframe', startF)
						Blender.Scene.GetCurrent().update(1)
				
				return exportParticleModel(start_matrix, dup_obj.getData(name_only=True), '', particle_file, keyframes)
			
			if particles_file is not None:
				pfile = particles_file
			else:
				pfile = file
			for psys in psystems:
				render_emitter = render_emitter and psys.renderEmitter
				if psys.type == Blender.Particle.TYPE['EMITTER'] or psys.type == Blender.Particle.TYPE['REACTOR']:
					
					if psys.drawAs == Blender.Particle.DRAWAS['OBJECT']:
						dup_obj = psys.duplicateObject
						
						locs = psys.getLoc()
						scales = psys.getSize()
						rots = psys.getRot()
						if(len(locs) != len(scales) or len(locs) != len(rots)):
							print("ERROR: Please bake particle systems before rendering (%i:%i:%i)"%(len(locs), len(scales), len(rots)))
							Blender.Draw.PupMenu("ERROR: Please bake particle systems before rendering%t|OK%x1")
							break
						for i in range(len(locs)):
							p_count += export_particle(dup_obj, i, psys, locs[i], rots[i], scales[i], pfile)
					
					elif psys.drawAs == Blender.Particle.DRAWAS['GROUP']:
						for grp_obj, grp_obj_mtx in obj.DupObjects:
							p_count += exportParticleModel(grp_obj_mtx, grp_obj.getData(name_only=True), '', pfile)
					
					else:
						continue
		
			if p_count > 0: print("\tExported %i particle instances" % p_count)
			
		if len(psystems) > 0 and not render_emitter: return
		
		mesh=getMeshFromObject(obj)
		if not mesh:
			return
		if len(mesh.faces) <= 0:
			return
		if (objecttype == "Curve") or (objecttype == "Text"):
			for f in mesh.faces:
				f.smooth = 1
		if (objecttype == "Text"):
			cu=Blender.Curve.Get(obj.getData().getName())
			mesh.materials = cu.getMaterials()
		if (objecttype == "Curve"):
			mesh.materials = obj.getData().getMaterials()
		emitter = 0
		exitportal = 0
		mat_idx=0
		if len(mesh.materials) > 0:
			for mat_i in mesh.materials:
				if matIsExitPortal(mat_i):
					exitportal = 1
				mat_idx=mat_idx+1
		if not mesh:
			print 'WARNING: ignoring object "%s", because linked mesh "%s" has no faces'%(objectname, meshname)
			return
		if len(mesh.faces) <= 0:
			print 'WARNING: ignoring object "%s", because linked mesh "%s" has no faces'%(objectname, meshname)
			return ""
		file.write("\t<!-- %s -->\n" %(objectname.replace("<","-").replace(">","-")))
		pos = matrix.translationPart()
		scl = matrix.scalePart()
		rot = matrix.rotationPart()
		# process indigo primitives
		if objectname.find(".sphere") != -1:
			bb = obj.getBoundBox()
			mat0=mesh.materials[0]
			radius = abs((bb[0]-bb[1])*scl)/2
			file.write("\t<sphere>\n")
			file.write("\t\t<center>%f %f %f</center>\n"%(pos[0]*ws, pos[1]*ws, pos[2]*ws))
			file.write("\t\t<radius>%f</radius>\n"%radius)
			file.write("\t\t<material_name>%s</material_name>\n"%mat0.name)
			file.write("\t</sphere>\n\n")
		elif objectname.find('.plane') != -1:
			try:
				mat0_name=mesh.materials[0].name
			except:
				mat0_name="Default"
			dist =ws*pos[0]*rot[2][0]+pos[1]*rot[2][1]+pos[2]*rot[2][2]
			file.write("\t<plane>\n")
			file.write("\t\t<normal>%f %f %f</normal>\n"%(rot[2][0], rot[2][1], rot[2][2]))
			file.write("\t\t<dist>%f</dist>\n"%dist)
			file.write("\t\t<material_name>%s</material_name>\n"%mat0_name)
			file.write("\t</plane>\n\n")
		else: #not sphere and not plane
			if (abs(scl[0]-scl[1])>1e-4) or (abs(scl[0]-scl[2])>1e-4):
				scalefact = 1.0
			else:
				scalefact = 1.0
			if exitportal:
				file.write("\t<exit_portal>\n")
			else:
				file.write("\t<model>\n")
			
			ipo = obj.getIpo()
			if ipo != None:
				icux = ipo[Blender.Ipo.OB_LOCX]
				icuy = ipo[Blender.Ipo.OB_LOCY]
				icuz = ipo[Blender.Ipo.OB_LOCZ]
				
				icurx = ipo[Blender.Ipo.OB_ROTX]
				icury = ipo[Blender.Ipo.OB_ROTY]
				icurz = ipo[Blender.Ipo.OB_ROTZ]
			
			startF = Blender.Get('curframe')
			
			file.write("\t\t<scale>%s</scale>\n" %(scalefact*ws))
			file.write("\t\t<rotation>\n")
			file.write("\t\t\t<matrix>\n")
			file.write("\t\t\t\t")
			
			for v in range(3):
				for z in range(3):
					file.write("%f " % (rot[z][v]))
			file.write("\n")
			file.write("\t\t\t</matrix>\n")
			file.write("\t\t</rotation>\n")
			
			write_static_position = True
			if IndigoUIControls.Camera.ObjMoBlur.val and ipo != None and (icux != None and icuy != None and icuz != None):
				
				keyframe_times = getMoBlurTiming(startF)
				
				if len(keyframe_times) > 1:
					write_static_position = False
					
					# from 0.0 to 1.0 in increments of time_step (plus force final time 1.0)
					times_written = set()
					
					#print('camera motion blur keyframe times: %s' % ' '.join(['%f' % i for i in keyframe_times]))
					for t, mbframe in keyframe_times:
						# Prevent writing duplicate keyframes
						# Convert to string to force rounded float comparison
						str_t = '%f'%t 
						if str_t not in times_written:
							#print('keyframe %s' % str_t)
							times_written.add(str_t)
						else:
							#print('skipping duplicate object motion blur keyframe time %s' % str_t)
							continue
						
						posn = Blender.Mathutils.Vector(
							icux[mbframe],
							icuy[mbframe],
							icuz[mbframe]
						) * ws
						
						try:
							rotn = Blender.Mathutils.Euler(
								(icurx[mbframe] - icurx[startF]) * 10,
								(icury[mbframe] - icury[startF]) * 10,
								(icurz[mbframe] - icurz[startF]) * 10
							)
						except:
							rotn = Blender.Mathutils.Euler(0,0,0) # zero is OK here because rotation is added to start position
						
						kf = {
							'file': file,
							'time': (t, mbframe),
							'posn': posn,
							'rotn': rotn,
						}
						writeKeyFrame(**kf)
			
			if write_static_position:
				# static position
				file.write("\t\t<pos>%f %f %f</pos>\n" %(pos[0]*ws, pos[1]*ws, pos[2]*ws))
			
			file.write("\t\t<mesh_name>%s</mesh_name>\n" %(imeshname))
			#Emission Scale
			if len(mesh.materials) > 0:
				emitter_idx=0
				for mat_i in mesh.materials:
					if mat_i != None:
						try:
							d = mat_i.properties['indigo'].convert_to_pyobject()
						except KeyError:
							d = {}
					
					#if matIsEmitter(mat_i):
					if readValue(d,'emitter',0):
						file.write("\t\t<emission_scale>\n")
						file.write("\t\t\t<material_name>%s</material_name>\n"%mesh.materials[emitter_idx].getName())
						file.write("\t\t\t<measure>")
						value=readValue(d,'emitter.scale_measure', IndigoTypes.EmissionMeasure._('FLUX'))
						if IndigoTypes.EmissionMeasure.isT(value, 'FLUX'):
							file.write("luminous_flux")
						elif IndigoTypes.EmissionMeasure.isT(value, 'INTENSITY'):
							file.write("luminous_intensity")
						elif IndigoTypes.EmissionMeasure.isT(value, 'EMITTANCE'):
							file.write("luminous_emittance")
						elif IndigoTypes.EmissionMeasure.isT(value, 'LUMINANCE'):
							file.write("luminance")
						file.write("</measure>\n")
						file.write("\t\t\t<value>%f</value>\n"%readValue(d,'emitter.scale_value',0))
						file.write("\t\t</emission_scale>\n")
							
						if readValue(d,'emitter.ies',0) and readValue(d,'emitter.ies_path','') != '':
							file.write("\t\t<ies_profile>\n")
							file.write("\t\t\t<material_name>%s</material_name>\n"%mesh.materials[emitter_idx].getName())
							file.write("\t\t\t<path>%s</path>\n"%readValue(d,'emitter.ies_path',''))
							file.write("\t\t</ies_profile>\n")
					emitter_idx+=1
			if exitportal:
				file.write("\t</exit_portal>\n")
			else:
				file.write("\t</model>\n\n")
	elif (objectdup):
		print "object DupGroup"
		if (obj.enableDupGroup):
			group = obj.DupGroup
			groupname = group.name
		for o, m in obj.DupObjects:
			exportObject(o, exportedobjlist, file, mat=mat, particles_file=particles_file, write_particles=write_particles)

def getMeshFromObject(ob, container_mesh=None, apply_modifiers=True, vgroups=True, scn=None):
	'''
	ob - the object that you want to get the mesh from
	container_mesh - a Blender.Mesh type mesh that is reused to avoid a new datablock per call to getMeshFromObject
	apply_modifiers - if enabled, subsurf bones etc. will be applied to the returned mesh. disable to get a copy of the mesh.
	vgroup - For mesh objects only, apply the vgroup to the the copied mesh. (slower)
	scn - Scene type. avoids getting the current scene each time getMeshFromObject is called.
	
	Returns Mesh or None
	'''
	
	if not scn:
		scn = Blender.Scene.GetCurrent()
	if not container_mesh:
		mesh=Blender.Mesh.New(ob.name)
	else:
		mesh= container_mesh
		mesh.verts= None
	
	ob_type = ob.type
	dataname = ob.getData(1)
	tempob= None
	if apply_modifiers or ob_type != 'Mesh':
		try:
			mesh.getFromObject(ob,0,1)
		except:
			return None
	else:
		'''
		Dont apply modifiers, copy the mesh.
		So we can transform the data. its easiest just to get a copy of the mesh.
		'''
		tempob= scn.objects.new(ob.getData(mesh=1))
		mesh.getFromObject(tempob, render=1)
		scn.objects.unlink(tempob)
	tmesh=ob.getData(mesh=1)
	try:
		d=tmesh.properties['subdivision'].convert_to_pyobject()
	except:
		d = {}
	mesh.properties['subdivision']=d
	
	return mesh

def writeIncludes(file, includelist):
	if len(includelist) > 0:
		for i in includelist:
			file.write("\t<include>\n")
			file.write("\t\t<pathname>%s</pathname>\n"%i)
			file.write("\t</include>\n\n")

def getMoBlurTiming(current_frame, first_and_last_only=False):
	fps=Blender.Scene.GetCurrent().getRenderingContext().fps		# frames per second
	exposure=1.0/IndigoUIControls.Camera.Shutter.val				# exposure seconds
	frames = exposure*fps											# exposure frames
	keyframe_times = []
	if first_and_last_only:
		keyframe_times = [
			(0.0, current_frame),
			(1.0, current_frame+frames)
		]
	elif frames > 1:
		time_step=1.0/int(frames)									# keyframe timestep
		keyframe_times = [
			(
				i,													# normalized keyframe number (0,1)
				(i*frames)+current_frame							# fractional keyframe number (real frame number)
			) for i in frange6(0.0, 1.0, time_step)
		]
		# Make sure we have the final KF included
		maxi, maxmbframe = keyframe_times[len(keyframe_times)-1]
		if maxi < 1.0:
			keyframe_times.append( (1.0, frames+current_frame) )
	
	return keyframe_times

### EXPORT TO FILES ####
# filename - file name where will be exported to
# objlist - objects to export
# meshlist - meshes to export
# matlist - materials to export
def exportIndigoScene(filename, objlist, meshlist, matlist, anim_progress=None, first_frame=False):
	Blender.Window.WaitCursor(1)
	
	animation_mode = (anim_progress is not None)
	pb = exportProgressBar(6, anim_progress)
	
	flyby_mode = IndigoUIControls.System.FlybyMode.val
	if animation_mode and flyby_mode:
		mesh_save_dir = os.path.join(os.path.dirname(filename), 'static_meshes')
		# At the start of anim export, remove static meshes otherwise they won't get
		# rewritten for this export if they already exist.
		if os.path.exists(mesh_save_dir) and first_frame:
			print('Cleaning static meshes from previous export')
			for mesh_file in os.listdir(mesh_save_dir):
				os.remove( os.path.join(mesh_save_dir, mesh_file) )
	else:
		mesh_save_dir = os.path.join(os.path.dirname(filename), '%05i'%Blender.Get('curframe'))
	
	if not os.path.exists(mesh_save_dir):
		os.makedirs(mesh_save_dir)
	
	current_mesh_index = 0
	
	IndigoGlobals.layer_names = ['Environment']
	IndigoGlobals.layer_name_counts = { 'Environment': 0 }
	
	ws = IndigoUIControls.Environment.WorldScale.val*(10**IndigoUIControls.Environment.WorldScaleExp.val)
	file = open(filename, 'w')
	file.write("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n\n")
	file.write('<scene>\n')
	
	# Write metadata
	file.write('\t<metadata>\n')
	file.write('\t\t<exporter>Blendigo v%s</exporter>\n' % blendigoVersion)
	file.write('\t\t<exporter_specific>\n\t\t\t<python_version>%s</python_version>\n\t\t</exporter_specific>\n' % PlatformInformation.python)
	file.write('\t\t<platform>%s - %s</platform>\n' % (PlatformInformation.type, PlatformInformation.uname))
	file.write('\t\t<author>%s</author>\n' % PlatformInformation.user)
	file.write('\t\t<created_date>%s</created_date>\n' % time.strftime('%Y-%m-%d %H:%M:%S GMT', time.gmtime()))
	file.write('\t</metadata>\n')
	
	# write render_settings
	pb.next('Writing Renderer Settings')
	file.write("\t<renderer_settings>\n")
	file.write("\t\t<width>%d</width>\n" % int(IndigoUIControls.Camera.SizeX.val*IndigoUIControls.Camera.ScaleSize.val/100.0))
	file.write("\t\t<height>%d</height>\n" % int(IndigoUIControls.Camera.SizeY.val*IndigoUIControls.Camera.ScaleSize.val/100.0))
	#if not Override.val:
	if True:
		file.write("\t\t<metropolis>%s</metropolis>\n" %(IndigoBoolean(IndigoUIControls.Renderer.MLT.val)))
#		if MLT.val:
#			file.write("\t\t<large_mutation_prob>%f</large_mutation_prob>\n"%LMP.val)
#		file.write("\t\t<max_change>%f</max_change>\n"%MaxChange.val)
		file.write("\t\t<max_num_consec_rejections>%d</max_num_consec_rejections>\n" % IndigoUIControls.Renderer.MaxNumConsRej.val)
		file.write("\t\t<bidirectional>%s</bidirectional>\n" %(IndigoBoolean(IndigoUIControls.Renderer.Bidirectional.val)))
#		file.write("\t\t<max_depth>%s</max_depth>\n" % MaxDepth)
		file.write("\t\t<logging>%s</logging>\n" %(IndigoBoolean(IndigoUIControls.System.Logging.val)))
		file.write("\t\t<info_overlay>%s</info_overlay>\n" %(IndigoBoolean(IndigoUIControls.System.IOverlay.val)))
		file.write("\t\t<save_untonemapped_exr>%s</save_untonemapped_exr>\n" %(IndigoBoolean(IndigoUIControls.System.SaveUTMExr.val)))
		file.write("\t\t<save_tonemapped_exr>%s</save_tonemapped_exr>\n" %(IndigoBoolean(IndigoUIControls.System.SaveTMExr.val)))
		file.write("\t\t<save_igi>%s</save_igi>\n" %(IndigoBoolean(IndigoUIControls.System.SaveIgi.val)))
#		file.write("\t\t<display_period>%s</display_period>\n" % float(IndigoUIControls.System.DisplayPeriod.val))
		file.write("\t\t<halt_time>%s</halt_time>\n" % (max(IndigoUIControls.System.HaltTime.val, -1)))
		file.write("\t\t<halt_samples_per_pixel>%s</halt_samples_per_pixel>\n" % IndigoUIControls.System.HaltSamples.val)
#		file.write("\t\t<frame_upload_period>%s</frame_upload_period>\n" %(FrameUp.val))
		file.write("\t\t<image_save_period>%s</image_save_period>\n" %(IndigoUIControls.System.ImageSave.val))
		file.write("\t\t<super_sample_factor>%s</super_sample_factor>\n" %(IndigoUIControls.Renderer.SuperSample.val))
		#file.write("\t\t<hybrid>%s</hybrid>\n" %(IndigoBoolean(Hybrid.val)))
		#file.write("\t\t<bih_tri_threshold>%s</bih_tri_threshold>\n" %(BIH.val))
		#file.write("\t\t<ray_origin_nudge_distance>%de%d</ray_origin_nudge_distance>\n"%(RayNudgeMant.val,RayNudgeExp.val))
#		file.write("\t\t<cache_trees>%s</cache_trees>\n" %(IndigoBoolean(CacheTree.val)))
		
		#file.write("\t\t<num_threads>%d</num_threads>\n"%Threads.val)
		
		file.write("\t\t<auto_choose_num_threads>%s</auto_choose_num_threads>\n"%(IndigoBoolean(IndigoUIControls.System.AutoThreads.val)))
		file.write("\t\t<splat_filter>\n")
		if IndigoTypes.Filters.isT(IndigoUIControls.Renderer.SplatFilter.val, 'BOX'):
			file.write("\t\t\t<box/>\n")
		if IndigoTypes.Filters.isT(IndigoUIControls.Renderer.SplatFilter.val, 'FASTBOX'):
			file.write("\t\t\t<fastbox/>\n")
		if IndigoTypes.Filters.isT(IndigoUIControls.Renderer.SplatFilter.val, 'GAUSSIAN'):
			file.write("\t\t\t<gaussian></gaussian>\n")
		if IndigoTypes.Filters.isT(IndigoUIControls.Renderer.SplatFilter.val, 'MN'):
			file.write("\t\t\t<mn_cubic>\n")
			file.write("\t\t\t\t<blur>%f</blur>\n"%IndigoUIControls.Renderer.SplatF_MN_Blur.val)
			file.write("\t\t\t\t<ring>%f</ring>\n"%IndigoUIControls.Renderer.SplatF_MN_Ring.val)
			file.write("\t\t\t</mn_cubic>\n")
		file.write("\t\t</splat_filter>\n")
		file.write("\t\t<downsize_filter>\n")
		
		if IndigoTypes.Filters.isT(IndigoUIControls.Renderer.DownsizeFilter.val, 'BOX'):
			file.write("\t\t\t<box/>\n")
		if IndigoTypes.Filters.isT(IndigoUIControls.Renderer.DownsizeFilter.val, 'GAUSSIAN'):
			file.write("\t\t\t<gaussian></gaussian>\n")
		if IndigoTypes.Filters.isT(IndigoUIControls.Renderer.DownsizeFilter.val, 'MN'):
			file.write("\t\t\t<mn_cubic>\n")
			file.write("\t\t\t\t<blur>%f</blur>\n"%IndigoUIControls.Renderer.DownsizeF_MN_Blur.val)
			file.write("\t\t\t\t<ring>%f</ring>\n"%IndigoUIControls.Renderer.DownsizeF_MN_Ring.val)
			file.write("\t\t\t</mn_cubic>\n")
		file.write("\t\t</downsize_filter>\n")
		file.write("\t\t<aperture_diffraction>%s</aperture_diffraction>\n"%(IndigoBoolean(IndigoUIControls.Camera.ApertureDiffr.val)));
		if IndigoUIControls.Camera.ApertureDiffr.val and IndigoUIControls.Camera.AperturePProc.val:
			file.write("\t\t<post_process_diffraction>true</post_process_diffraction>\n")
		else:
			file.write("\t\t<post_process_diffraction>false</post_process_diffraction>\n")
		
	file.write("\t\t<watermark>%s</watermark>\n"%(IndigoBoolean(IndigoUIControls.System.Watermark.val)));
	if Blender.Scene.GetCurrent().getRenderingContext().borderRender:
		region=Blender.Scene.GetCurrent().getRenderingContext().border
		file.write("\t\t<render_region>\n")
		file.write("\t\t\t<x1>%d</x1>\n"%(int(IndigoUIControls.Camera.SizeX.val*IndigoUIControls.Camera.ScaleSize.val/100.0)*region[0]))
		file.write("\t\t\t<y1>%d</y1>\n"%(int(IndigoUIControls.Camera.SizeY.val*IndigoUIControls.Camera.ScaleSize.val/100.0)-(IndigoUIControls.Camera.SizeY.val*IndigoUIControls.Camera.ScaleSize.val/100)*region[3]))
		file.write("\t\t\t<x2>%d</x2>\n"%(int(IndigoUIControls.Camera.SizeX.val*IndigoUIControls.Camera.ScaleSize.val/100.0)*region[2]))
		file.write("\t\t\t<y2>%d</y2>\n"%(int(IndigoUIControls.Camera.SizeY.val*IndigoUIControls.Camera.ScaleSize.val/100.0)-(IndigoUIControls.Camera.SizeY.val*IndigoUIControls.Camera.ScaleSize.val/100)*region[1]))
		file.write("\t\t</render_region>\n")
	if IndigoUIControls.Renderer.AlphaMask.val:
		file.write("\t\t<render_foreground_alpha>true</render_foreground_alpha>\n")
	
	if IndigoUIControls.System.CloseGUIOnHalt.val:
		file.write('\t\t<close_gui_on_halt>true</close_gui_on_halt>\n')
	else:
		file.write('\t\t<close_gui_on_halt>false</close_gui_on_halt>\n')
	
	file.write("\t</renderer_settings>\n\n")
	
	# write tone mapping
	pb.next('Writing Tonemapping Settings')
	if IndigoTypes.Tonemapping.isT(IndigoUIControls.Tonemapping.ToneMapType.val, 'LINEAR'):
		if (IndigoUIControls.Camera.Autoexp.val == 0):
			exposure = IndigoUIControls.Tonemapping.ToneMapScale.val * (10**IndigoUIControls.Tonemapping.ToneMapScaleExp.val)
		else:
			if IndigoUIControls.Camera.Shutter.val == 0:
				print "WARNING: Shutter is not set (using 125)"
				IndigoUIControls.Camera.Shutter.val = 125
			exposure = IndigoUIControls.Tonemapping.ToneMapScale.val * (10**IndigoUIControls.Tonemapping.ToneMapScaleExp.val) * float(IndigoUIControls.Camera.FilmIso.val) / float(IndigoUIControls.Camera.Shutter.val)
		file.write("\t<tonemapping>\n")
		file.write("\t\t<linear>\n")
		file.write("\t\t\t<scale>%g</scale>\n" % exposure)
		file.write("\t\t</linear>\n")
		file.write("\t</tonemapping>\n\n")
	if IndigoTypes.Tonemapping.isT(IndigoUIControls.Tonemapping.ToneMapType.val, 'REINHARD'):
		file.write("\t<tonemapping>\n")
		file.write("\t\t<reinhard>\n")
		file.write("\t\t\t<pre_scale>%g</pre_scale>\n" %IndigoUIControls.Tonemapping.ToneMapPreScale.val)
		file.write("\t\t\t<post_scale>%g</post_scale>\n" %IndigoUIControls.Tonemapping.ToneMapPostScale.val)
		file.write("\t\t\t<burn>%g</burn>\n" %IndigoUIControls.Tonemapping.ToneMapBurn.val)
		file.write("\t\t</reinhard>\n")
		file.write("\t</tonemapping>\n\n")
	if IndigoTypes.Tonemapping.isT(IndigoUIControls.Tonemapping.ToneMapType.val, 'CAMERA'):
		file.write("\t<tonemapping>\n")
		file.write("\t\t<camera>\n");
		file.write("\t\t<ev_adjust>%g</ev_adjust>\n"%IndigoUIControls.Tonemapping.ToneMapEV.val)
		file.write("\t\t<film_iso>%g</film_iso>\n"%float(IndigoUIControls.Camera.FilmIso.val))
		if IndigoUIControls.Tonemapping.ToneMapResponseCurve.val:
			try:
				file.write("\t\t<response_function_path>data/camera_response_functions/%s.txt</response_function_path>\n"%IndigoUIControls.Camera.tonemapcurves[IndigoUIControls.Tonemapping.ToneMapResponseCurve.val])
			except:
				file.write("\t\t<response_function_path>data/camera_response_functions/dscs315.txt</response_function_path>\n")
		else:
			file.write("\t\t<response_function_path>data/camera_response_functions/dscs315.txt</response_function_path>\n")
		file.write("\t\t</camera>\n")
		file.write("\t</tonemapping>\n\n")
	
	# write Color background
	pb.next('Writing Environment')
	if IndigoTypes.Environment.isT(IndigoUIControls.Environment.EnvType.val, 'BACKGROUND'):
		#worldcolor = Blender.World.Get('World').getHor()
		file.write("\t<background>\n")
		file.write("\t\t<spectrum>\n")
		file.write("\t\t\t<rgb>\n")
		file.write("\t\t\t\t<rgb>%g %g %g</rgb>\n" %(IndigoUIControls.Environment.BGColor.val[0]*IndigoUIControls.Environment.BGGain.val, IndigoUIControls.Environment.BGColor.val[1]*IndigoUIControls.Environment.BGGain.val, IndigoUIControls.Environment.BGColor.val[2]*IndigoUIControls.Environment.BGGain.val))
		file.write("\t\t\t\t<gamma>2.2</gamma>\n")
		file.write("\t\t\t</rgb>\n")
		file.write("\t\t</spectrum>\n")
		file.write("\t</background>\n\n")
		IndigoGlobals.layer_name_counts['Environment'] += 1
	
	##### Process sun ######
	if IndigoTypes.Environment.isT(IndigoUIControls.Environment.EnvType.val, 'SUNSKY'):
		flag_sun = 0
		for obj in  objlist:
			if obj.getType() == "Lamp":
				
				if obj.data.getType() in [0, 1] and not flag_sun: # [lamp object, sun object]
					invmatrix = Blender.Mathutils.Matrix(obj.getInverseMatrix())
					file.write("\t<skylight>\n")
					file.write("\t\t<sundir>%f %f %f</sundir>\n" % (invmatrix[0][2], invmatrix[1][2], invmatrix[2][2]))
					file.write("\t\t<turbidity>%f</turbidity>\n" %IndigoUIControls.Environment.Turbidity.val)
					if IndigoUIControls.Environment.ExtraAtm.val:
						file.write("\t\t<extra_atmospheric>%s</extra_atmospheric>\n" %IndigoBoolean(IndigoUIControls.Environment.ExtraAtm.val))
					
					if IndigoUIControls.Environment.SunLayerToggle.val:
						if 'Sun' not in IndigoGlobals.layer_names:
							IndigoGlobals.layer_names.append('Sun')
						if 'Sun' in IndigoGlobals.layer_name_counts.keys():
							IndigoGlobals.layer_name_counts['Sun'] += 1
						else:
							IndigoGlobals.layer_name_counts['Sun'] = 1
						sun_layer = IndigoGlobals.layer_names.index('Sun')
						file.write("\t\t<sun_layer>%d</sun_layer>\n"%sun_layer)
						
					if IndigoUIControls.Environment.SkyLayerToggle.val:
						if 'Sky' not in IndigoGlobals.layer_names:
							IndigoGlobals.layer_names.append('Sky')
						if 'Sky' in IndigoGlobals.layer_name_counts.keys():
							IndigoGlobals.layer_name_counts['Sky'] += 1
						else:
							IndigoGlobals.layer_name_counts['Sky'] = 1
						sky_layer = IndigoGlobals.layer_names.index('Sky')
						file.write("\t\t<sky_layer>%d</sky_layer>\n"%sky_layer)
					
					file.write("\t</skylight>\n\n")
					flag_sun=1
		
		if not flag_sun:
			#Blender.Draw.PupMenu("Error%t|No Sun found");
			file.write("\t<background>\n")
			file.write("\t\t<spectrum>\n")
			file.write("\t\t\t<uniform>\n")
			file.write("\t\t\t\t<value>1.0</value>\n")
			file.write("\t\t\t</uniform>\n")
			file.write("\t\t</spectrum>\n")
			file.write("\t</background>\n\n")
			# Don't change the UI settings here.
			print("ERROR: Environment type was set to Physical Sun+Sky, but no sun lamp was found in the scene! A uniform background has been set for this export.")
			IndigoGlobals.layer_name_counts['Environment'] += 1
	
	##### Process Env Map #####
	if IndigoTypes.Environment.isT(IndigoUIControls.Environment.EnvType.val, 'MAP'):
		HDRI_file = IndigoUIControls.Environment.EnvFile.val
		if os.path.exists(HDRI_file): #IndigoUIControls.Environment.EnvFile.val != "none" and IndigoUIControls.Environment.EnvFile.val != "":
			file.write("\t<env_map>\n")
			file.write("\t\t<%s>\n" %(IndigoUIControls.Environment.EnvMapTypeV[IndigoUIControls.Environment.EnvMapType.val]))
			file.write("\t\t\t<path>%s</path>\n" % utils.getRelativePath(file.name, HDRI_file))
			file.write("\t\t\t<gain>%g</gain>\n" %(IndigoUIControls.Environment.EnvGain.val*1e4)) #scaled for using with meshlight
			if IndigoUIControls.Environment.EnvMapType.val == 0:
				file.write("\t\t\t<width>%d</width>\n" %(IndigoUIControls.Environment.EnvWidth.val))
			file.write("\t\t</%s>\n" %(IndigoUIControls.Environment.EnvMapTypeV[IndigoUIControls.Environment.EnvMapType.val]))
			file.write("\t</env_map>\n\n")
			IndigoGlobals.layer_name_counts['Environment'] += 1
		else:
			print("ERROR: Environment type was set to Map, but the path given is not valid. No environment exported.")
	
	##### Process camera ######
	pb.next('Writing Camera')
	currentscene = Blender.Scene.GetCurrent()
	#camObj = currentscene.getCurrentCamera()
	camObj = currentscene.objects.camera
	if camObj:
		matrix = camObj.getMatrix()
		pos = matrix[3]
		forwards = matrix[2]
		up = matrix[1]
		file.write("\t<camera>\n")
		
		aspect = float(IndigoUIControls.Camera.SizeX.val) / float(IndigoUIControls.Camera.SizeY.val)
		film = 0.001*IndigoUIControls.Camera.FilmWidth.val
		
		f_stop = float(IndigoUIControls.Camera.FStop.val) * 0.1
		FOV=camObj.data.angle
		if aspect < 1:
			FOV = FOV*aspect
		lens_sensor_dist = film/(2*math.tan(FOV*math.pi/360))
		
		if IndigoUIControls.Camera.FStop.val == 0:
			print "WARNING: FStop is not set (using 8.0)"
			IndigoUIControls.Camera.FStop.val = 80
		aperture_radius = lens_sensor_dist / (2*f_stop)
		file.write("\t\t<aperture_radius>%f</aperture_radius>\n" % aperture_radius)
		exposure=float(1/float(IndigoUIControls.Camera.Shutter.val))
		file.write("\t\t<exposure_duration>%f</exposure_duration>\n"%exposure)
		
		if IndigoUIControls.Camera.AutoFocus.val:
			focus_dist = 1.0
		else:
			focus_dist = IndigoUIControls.Camera.FocusDistance.val
			
			try:
				caIpo = Blender.Ipo.Get('CaIpo')
			except:
				caIpo = None
			
			if caIpo != None:
				icu = caIpo[Blender.Ipo.CA_FDIST ]
				if icu != None and len(icu.bezierPoints) > 0:
					focus_dist=icu[Blender.Get('curframe')]
		
		file.write("\t\t<focus_distance>%f</focus_distance>\n" % (focus_dist*ws))
		file.write("\t\t<aspect_ratio>%g</aspect_ratio>\n" % aspect)
		file.write("\t\t<sensor_width>%f</sensor_width>\n" % film)
		file.write("\t\t<lens_sensor_dist>%f</lens_sensor_dist>\n" % lens_sensor_dist)
		file.write("\t\t<white_balance>%s</white_balance>\n" % IndigoUIControls.Camera.whiteBalanceV[IndigoUIControls.Camera.WhiteBalance.val])
		
		if IndigoUIControls.Camera.ApertureDiffr.val:
			file.write("\t\t<aperture_shape>\n")
			
			if IndigoUIControls.Camera.ApertureType.val == 1:
				file.write("\t\t\t<circular />\n")
			elif IndigoUIControls.Camera.ApertureType.val == 2:
				file.write("\t\t\t<generated>\n")
				file.write("\t\t\t\t<num_blades>%d</num_blades>\n"%IndigoUIControls.Camera.BladesNum.val)
				file.write("\t\t\t\t<blade_offset>%f</blade_offset>\n"%float(IndigoUIControls.Camera.BladeOffset.val))
				file.write("\t\t\t\t<blade_curvature_radius>%f</blade_curvature_radius>\n"%float(IndigoUIControls.Camera.BladeCurvature.val))
				file.write("\t\t\t\t<start_angle>%f</start_angle>\n"%float(IndigoUIControls.Camera.StartAngle.val))
				file.write("\t\t\t</generated>\n")
			elif IndigoUIControls.Camera.ApertureType.val == 3:
				file.write("\t\t\t<image>\n")
				file.write("\t\t\t<path>%s</path>\n"%(utils.getRelativePath(file.name, IndigoUIControls.Camera.ApertureImg.val)))
				file.write("\t\t\t</image>\n")
				
			file.write("\t\t</aperture_shape>\n")
				
			if IndigoUIControls.Camera.ApertureType.val in [1,2,3] and not IndigoUIControls.Camera.ObstacleMap.val == '':
				file.write("\t\t<obstacle_map>\n")
				file.write("\t\t\t<path>%s</path>\n"%(utils.getRelativePath(file.name, IndigoUIControls.Camera.ObstacleMap.val)))
				file.write("\t\t</obstacle_map>\n")
			
		if camObj.data.shiftX != 0:
			sx = camObj.data.shiftX * film
			file.write("\t\t<lens_shift_right_distance>%f</lens_shift_right_distance>\n"%sx)
			
		if camObj.data.shiftY != 0:
			sy = camObj.data.shiftY * film
			file.write("\t\t<lens_shift_up_distance>%f</lens_shift_up_distance>\n"%sy)
		
		if IndigoUIControls.Camera.AutoFocus.val==1:
			file.write("\t\t<autofocus/>\n")
		
		write_static_position = True
		if IndigoUIControls.Camera.CamMoBlur.val:
			ipo = camObj.getIpo()
			
			if ipo != None:
				icux = ipo[Blender.Ipo.OB_LOCX]
				icuy = ipo[Blender.Ipo.OB_LOCY]
				icuz = ipo[Blender.Ipo.OB_LOCZ]
				
				icurx = ipo[Blender.Ipo.OB_ROTX]
				icury = ipo[Blender.Ipo.OB_ROTY]
				icurz = ipo[Blender.Ipo.OB_ROTZ]
				
				current_frame = Blender.Get('curframe')
				
				keyframe_times = getMoBlurTiming(current_frame)
				
				if len(keyframe_times) > 1:
					write_static_position = False
					#file.write("\t\t<up>0.0 1.0 0.0</up>\n")
					#file.write("\t\t<forwards>0.0 0.0 -1.0</forwards>\n")
					
					file.write("\t\t<up>0.0 1.0 0.0</up>\n")
					file.write("\t\t<forwards>0.0 0.0 -1.0</forwards>\n")
					
					try:
						start_rotn = Blender.Mathutils.Euler(
							icurx[current_frame] * 10, # IPO rotation values are in 10 degree units
							icury[current_frame] * 10,
							icurz[current_frame] * 10
						)
					except:
						start_rotn = camObj.matrix.toEuler() # insert camera static rotation
					
					# from 0.0 to 1.0 in increments of time_step (plus force final time 1.0)
					times_written = set()
					
					#print('camera motion blur keyframe times: %s' % ' '.join(['%f' % i for i in keyframe_times]))
					for t, mbframe in keyframe_times:
						# Prevent writing duplicate keyframes
						# Convert to string to force rounded float comparison
						str_t = '%f'%t 
						if str_t not in times_written:
							#print('keyframe %s' % str_t)
							times_written.add(str_t)
						else:
							#print('skipping duplicate camera motion blur keyframe time %s' % str_t)
							continue
						
						posn = Blender.Mathutils.Vector(icux[mbframe], icuy[mbframe], icuz[mbframe]) * ws
						
						try:
							rotn = Blender.Mathutils.Euler(
								icurx[mbframe] * 10, # IPO rotation values are in 10 degree units
								icury[mbframe] * 10,
								icurz[mbframe] * 10
							)
						except:
							rotn = camObj.matrix.toEuler() # insert camera static rotation
						
						rotn.makeCompatible( start_rotn )
						
						kf = {
							'file': file,
							'time': (t, mbframe),
							'posn': posn,
							'rotn': rotn,
						}
						writeKeyFrame(**kf)
					
					#print('camera motion blur keyframes written: %s' % ' '.join(times_written))
		
		if write_static_position:
			file.write("\t\t<pos>%f %f %f</pos>\n" % (pos[0]*ws, pos[1]*ws, pos[2]*ws))
			file.write("\t\t<up>%f %f %f</up>\n" % (up[0], up[1], up[2]))
			file.write("\t\t<forwards>%f %f %f</forwards>\n" % (-forwards[0], -forwards[1], -forwards[2]))
		
		file.write("\t</camera>\n\n")
	
	# Seperate materials file?
	pb.next('Writing Materials')
	#if (SepMatFile.val):
	#	matsfn = Blender.sys.makename(filename, '-mats.xml')
	#	matsfile=open(matsfn,'w')
	#	matsfile.write("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n\n")
	#	matsfile.write('<scenedata>\n')
	#else:
	matsfile = file
	
	matsfile.write("\t<material>\n")
	matsfile.write("\t\t<name>Default</name>\n")
	matsfile.write("\t\t<diffuse>\n")
	matsfile.write("\t\t\t<albedo>\n\t\t\t\t<constant><rgb>\n\t\t\t\t\t<rgb>0.95 0.95 0.95</rgb>\n\t\t\t\t\t<gamma>2.2</gamma>\n\t\t\t\t</rgb></constant>\n\t\t\t</albedo>\n")
	matsfile.write("\t\t</diffuse>\n")
	matsfile.write("\t</material>\n\n")
	matsfile.write("\t<material>\n")
	matsfile.write("\t\t<name>null</name>\n")
	matsfile.write("\t\t<null_material/>\n")
	matsfile.write("\t</material>\n\n")
	exportedmatlist = ["Default", "null"]
	
	includelist = []
	
	for obj in objlist:
		try:
			if obj.getType() == "Text":
				cu=Blender.Curve.Get(obj.getData().getName())
				matlist=cu.getMaterials()
			else:
				matlist=obj.getData().getMaterials()
		except:
			matlist=[]
		
		for mat in matlist:
			matname = mat.getName()
			#print("  * " + matname)

			if matname not in exportedmatlist:
				il, matname = exportMaterial(mat, matsfile, exportedmatlist)
				for i in il:
					if i not in includelist:
						includelist.append(i)
				exportedmatlist.append(matname)
	
	writeIncludes(matsfile, includelist)
	
	# Seperate materials file?
	#if (SepMatFile.val):
	#	matsfile.write('</scenedata>\n')
	#	matsfile.close
		
	#	print("Writing relative matfile %s to filename %s"%(matsfile.name, filename))
	#	relmatsfn = utils.getRelativePath(filename, matsfile.name)
	#	file.write("\t<include>\n")
	#	file.write("\t\t<pathname>%s</pathname>\n"%relmatsfn)
	#	file.write("\t</include>\n\n")

	pb.next('Writing Meshes')
	exportedmeshlist=[]
	#pbm = exportProgressBar(len(objlist), pb)
	for obj in objlist:
		#pbm.counter('Writing Meshes')
		objecttype = obj.getType()
		if objecttype in ['Mesh','Curve','Text']:
			meshname = obj.getData(name_only=1)
			if(objecttype == "Curve"):
				meshname +='_cu'
			mesh=getMeshFromObject(obj)
			if not mesh:
				continue
			if len(mesh.faces) <= 0:
				continue
			if (objecttype == "Curve"):
				mesh.materials = obj.getData().getMaterials()
			if (objecttype == "Text"):
				curveName=obj.getData().getName()
				mesh.materials = Blender.Curve.Get(curveName).getMaterials()
			
			if meshname not in exportedmeshlist:
				uvlayer='default'
				uvflag=0
				if len(mesh.getUVLayerNames()) != 0:
					uvlayer=mesh.getUVLayerNames()
					uvflag=1
				for mat in mesh.materials:
					if 'indigo' not in mat.properties.keys():
						convertMaterial(mat)	
						
					d = mat.properties['indigo'].convert_to_pyobject()
					if not uvflag and ('albedo.path' in d and d['albedo.path'] != '' or 'bamp.path' in d and d['bump.path'] != '' or 'displace.path' in d and d['displace.path'] != ''):
						Blender.Window.WaitCursor(0)
						message="Error%t|"
						message+="Material '%s' in Mesh '%s' has no UVset!"%(mat.getName(),meshname)
						Blender.Draw.PupMenu(message,20)
						return 1
					#print d['albedo.uv_set']
					
				# Conditionally write the binary data
				if IndigoUIControls.System.ExportMeshs.val:
					exportMeshBinary(mesh, meshname, mesh_save_dir, current_mesh_index, obj.getMatrix(), file, animation_mode, flyby_mode)
					
				# Always write the <mesh> element
				exportMeshElement(meshname, mesh, current_mesh_index, file, animation_mode, flyby_mode)
				current_mesh_index += 1
				exportedmeshlist.append(meshname)
	#pbm.finished()
	
	# detect if we need to separate particle instance <model> tags into a 'static' file
	write_particles = True
	particles_file = None
	if animation_mode and flyby_mode:
		particles_filename = file.name[:-9]+'static_particles.igs'
		file.write('\t<include><pathname>%s</pathname></include>\n' % utils.getRelativePath(file.name, particles_filename))
		if not os.path.exists(particles_filename):
			particles_file = open(particles_filename, 'w')
			particles_file.write('<scenedata>\n')
		else:
			write_particles = False
			print('\tRe-using static particles file %s'%particles_filename)
	
	exportedobjlist=[]
	for obj in objlist:
		objectname = obj.getName()
		if objectname not in exportedobjlist:
			exportObject(obj, exportedobjlist, file, particles_file=particles_file, write_particles=write_particles)
			exportedobjlist.append(objectname)
	
	if animation_mode and flyby_mode and write_particles:
		particles_file.write('</scenedata>\n')
		particles_file.close()

	if len(IndigoGlobals.layer_names) > 0:
		for i, l in enumerate(IndigoGlobals.layer_names):
			if IndigoGlobals.layer_name_counts[l] > 0:
				file.write("\t<layer_name>\n")
				file.write("\t\t<layer_index>%i</layer_index>\n" % i)
				file.write("\t\t<layer_name>%s</layer_name>\n" % l)
				file.write("\t</layer_name>\n")
	
	file.write('</scene>\n')
	file.close()
	pb.finished()
	Blender.Window.WaitCursor(0)

def frange6(*args):
	"""A float range generator."""
	start = 0.0
	step = 1.0

	l = len(args)
	if l == 1:
		end = args[0]
	elif l == 2:
		start, end = args
	elif l == 3:
		start, end, step = args
		if step == 0.0:
			raise ValueError, "step must not be zero"
	else:
		raise TypeError, "frange expects 1-3 arguments, got %d" % l

	v = start
	while True:
		if (step > 0 and v >= end) or (step < 0 and v <= end):
			raise StopIteration
		yield v
		v += step

def exportMatPreview(mat):
	fileName = os.path.join(utils.getTempDir(), 'preview_scene.igs')
	
	f = open(fileName, "w")
	f.write("<?xml version='1.0' encoding='utf-8'?>\n<scene>\n")
	
	il, matname = exportMaterial(mat, f, ["Default", "null"], "preview_material", 'default')
	
	f.write("\n\t<include>\n\t\t<pathname>" + utils.getPreviewScenePath(IndigoPaths) + "</pathname>\n\t</include>\n</scene>")
	f.close()
	
	utils.launchGui(fileName,"")

def export(filename, anim):
	IndigoFilePath=filename
	time1 = Blender.sys.time()
	filepath = Blender.sys.dirname(filename)
	
	objlist = []
	matlist = []
	meshlist = []
	Blender.Draw.Draw()#Force an immediate redraw
	
	if anim:
		startF = Blender.Get('staframe')
		endF = Blender.Get('endframe')
		pb = exportProgressBar(endF-startF +1)
		print('Exporting Animation; frame range %d-%d' % (startF, endF))
		#print('start %d end %d'%(startF,endF))
		first_frame = True
		for i in range(startF, endF+1):
			framefilename = Blender.sys.makename(filename, "-%05d.igs" % i)
			print('Exporting frame %d to %s'%(i, framefilename))
			pb.next('Exporting frame %d'%i)
			Blender.Set('curframe', i)
			Blender.Scene.GetCurrent().update(1)
			#Blender.Redraw()
			activelayers = Blender.Window.ViewLayers()
			for obj in Blender.Scene.GetCurrent().objects:
				if not obj.restrictDisplay:
					visible = 0
					for layer in obj.layers:
						if layer in activelayers:
							visible = 1
					if visible:
						objlist.append(obj)
			exportIndigoScene(framefilename, objlist, meshlist, matlist, pb, first_frame=first_frame)
			first_frame = False
			del objlist[:]#delete all list items
			del meshlist[:]
			del matlist[:]
		pb.finished()
		Blender.Set('curframe', endF)
		Blender.Scene.GetCurrent().update(1)
		#Blender.Redraw()
	else:
		print("Export Still to %s"%filename)
		#get all the objects in this scene in visible layers
		activelayers = Blender.Window.ViewLayers()
		for obj in Blender.Scene.GetCurrent().objects:
			if not obj.restrictDisplay:
				visible = 0
				for layer in obj.layers:
					if layer in activelayers:
						visible = 1
				if visible:
					objlist.append(obj)
		exportIndigoScene(filename, objlist, meshlist, matlist)
		
	time2 = Blender.sys.time()
	print("Processing time: %f\n" %(time2-time1))
	
	options = list()
	
	if IndigoTypes.Networking.isT(IndigoUIControls.System.NetworkWM.val, 'WORKING'):
		options.extend(["-n", "wm"])
	elif IndigoTypes.Networking.isT(IndigoUIControls.System.NetworkWM.val, 'MASTER'):
		options.extend(["-n", "m"])
	
	if not anim and IndigoUIControls.System.ExecuteIndigo.val:
		if not IndigoUIControls.System.AutoThreads.val:
			options.extend(['-t', '%d' % IndigoUIControls.System.Threads.val])
		if (IndigoUIControls.System.IndigoOutPath.val != "")  and os.path.exists(os.path.dirname(IndigoUIControls.System.IndigoOutPath.val)):
			options.extend(['-o', '"%s"' % IndigoUIControls.System.IndigoOutPath.val])
			if IndigoUIControls.System.SaveIgi.val:
				options.extend(['-igio', '"%s"' % os.path.join(
					os.path.dirname(IndigoUIControls.System.IndigoOutPath.val),
					string.split(os.path.basename(IndigoUIControls.System.IndigoOutPath.val),'.')[0]+".igi"
				)])
			if IndigoUIControls.System.SaveTMExr.val:
				options.extend(['-texro', '"%s"' % os.path.join(
					os.path.dirname(IndigoUIControls.System.IndigoOutPath.val),
					string.split(os.path.basename(IndigoUIControls.System.IndigoOutPath.val),'.')[0]+"-tm.exr"
				)])
			if IndigoUIControls.System.SaveUTMExr.val:
				options.extend(['-uexro', '"%s"' % os.path.join(
					os.path.dirname(IndigoUIControls.System.IndigoOutPath.val),
					string.split(os.path.basename(IndigoUIControls.System.IndigoOutPath.val),'.')[0]+"-utm.exr"
				)])
		if utils.launchGui(filename, ' '.join(options), IndigoUIControls.System.ProcessPrio.val) == False:
			print('However, scene was saved to %s' % filename)
	
	if anim and IndigoUIControls.System.ExecuteIndigo.val:
		if not IndigoUIControls.System.AutoThreads.val:
			utils.launchAnimation(filename, ' '.join(options), IndigoUIControls.System.Threads.val, IndigoUIControls.System.IndigoOutPath.val)
		else:
			utils.launchAnimation(filename, ' '.join(options), 0, IndigoUIControls.System.IndigoOutPath.val)

def exportStill(filename):
	export(filename, False)

def exportAnim(filename):
	export(filename, True)

# load/save other settings to blender scene
def readWriteSettings(write=0):
	try:
		d = Blender.Scene.GetCurrent().properties['indigo'].convert_to_pyobject()
	except KeyError:
		d = {}
	readWriteValue(d, 'DefaultExport', IndigoUIControls.System.DefaultExport, write, 1)
	readWriteValue(d, 'ExportMeshs', IndigoUIControls.System.ExportMeshs, write, 1)
	readWriteValue(d, 'FlybyMode', IndigoUIControls.System.FlybyMode, write, 0)
	readWriteValue(d, 'ClayRender', IndigoUIControls.Renderer.ClayRender, write, 0)
	readWriteValue(d, 'scale size', IndigoUIControls.Camera.ScaleSize, write, 100)
	readWriteValue(d, 'film width', IndigoUIControls.Camera.FilmWidth, write, 35)
	readWriteValue(d, 'fstop', IndigoUIControls.Camera.FStop, write, 80)
	#readWriteValue(d, 'hstop', HStop, write, 0)
	readWriteValue(d, 'lens.shift.up', IndigoUIControls.Camera.LensShiftUp, write, 0)
	readWriteValue(d, 'lens.shift.right', IndigoUIControls.Camera.LensShiftRight, write, 0)
	readWriteValue(d, 'focusdistance', IndigoUIControls.Camera.FocusDistance, write, 2.0)
	readWriteValue(d, 'autofocus', IndigoUIControls.Camera.AutoFocus, write, 1)
	readWriteValue(d, 'selfocus', IndigoUIControls.Camera.SelFocus, write, 0)
	readWriteValue(d, 'curfocus', IndigoUIControls.Camera.CurFocus, write, 0)
	readWriteValue(d, 'cam_mo_blur', IndigoUIControls.Camera.CamMoBlur, write, 0)
	readWriteValue(d, 'mo_blur', IndigoUIControls.Camera.ObjMoBlur, write, 0)
	readWriteValue(d, 'prt_mo_blur', IndigoUIControls.Camera.PrtMoBlur, write, 0)
	readWriteValue(d, 'aperture_diffraction', IndigoUIControls.Camera.ApertureDiffr, write, 0)
	readWriteValue(d, 'aperturen_type', IndigoUIControls.Camera.ApertureType, write, 2)
	readWriteValue(d, 'aperture_pproc', IndigoUIControls.Camera.AperturePProc, write, 0)
	readWriteValue(d, 'aperture_blades_num', IndigoUIControls.Camera.BladesNum, write, 8)
	readWriteValue(d, 'aperture_blade_offset', IndigoUIControls.Camera.BladeOffset, write, 0.4)
	readWriteValue(d, 'aperture_blade_curvature', IndigoUIControls.Camera.BladeCurvature, write, 3)
	readWriteValue(d, 'aperture_start_angle', IndigoUIControls.Camera.StartAngle, write, 0.2)
	readWriteValue(d, 'opstacle_map', IndigoUIControls.Camera.ObstacleMap, write, '')
	readWriteValue(d, 'aperture_img', IndigoUIControls.Camera.ApertureImg, write, '')
	readWriteValue(d, 'white balance', IndigoUIControls.Camera.WhiteBalance, write, 0)
	readWriteValue(d, 'filmiso', IndigoUIControls.Camera.FilmIso, write, 100)
	readWriteValue(d, 'mlt', IndigoUIControls.Renderer.MLT, write, 0)
	readWriteValue(d, 'splat_filter', IndigoUIControls.Renderer.SplatFilter,write, 2)
	readWriteValue(d, 'splatf_mn_blur', IndigoUIControls.Renderer.SplatF_MN_Blur,write, 0.3)
	readWriteValue(d, 'splatf_mn_ring', IndigoUIControls.Renderer.SplatF_MN_Ring,write, 0.3)
	readWriteValue(d, 'downsize_filter', IndigoUIControls.Renderer.DownsizeFilter,write, 2)
	readWriteValue(d, 'downsizef_mn_blur', IndigoUIControls.Renderer.DownsizeF_MN_Blur,write, 0.3)
	readWriteValue(d, 'downsizef_mn_ring', IndigoUIControls.Renderer.DownsizeF_MN_Ring,write, 0.3)
	readWriteValue(d, 'maxnumconsrej', IndigoUIControls.Renderer.MaxNumConsRej, write, 1000)
	readWriteValue(d, 'bidirectional', IndigoUIControls.Renderer.Bidirectional, write, 1)
	readWriteValue(d, 'threads', IndigoUIControls.System.Threads, write, 1)
	readWriteValue(d, 'logging', IndigoUIControls.System.Logging, write, 1)
	readWriteValue(d, 'ioverlay', IndigoUIControls.System.IOverlay, write, 0)
	readWriteValue(d, 'saveutmexr', IndigoUIControls.System.SaveUTMExr, write, 0)
	readWriteValue(d, 'savetmexr', IndigoUIControls.System.SaveTMExr, write, 0)
	readWriteValue(d, 'saveigi', IndigoUIControls.System.SaveIgi, write, 0)
#	readWriteValue(d, 'usegui', UseGUI,write,1)
	readWriteValue(d, 'watermark', IndigoUIControls.System.Watermark,write,0)
	readWriteValue(d, 'network.wm', IndigoUIControls.System.NetworkWM,write, 0)
	if sys.platform[:3] == 'win':
		readWriteValue(d, 'process_prio', IndigoUIControls.System.ProcessPrio, write, 1)
	else:
		readWriteValue(d, 'process_prio', IndigoUIControls.System.ProcessPrio, write, 10)
	
	readWriteValue(d, 'indigo_path', IndigoUIControls.System.IndigoPath, write, '')
	
	readWriteValue(d, 'indigo_out_path', IndigoUIControls.System.IndigoOutPath, write, '')
	#readWriteValue(d, 'network.sl', NetworkSL,write,0)
#	readWriteValue(d, 'display period', IndigoUIControls.System.DisplayPeriod, write, 20)
	readWriteValue(d, 'tonemaptype', IndigoUIControls.Tonemapping.ToneMapType, write, 1)
	readWriteValue(d, 'tonemapscale', IndigoUIControls.Tonemapping.ToneMapScale, write, 1.0)
	readWriteValue(d, 'tonemapscaleexp', IndigoUIControls.Tonemapping.ToneMapScaleExp, write, 0)
	readWriteValue(d, 'tonemapev', IndigoUIControls.Tonemapping.ToneMapEV, write, 0)
	readWriteValue(d, 'tonemapres', IndigoUIControls.Tonemapping.ToneMapResponseCurve, write, 0)
	readWriteValue(d, 'shutter', IndigoUIControls.Camera.Shutter, write, 125)
	readWriteValue(d, 'autoexp', IndigoUIControls.Camera.Autoexp, write, 0)
	readWriteValue(d, 'tonemapprescale', IndigoUIControls.Tonemapping.ToneMapPreScale, write, 4.0)
	readWriteValue(d, 'tonemappostscale', IndigoUIControls.Tonemapping.ToneMapPostScale, write, 1.2)
	readWriteValue(d, 'tonemapburn', IndigoUIControls.Tonemapping.ToneMapBurn, write, 6.0)
	readWriteValue(d, 'envtype', IndigoUIControls.Environment.EnvType, write, IndigoTypes.Environment._('SUNSKY'))
	readWriteValue(d, 'envfile', IndigoUIControls.Environment.EnvFile, write, '')
	readWriteValue(d, 'envmaptype', IndigoUIControls.Environment.EnvMapType, write, 1)
	readWriteValue(d, 'envgain', IndigoUIControls.Environment.BGGain, write, 1.0)
	#readWriteValue(d, 'envwidth', IndigoUIControls.Environment.EnvWidth, write, 640)
	readWriteValue(d, 'sun_layer_toggle', IndigoUIControls.Environment.SunLayerToggle, write, 1)
	readWriteValue(d, 'sun_layer', IndigoUIControls.Environment.SunLayer, write, 0)
	readWriteValue(d, 'sky_layer_toggle', IndigoUIControls.Environment.SkyLayerToggle, write, 1)
	readWriteValue(d, 'sky_layer', IndigoUIControls.Environment.SkyLayer, write, 1)
	readWriteValue(d, 'turbidity', IndigoUIControls.Environment.Turbidity, write, 2.2)
	#readWriteValue(d, 'extraAtm', ExtraAtm,write,0)
	#readWriteValue(d, 'sepmatfile', SepMatFile, write, 0)
	#readWriteValue(d, 'sepmeshfile', SepMeshFile, write, 0)
	readWriteValue(d, 'ExecuteIndigo', IndigoUIControls.System.ExecuteIndigo, write, 1)
	readWriteValue(d, 'HaltTime', IndigoUIControls.System.HaltTime, write, -1.0)
	readWriteValue(d, 'HaltSamples', IndigoUIControls.System.HaltSamples, write, -1.0)
	readWriteValue(d, 'CloseGUIOnHalt', IndigoUIControls.System.CloseGUIOnHalt, write, 0)
	readWriteValue(d, 'imagesave', IndigoUIControls.System.ImageSave, write, 120)
	readWriteValue(d, 'super_sample_factor', IndigoUIControls.Renderer.SuperSample, write, 2)
	#readWriteValue(d, 'bih', BIH, write, 500000)
	#readWriteValue(d, 'rayNudgeMant',RayNudgeMant, write, 1)
	#readWriteValue(d, 'rayNudgeExp',RayNudgeExp, write, -5)
	#readWriteValue(d, 'hybrid', Hybrid, write, 0)
	readWriteValue(d, 'autoChoose', IndigoUIControls.System.AutoThreads, write, 1)
	#readWriteValue(d, 'override',Override, write, 0)
	
	readWriteValue(d, 'backgroundgain', IndigoUIControls.Environment.BGGain, write, 1.0)
	readWriteValue(d, 'worldscale', IndigoUIControls.Environment.WorldScale, write, 1.0)
	readWriteValue(d, 'worldscaleexp', IndigoUIControls.Environment.WorldScaleExp, write, 0)
	
	#if write == 0:
	#	if 'subdivs' in d:
	#		IndigoUIControls.Mesh.subdivMeshes = d['subdivs']
	#else:
	#	d['subdivs'] = IndigoUIControls.Mesh.subdivMeshes
		
	try:
		if write == 0:
			IndigoUIControls.Environment.BGColor.val = (d['backgroundcolor.r'], d['backgroundcolor.g'], d['backgroundcolor.b'])
		else:
			d['backgroundcolor.r'] = IndigoUIControls.Environment.BGColor.val[0]
			d['backgroundcolor.g'] = IndigoUIControls.Environment.BGColor.val[1]
			d['backgroundcolor.b'] = IndigoUIControls.Environment.BGColor.val[2]
	except KeyError:
		print ""
	
	if write != 0:
		Blender.Scene.GetCurrent().properties['indigo'] = d

def calcFL():
	#global hyperFocal, NearFocusLimit, FarFocusLimit
	currentscene = Blender.Scene.GetCurrent()
	camObj = currentscene.objects.camera
	
	if camObj:
		focal = camObj.data.lens
		coc = 0.029
		fstopIndex = float(FStop.val) / 10
		f_number = 2 ** (fstopIndex / 2)
		focus = FocusDistance.val * 1000
		hyperFocal = (focal * focal) / (f_number * coc) + focal
		NearFocusLimit = ((hyperFocal - focal) * focus) / (hyperFocal + focus - (2*focal))
		NearFocusLimit = str(float(int(NearFocusLimit / 10)) / 100)
		if ((hyperFocal - focus) <= 10):
			FarFocusLimit = "Infinity";  #set infinity at 10000m
		else:
			FarFocusLimit = ((hyperFocal-focal) * focus) / (hyperFocal - focus)
			FarFocusLimit = str(float(int(FarFocusLimit/ 10)) / 100)
		hyperFocal = str(float(int(hyperFocal) / 10) / 100)
	else:
		hyperFocal = "There's no active camera!"
		NearFocusLimit = "There's no active camera!"
		FarFocusLimit = "There's no active camera!"

# 'Camera' Tab
def drawCamera():
	drawScreenButtons()
	
	gui = guiLayout()
	gui.y = int(scrollbar.getTop()) - 70
	
	gui.title("Camera setting")
	
	gui.subtitle("Render dimensions")
	gui.newline("Width")
	IndigoUIControls.Camera.SizeX = Blender.Draw.Number(
		"",
		IndigoEvents.UI.evtchangesize,
		gui.x, gui.y,
		80, 18,
		Blender.Scene.GetCurrent().getRenderingContext().imageSizeX(),
		4, 10000,
		"width of the render max 10,000"
	)
	gui.units("pixels", 80)
	
	gui.newline("Height")
	IndigoUIControls.Camera.SizeY = Blender.Draw.Number(
		"",
		IndigoEvents.UI.evtchangesize,
		gui.x, gui.y,
		80, 18,
		Blender.Scene.GetCurrent().getRenderingContext().imageSizeY(),
		4, 10000,
		"height of the render max 10,000"
	)
	gui.units("pixels", 80)
	
	gui.newline("Scale")
	IndigoUIControls.Camera.ScaleSize = Blender.Draw.Menu(IndigoUIControls.Camera.strScaleSize, IndigoEvents.evtNoEvt, gui.x, gui.y, 80, 18, IndigoUIControls.Camera.ScaleSize.val, "scale image size of ...")
	
	gui.subtitle("Film setting")
	
	gui.newline("Film width")
	IndigoUIControls.Camera.FilmWidth = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 60, 18, IndigoUIControls.Camera.FilmWidth.val, 8, 254, "width of the \"film\" in mm")
	gui.units("mm", 60)
	
	gui.newline("Film ISO")
	IndigoUIControls.Camera.FilmIso = Blender.Draw.Menu("25%x25|50%x50|100%x100|200%x200|400%x400|800%x800|1600%x1600", IndigoEvents.evtNoEvt, gui.x, gui.y, 75, 18, IndigoUIControls.Camera.FilmIso.val, " \"film\" speed")
	gui.units("iso units", 75)
	
	gui.newline("Exposure")
	gui.units("1/", -21)
	IndigoUIControls.Camera.Shutter = Blender.Draw.Menu("1%x1|2%x2|4%x4|8%x8|15%x15|30%x30|60%x60|125%x125|250%x250|500%x500|1000%x1000|2000%x2000", IndigoEvents.evtNoEvt, gui.x, gui.y, 75, 18, IndigoUIControls.Camera.Shutter.val, "exposure time in seconds")
	gui.units("or", 75)
	
	IndigoUIControls.Camera.Autoexp = Blender.Draw.Toggle("Autoexposure", IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Camera.Autoexp.val, "enable auto exposure")
	
	# To be re-enabled.... perhaps
	gui.newline("F-Stop")
#	HStop = Blender.Draw.Toggle("halfstops", IndigoEvents.evtNoEvt,gui.x, gui.y, 60, 18,HStop.val, "enable the halfstop range in menu")
#	if HStop.val == 0:
	IndigoUIControls.Camera.FStop = Blender.Draw.Menu("0.5%x5|0.7%x7|1.0%x10|1.4%x14|2%x20|2.8%x28|4%x40|5.6%x56|8%x80|11%x110|16%x160|22%x220|32%x320|64%x640|90%x900|128%x1280", IndigoEvents.evtNoEvt, gui.x, gui.y, 75, 18, IndigoUIControls.Camera.FStop.val, "depth of field, (blurring of fore and background)")
	# else:
	# 	FStop = Blender.Draw.Menu("F-Stop%t|1.2%x12|1.7%x17|2.4%x24|3.3%x33|4.8%x48|6.6%x66|9.6%x96|13.2%x132|19.2%x192|26.4%x264|44%x440", IndigoEvents.evtNoEvt, 80, y-145, 75, 18, FStop.val, "depth of field, (blurring of fore and background)")
	
	#gui.newline("Lens Shift")
	#gui.newline("    Up ")
	#IndigoUIControls.Camera.LensShiftUp   = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Camera.LensShiftUp.val, -254.0, 254.0, "Up shift")
	#gui.units("mm", 100)
	#gui.newline("    Right ")
	#IndigoUIControls.Camera.LensShiftRight= Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Camera.LensShiftRight.val, -254.0, 254.0, "Right shift")
	#gui.units("mm", 100)
	
	gui.subtitle("       Aperture diffraction")
	
	IndigoUIControls.Camera.ApertureDiffr = Blender.Draw.Toggle(
		IndigoUIControls.PlusMinusToggleText(IndigoUIControls.Camera.ApertureDiffr),
		IndigoEvents.evtNoEvt,
		16, gui.y + 6,
		20, 18,
		IndigoUIControls.Camera.ApertureDiffr.val,
		"enable aperture diffraction"
	)
	
	if(IndigoUIControls.Camera.ApertureDiffr.val):
		gui.newline("   Post Process")
		IndigoUIControls.Camera.AperturePProc = Blender.Draw.Toggle(
			IndigoUIControls.OnOffToggleText(IndigoUIControls.Camera.AperturePProc),
			IndigoEvents.evtNoEvt,
			gui.x, gui.y,
			35, 18,
			IndigoUIControls.Camera.AperturePProc.val,
			"Post process aperture diffraction - faster, but less accurate"
		)
		gui.newline("   Aperture type")
		IndigoUIControls.Camera.ApertureType = Blender.Draw.Menu("Circular%x1|Generated%x2|Image%x3", IndigoEvents.evtNoEvt,gui.x, gui.y, 120, 18, IndigoUIControls.Camera.ApertureType.val, "Type pf aperture")
		
		if IndigoUIControls.Camera.ApertureType.val == 2:
			gui.newline("   Blades")
			IndigoUIControls.Camera.BladesNum = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Camera.BladesNum.val, 3, 20, "number of blades")
			
			gui.newline("   Offset")
			IndigoUIControls.Camera.BladeOffset = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 80, 18, IndigoUIControls.Camera.BladeOffset.val, 0, 0.5, "blades offset")
			gui.units("radians", 80)
			
			gui.newline("   Curvature")
			IndigoUIControls.Camera.BladeCurvature = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 80, 18, IndigoUIControls.Camera.BladeCurvature.val, 0, 10, "blade curvature radius")
			gui.units("radians", 80)
			
			gui.newline("   Start Angle")
			IndigoUIControls.Camera.StartAngle = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 80, 18, IndigoUIControls.Camera.StartAngle.val, 0, 2, "blades start angle")
			gui.units("radians", 80)
		
		if IndigoUIControls.Camera.ApertureType.val == 3:
			gui.newline("   Aperture Image")
			IndigoUIControls.Camera.ApertureImg = Blender.Draw.String("filename: ", IndigoEvents.evtNoEvt, gui.x, gui.y, 300, 18, IndigoUIControls.Camera.ApertureImg.val, 255, "the file name a square PNG with power-of-two dimension at least 512x512")
			Blender.Draw.Button("...", IndigoEvents.UI.evtloadaperture, gui.x + 310, gui.y, 20 ,18,"open file selector")
		
		if IndigoUIControls.Camera.ApertureType.val in [1, 2, 3]:
			gui.newline("   Obstacle map")
			IndigoUIControls.Camera.ObstacleMap = Blender.Draw.String("filename: ", IndigoEvents.evtNoEvt, gui.x, gui.y, 300, 18, IndigoUIControls.Camera.ObstacleMap.val, 255, "the file name a square PNG with power-of-two dimension at least 512x512")
			Blender.Draw.Button("...", IndigoEvents.UI.evtloadobstacle, gui.x + 310, gui.y, 20 ,18,"open file selector")
	
	gui.subtitle("Camera setting")
	
	gui.newline("White balance")
	IndigoUIControls.Camera.WhiteBalance = Blender.Draw.Menu(IndigoUIControls.Camera.strWhiteBalance, IndigoEvents.evtNoEvt, gui.x, gui.y, 75, 18, IndigoUIControls.Camera.WhiteBalance.val, "set the white balance for different lighting conditions")
	
	gui.newline("Focus distance")
	
	IndigoUIControls.Camera.AutoFocus = Blender.Draw.Toggle(
		"Autofocus",
		IndigoEvents.evtNoEvt,
		gui.x, gui.y,
		70, 18,
		IndigoUIControls.Camera.AutoFocus.val,
		"enable autofocus"
	)
	
	if not IndigoUIControls.Camera.AutoFocus.val:
		IndigoUIControls.Camera.FocusDistance = Blender.Draw.Number(
			"",
			IndigoEvents.UI.evtSetDofDist,
			gui.x + 80, gui.y,
			80, 18,
			Blender.Scene.GetCurrent().objects.camera.data.dofDist, #IndigoUIControls.Camera.FocusDistance.val,
			0.0, 5000,
			"the distance to objects that are clearly in focus"
		)
		gui.units("m", 160)
		
		gui.newline("")
		Blender.Draw.Button("Selected",
			IndigoEvents.UI.evtFocusSelected,
			gui.x, gui.y,
			60, 18,
			"get the distance from the selected object"
		)
		
		gui.units("or", 80)
		Blender.Draw.Button("3Dcursor",
			IndigoEvents.UI.evtFocusCursor,
			gui.x, gui.y,
			60, 18,
			"get the distance from the 3d cursor"
		)
	
	if (IndigoUIControls.Camera.FilmWidth.val == 35.0) and (False):
		calcFL()
		gui.newline("   Hyperfocal")
		gui.text(hyperFocal)
		gui.units("m")
		
		gui.newline("   Near focus")
		gui.text(NearFocusLimit)
		gui.units("m")
		
		gui.newline("   Far focus")
		gui.text(FarFocusLimit)
		gui.units("m")
	
	
	gui.subtitle("Motion blur")
	gui.newline(" ")
	IndigoUIControls.Camera.CamMoBlur = Blender.Draw.Toggle("Camera", IndigoEvents.evtNoEvt, gui.x, gui.y, 75, 18, IndigoUIControls.Camera.CamMoBlur.val, "Enable Camera Motion Blur")
	
	gui.units(" ", 80)
	IndigoUIControls.Camera.ObjMoBlur = Blender.Draw.Toggle("Object", IndigoEvents.evtNoEvt, gui.x, gui.y, 75, 18, IndigoUIControls.Camera.ObjMoBlur.val, "Enable Object Motion Blur")
	
	gui.units(" ", 80)
	IndigoUIControls.Camera.PrtMoBlur = Blender.Draw.Toggle("Particle", IndigoEvents.evtNoEvt, gui.x, gui.y, 75, 18, IndigoUIControls.Camera.PrtMoBlur.val, "Enable Particle Motion Blur; WARNING: SLOW!!")
	
	drawExportButtons(gui)
	
	return gui.y

# 'Environment' Tab
def drawEnv():
	drawScreenButtons()
	
	gui = guiLayout()
	gui.y = int(scrollbar.getTop()) - 70
	
	gui.title("Environment setting")
	gui.newline()
	
	gui.newline("Worldscale:")
	IndigoUIControls.Environment.WorldScale = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 70, 18, IndigoUIControls.Environment.WorldScale.val, 1,9, "scale factor of scene at export else 1 Bl unit == 1 metre in Indigo")
	gui.units('x',75)
	IndigoUIControls.Environment.WorldScaleExp = Blender.Draw.Number("10^", IndigoEvents.evtNoEvt, gui.x, gui.y, 70, 18, IndigoUIControls.Environment.WorldScaleExp.val, -4,6, "Exponent")
	
	gui.newline("Environment type")
	IndigoUIControls.Environment.EnvType = Blender.Draw.Menu(IndigoTypes.Environment.menuString(),IndigoEvents.evtNoEvt, gui.x, gui.y, 205, 18, IndigoUIControls.Environment.EnvType.val, "set the environment type")
	
	if IndigoTypes.Environment.isT(IndigoUIControls.Environment.EnvType.val, 'MAP'):
		gui.subtitle("Environment map")

		gui.newline("EXR map filename")
		IndigoUIControls.Environment.EnvFile = Blender.Draw.String("", IndigoEvents.evtNoEvt, gui.x, gui.y, 250, 18, IndigoUIControls.Environment.EnvFile.val, 255, "the file name of an .float/.exr probe")
		Blender.Draw.Button("...", IndigoEvents.UI.evtloadimg, gui.x + 255, gui.y, 20 ,18,"open file selector")

		gui.newline("Gain")
		IndigoUIControls.Environment.EnvGain = Blender.Draw.Number("Gain: ", IndigoEvents.evtNoEvt,gui.x, gui.y, 105, 18, IndigoUIControls.Environment.EnvGain.val, 0.001, 10000.00, "gain adjustment, range 0.001-1000")
		gui.units("dimensionless", 105)

		gui.newline("Mapping")
		IndigoUIControls.Environment.EnvMapType = Blender.Draw.Menu(IndigoUIControls.Environment.strEnvMapType, IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Environment.EnvMapType.val, "set the map type of the probe")

		if IndigoUIControls.Environment.EnvMapType.val == 0:
			gui.newline("  Width")
			IndigoUIControls.Environment.EnvWidth = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Environment.EnvWidth.val, 1, 10000, "input width (= height) of map")
			gui.units("pixels", 100)
		
	if IndigoTypes.Environment.isT(IndigoUIControls.Environment.EnvType.val, 'SUNSKY'):
		gui.subtitle("       Sun layer")
		IndigoUIControls.Environment.SunLayerToggle = Blender.Draw.Toggle(IndigoUIControls.PlusMinusToggleText(IndigoUIControls.Environment.SunLayerToggle), IndigoEvents.evtNoEvt, 16, gui.y + 5, 20,18, IndigoUIControls.Environment.SunLayerToggle.val, "Enable Sun Layer")
		
		# Sun layer name is fixed to 'Sun'
		#if IndigoUIControls.Environment.SunLayerToggle.val:
			#gui.newline("Layer #")
			#IndigoUIControls.Environment.SunLayer = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 50, 18, IndigoUIControls.Environment.SunLayer.val, 0, 9, "layer that sun will be drawn to")

		gui.subtitle("       Sky layer")
		IndigoUIControls.Environment.SkyLayerToggle = Blender.Draw.Toggle(IndigoUIControls.PlusMinusToggleText(IndigoUIControls.Environment.SkyLayerToggle), IndigoEvents.evtNoEvt, 16, gui.y + 5, 20,18, IndigoUIControls.Environment.SkyLayerToggle.val, "Enable Sky Layer")
		if IndigoUIControls.Environment.SkyLayerToggle.val:
			
			# Sky layer name is fixed to 'Sky'
			#gui.newline("Layer #")
			#IndigoUIControls.Environment.SkyLayer = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 50, 18, IndigoUIControls.Environment.SkyLayer.val, 0, 9, "layer that sky light will be drawn to")

			gui.newline("Sky turbidity")
			IndigoUIControls.Environment.Turbidity = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Environment.Turbidity.val, 1.5, 7.5, "set sky turbidity, range 2 clear - 5 hazy")
			gui.units("dimensionless", 100)

			# gui.newline("Extra Atmospheric")
			# ExtraAtm = Blender.Draw.Toggle('X', IndigoEvents.evtNoEvt,gui.x, gui.y,30,18,ExtraAtm.val,"Extra Atmospheric Scattering")
	
	if IndigoTypes.Environment.isT(IndigoUIControls.Environment.EnvType.val, 'BACKGROUND'):
		gui.subtitle("Background Colour")
		gui.newline("Colour", 15)
		IndigoUIControls.Environment.BGColor = Blender.Draw.ColorPicker(IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 30, IndigoUIControls.Environment.BGColor.val, "colour and picker (LMB)")
		gui.newline("Gain")
		IndigoUIControls.Environment.BGGain = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Environment.BGGain.val, 1e-4, 1e5, "set background color gain")
		
	
	drawExportButtons()
	return gui.y

# 'Renderer' Tab
def drawSettings(y):	
	drawScreenButtons()
	
	gui = guiLayout()
	gui.y = int(scrollbar.getTop()) - 70
	
	gui.title("Renderer settings")
	
	gui.subtitle("Render type")
	
	gui.newline("Alpha render") 
	IndigoUIControls.Renderer.AlphaMask = Blender.Draw.Toggle(IndigoUIControls.OnOffToggleText(IndigoUIControls.Renderer.AlphaMask), IndigoEvents.evtNoEvt, gui.x, gui.y, 35, 18, IndigoUIControls.Renderer.AlphaMask.val, "enable Alpha Mask rendering")

	gui.newline("Clay render")
	IndigoUIControls.Renderer.ClayRender = Blender.Draw.Toggle(IndigoUIControls.OnOffToggleText(IndigoUIControls.Renderer.ClayRender), IndigoEvents.evtNoEvt, gui.x, gui.y, 35,18, IndigoUIControls.Renderer.ClayRender.val, "only grey diffuse material")
	
	gui.subtitle("Render method")

	gui.newline("Metropolis mode")
	IndigoUIControls.Renderer.MLT = Blender.Draw.Toggle(IndigoUIControls.OnOffToggleText(IndigoUIControls.Renderer.MLT), IndigoEvents.evtNoEvt, gui.x, gui.y, 35, 18, IndigoUIControls.Renderer.MLT.val, "enable MLT (useful for indirect light) else pathtracer (for direct light)")
	gui.units("Complex light transport system for slow scenes", 35)

	gui.newline("Bidirectional mode")
	IndigoUIControls.Renderer.Bidirectional = Blender.Draw.Toggle(IndigoUIControls.OnOffToggleText(IndigoUIControls.Renderer.Bidirectional), IndigoEvents.evtNoEvt, gui.x, gui.y, 35, 18, IndigoUIControls.Renderer.Bidirectional.val, "enable bidirectional tracing (useful for indirect light and caustics)")
	gui.units("We recommend you leave this enabled", 35)
	
	gui.subtitle("      Advanced settings")
	IndigoUIControls.Renderer.rendererAdvanced = Blender.Draw.Toggle(IndigoUIControls.PlusMinusToggleText(IndigoUIControls.Renderer.rendererAdvanced), IndigoEvents.evtNoEvt, 16, gui.y + 6, 20, 18, IndigoUIControls.Renderer.rendererAdvanced.val, "")
	
	if IndigoUIControls.Renderer.rendererAdvanced.val:
		gui.newline("Splat filter")
		IndigoUIControls.Renderer.SplatFilter = Blender.Draw.Menu(IndigoTypes.Filters.menuString(), IndigoEvents.evtNoEvt,gui.x, gui.y,130,18, IndigoUIControls.Renderer.SplatFilter.val, "Splat Filter Type")
		
		if IndigoTypes.Filters.isT(IndigoUIControls.Renderer.SplatFilter.val, 'MN'):
			gui.newline("  Blur")
			IndigoUIControls.Renderer.SplatF_MN_Blur = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y,100, 18, IndigoUIControls.Renderer.SplatF_MN_Blur.val, 0.0, 1.0, "Blur value")
			gui.units("pixels", 100)

			gui.newline("  Ring")
			IndigoUIControls.Renderer.SplatF_MN_Ring = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y,100,18, IndigoUIControls.Renderer.SplatF_MN_Ring.val, 0.0, 1.0, "Ring value")
			gui.units("pixels", 100)

		gui.newline("Downsize filter")
	
		IndigoUIControls.Renderer.DownsizeFilter = Blender.Draw.Menu(IndigoTypes.Filters.menuString(), IndigoEvents.evtNoEvt,gui.x, gui.y,150,18, IndigoUIControls.Renderer.DownsizeFilter.val, "Downsize Filter Type")
		
		if IndigoTypes.Filters.isT(IndigoUIControls.Renderer.DownsizeFilter.val, 'MN'):
			gui.newline("  Blur")
			IndigoUIControls.Renderer.DownsizeF_MN_Blur = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y,100,18, IndigoUIControls.Renderer.DownsizeF_MN_Blur.val, 0.0, 1.0, "Blur value")
			gui.units("pixels", 100)

			gui.newline("  Ring")
			IndigoUIControls.Renderer.DownsizeF_MN_Ring = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y,100,18, IndigoUIControls.Renderer.DownsizeF_MN_Ring.val, 0.0, 1.0, "Ring value")
			gui.units("pixels", 100)
	
		gui.newline("MNCR")
		IndigoUIControls.Renderer.MaxNumConsRej = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Renderer.MaxNumConsRej.val, 0, 100000, "low values give more biased calculations and fewer fireflies")
		gui.units("rejections", 100)

		gui.newline("Supersampling")
		IndigoUIControls.Renderer.SuperSample = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 100,18, IndigoUIControls.Renderer.SuperSample.val,1,4, "Helps reduce anti alias edges but eats memory")
		gui.units("times", 100)
	
	drawExportButtons(gui)
	return gui.y

# 'System' Tab
def drawSystem(y):
	drawScreenButtons()
	
	gui = guiLayout()
	gui.y = int(scrollbar.getTop()) - 70
	
	gui.title("Renderer settings")

	# Override = Blender.Draw.Toggle("Override", IndigoEvents.evtNoEvt, 10, y-100, 80, 18, Override.val, "Override settings with ini file values or defaults")
	
	if utils.isMac():
		gui.subtitle('Indigo Location')
		gui.newline("Indigo App")
		IndigoUIControls.System.IndigoPath = Blender.Draw.String("", IndigoEvents.evtNoEvt, gui.x, gui.y, 250, 18, IndigoUIControls.System.IndigoPath.val, 399, "Path to Indigo app package")
		Blender.Draw.Button("...", IndigoEvents.UI.evtSetIndigoPath, gui.x + 255, gui.y, 20, 18, "open file selector")
	
	gui.subtitle("Performance")

	gui.newline("Number of threads")
	IndigoUIControls.System.Threads = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y,50,18, IndigoUIControls.System.Threads.val, 1,8, "the number of processor threads/cores to be used")
	gui.units(" or ",50)
	IndigoUIControls.System.AutoThreads = Blender.Draw.Toggle("Automatic", IndigoEvents.evtNoEvt, gui.x, gui.y,80,18, IndigoUIControls.System.AutoThreads.val, "Enable auto choice of threads number")
	
#	gui.newline("Display period")
#	IndigoUIControls.System.DisplayPeriod = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y,100, 18, IndigoUIControls.System.DisplayPeriod.val, 10, 100000, "the time between GUI display updates [sec]")
#	gui.units("seconds", 100)
	
	gui.newline("Save period")
	IndigoUIControls.System.ImageSave = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y,100, 18, IndigoUIControls.System.ImageSave.val, 20, 100000, "the time between saving .png images [sec]")
	gui.units("seconds", 100)

#	FrameUp = Blender.Draw.Number("Frame Upl:", IndigoEvents.evtNoEvt, 10, y-160, 205, 18, FrameUp.val, 20, 10000, "the time between network slave uploads of image to master [sec]")

	gui.subtitle("Halting")
	
	gui.newline("Halt time")
	IndigoUIControls.System.HaltTime = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y,100, 18, IndigoUIControls.System.HaltTime.val, -1, 86400, "the rendering duration [sec] setting -1 will continue until manual exit")
	gui.units("seconds", 100)
	
	gui.newline("Halt SPP")
	IndigoUIControls.System.HaltSamples = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y,100, 18, IndigoUIControls.System.HaltSamples.val, -1, 10000, "the rendering quality setting -1 will continue until manual exit")
	gui.units("samples per pixel", 100)
	gui.newline('Close GUI On Halt')
	IndigoUIControls.System.CloseGUIOnHalt = Blender.Draw.Toggle(
		IndigoUIControls.OnOffToggleText(IndigoUIControls.System.CloseGUIOnHalt),
		IndigoEvents.evtNoEvt,
		gui.x, gui.y,
		35, 18,
		IndigoUIControls.System.CloseGUIOnHalt.val,
		"Close the GUI when halt spp or time is reached"
	)
	
	gui.subtitle("Image saving options")

	gui.newline("Add Indigo logo")
	IndigoUIControls.System.Watermark = Blender.Draw.Toggle(IndigoUIControls.OnOffToggleText(IndigoUIControls.System.Watermark), IndigoEvents.evtNoEvt, gui.x + 20, gui.y, 35, 18, IndigoUIControls.System.Watermark.val,"Add indigo logo")

	gui.newline("Write untonemapped EXR")
	IndigoUIControls.System.SaveUTMExr = Blender.Draw.Toggle(IndigoUIControls.OnOffToggleText(IndigoUIControls.System.SaveUTMExr), IndigoEvents.evtNoEvt, gui.x + 20, gui.y, 35, 18, IndigoUIControls.System.SaveUTMExr.val, "save image also as an untonemapped .exr file")

	gui.newline("Write tonemapped EXR")
	IndigoUIControls.System.SaveTMExr = Blender.Draw.Toggle(IndigoUIControls.OnOffToggleText(IndigoUIControls.System.SaveTMExr), IndigoEvents.evtNoEvt, gui.x + 20, gui.y, 35, 18, IndigoUIControls.System.SaveTMExr.val, "save image also as a tonemapped .exr file")

	gui.newline("Write IGI")
	IndigoUIControls.System.SaveIgi = Blender.Draw.Toggle(IndigoUIControls.OnOffToggleText(IndigoUIControls.System.SaveIgi), IndigoEvents.evtNoEvt, gui.x + 20, gui.y, 35, 18, IndigoUIControls.System.SaveIgi.val, "save image also as an .igi file WARNING: can be large and slow")
	
	gui.subtitle("Networking")
	gui.newline('Network rendering')
	IndigoUIControls.System.NetworkWM = Blender.Draw.Menu(
		IndigoTypes.Networking.menuString(),
		IndigoEvents.evtNoEvt,
		gui.x, gui.y,
		150, 18,
		IndigoUIControls.System.NetworkWM.val,
		"Run indigo as a network master"
	)
	
	gui.subtitle("Other...")
	
	gui.newline("Logging")
	IndigoUIControls.System.Logging = Blender.Draw.Toggle(IndigoUIControls.OnOffToggleText(IndigoUIControls.System.Logging), IndigoEvents.evtNoEvt, gui.x, gui.y, 35, 18, IndigoUIControls.System.Logging.val, "enable writing a log.txt")
	
	gui.newline("Information overlay")
	IndigoUIControls.System.IOverlay = Blender.Draw.Toggle(IndigoUIControls.OnOffToggleText(IndigoUIControls.System.IOverlay), IndigoEvents.evtNoEvt, gui.x, gui.y, 35, 18, IndigoUIControls.System.IOverlay.val, "enable info text writing onto the image")

	gui.newline("PNG file name")
	IndigoUIControls.System.IndigoOutPath = Blender.Draw.String("", IndigoEvents.evtNoEvt, gui.x, gui.y, 250, 18, IndigoUIControls.System.IndigoOutPath.val, 399, "Path to Indigo output")
	Blender.Draw.Button("...", IndigoEvents.UI.evtSetIndigoOutPath, gui.x + 255, gui.y, 20, 18, "open file selector")
	
	gui.newline("Autogenerate igs path")
	IndigoUIControls.System.DefaultExport = Blender.Draw.Toggle(IndigoUIControls.OnOffToggleText(IndigoUIControls.System.DefaultExport), IndigoEvents.evtNoEvt,gui.x, gui.y,35,18, IndigoUIControls.System.DefaultExport.val, "use \"default.igs\" as the file name")

	gui.newline("Autostart indigo ")
	IndigoUIControls.System.ExecuteIndigo = Blender.Draw.Toggle(IndigoUIControls.OnOffToggleText(IndigoUIControls.System.ExecuteIndigo), IndigoEvents.evtNoEvt,gui.x, gui.y,35,18, IndigoUIControls.System.ExecuteIndigo.val, "enable auto start of Indigo after export - to path set in Indigowrapper.conf")
	
	# if not ExportMeshs.val: SepMeshFile.val = 1  # without mesh export force use of external mesh file
	# SepMeshFile = Blender.Draw.Toggle("sep. meshes", IndigoEvents.evtNoEvt,260,gui.y,80,18, SepMeshFile.val, "enable saving meshes to a separate file")
	# SepMatFile = Blender.Draw.Toggle("sep. mats", IndigoEvents.evtNoEvt, 340,gui.y,80,18, SepMatFile.val, "enable saving materials to a separate file")
	
	gui.newline("Process priority")
	if sys.platform[:3] == 'win':
		IndigoUIControls.System.ProcessPrio=Blender.Draw.Menu("NORMAL %x0|BELOW_NORMAL %x1", IndigoEvents.evtNoEvt, gui.x, gui.y, 200, 18, IndigoUIControls.System.ProcessPrio.val, "set process priority for Indigo")
	else:
		IndigoUIControls.System.ProcessPrio=Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 50, 18, IndigoUIControls.System.ProcessPrio.val, 1, 19, "the NICE increment")
	
	drawExportButtons()
	return gui.y

# 'Mesh' Tab
def meshGUI():
	def MeshSubdivReset():
		IndigoUIControls.Mesh.MaxNumSubdiv.val = 2
		IndigoUIControls.Mesh.SubdivPixelTreshold.val = 0.0
		IndigoUIControls.Mesh.SubdivCurvatureTreshold.val = 0.1
		IndigoUIControls.Mesh.SubdivSmooth.val = 1
	
	drawScreenButtons()
	
	gui = guiLayout()
	gui.y = int(scrollbar.getTop()) - 70
	gui.title("Mesh subdivision")
	
	scn=Blender.Scene.GetCurrent()
	if len(scn.objects.selected) == 1:
		obj = Blender.Object.GetSelected()[0]
		
		meshname=obj.getData(name_only=1)
		objecttype = obj.getType()
		if(objecttype == "Curve"):
			meshname +='_cu'
		imeshname = meshname.replace("<","-").replace(">","-")
		
		mesh = obj.getData()
		try:
			d=mesh.properties['subdivision'].convert_to_pyobject()
		except:
			d = {}
		#Subdivision
		if imeshname in d:
			IndigoUIControls.Mesh.meshSubdivide.val = 1
			IndigoUIControls.Mesh.MaxNumSubdiv.val  = d[imeshname][0]
			IndigoUIControls.Mesh.SubdivPixelTreshold.val = d[imeshname][1]
			IndigoUIControls.Mesh.SubdivCurvatureTreshold.val = d[imeshname][2]
			IndigoUIControls.Mesh.SubdivSmooth.val = d[imeshname][3]
		else:
			IndigoUIControls.Mesh.meshSubdivide.val = 0
			MeshSubdivReset()
			
		gui.subtitle("       Subdivide")
	
		IndigoUIControls.Mesh.meshSubdivide = Blender.Draw.Toggle(IndigoUIControls.PlusMinusToggleText(IndigoUIControls.Mesh.meshSubdivide), IndigoEvents.UI.evtSubdivMesh, 16, gui.y + 6, 20, 18, IndigoUIControls.Mesh.meshSubdivide.val, "enable Indigo Subdivide")
		
		if IndigoUIControls.Mesh.meshSubdivide.val:
			gui.newline("Max Num Subdivisions: ")
			IndigoUIControls.Mesh.MaxNumSubdiv = Blender.Draw.Number("", IndigoEvents.UI.evtUpdateSubdiv, gui.x+80, gui.y, 80, 18, IndigoUIControls.Mesh.MaxNumSubdiv.val, 1, 20, "Set the Max number for subdivision level")
			gui.newline("Pixel Treshold")
			IndigoUIControls.Mesh.SubdivPixelTreshold = Blender.Draw.Number("", IndigoEvents.UI.evtUpdateSubdiv, gui.x+80, gui.y, 80, 18, IndigoUIControls.Mesh.SubdivPixelTreshold.val, 0.0, 100.0, "triangles will not be subdivided if they subtend a smaller pixel width than this threshold")
			gui.newline("Curvature Treshold")
			IndigoUIControls.Mesh.SubdivCurvatureTreshold = Blender.Draw.Number("", IndigoEvents.UI.evtUpdateSubdiv, gui.x+80, gui.y, 80, 18, IndigoUIControls.Mesh.SubdivCurvatureTreshold.val, -1, 100.0, "triangles will not be subdivided if their curvature is lower than this value")
			gui.newline("")
			IndigoUIControls.Mesh.SubdivSmooth = Blender.Draw.Toggle("Subdivision Smoothing", IndigoEvents.UI.evtUpdateSubdiv, gui.x+80, gui.y, 160, 18, IndigoUIControls.Mesh.SubdivSmooth.val, "enable Subdivision Smoothing")
	drawExportButtons()
	return gui.y


# 'Tonemapping' Tab
def drawTonemap(y):	
	drawScreenButtons()
	
	gui = guiLayout()
	gui.y = int(scrollbar.getTop()) - 70

	gui.title("Tonemapping")
	gui.newline()
	
	gui.newline("Tonemapping method")
	IndigoUIControls.Tonemapping.ToneMapType = Blender.Draw.Menu(IndigoTypes.Tonemapping.menuString(), IndigoEvents.evtNoEvt, gui.x, gui.y, 105, 18, IndigoUIControls.Tonemapping.ToneMapType.val, "set the type of the tonemapping")
	
	if IndigoTypes.Tonemapping.isT(IndigoUIControls.Tonemapping.ToneMapType.val, 'LINEAR'):
		gui.subtitle("Linear tonemapping")
		gui.newline("Scale")
		IndigoUIControls.Tonemapping.ToneMapScale = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Tonemapping.ToneMapScale.val, 0, 1, "Linear scale: refer to the Indigo Manual ;)")
		gui.units(" x 10 ^", 100)
		
		IndigoUIControls.Tonemapping.ToneMapScaleExp = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 35, 18, IndigoUIControls.Tonemapping.ToneMapScaleExp.val, -20, 20, "Linear scale exponent")
		#gui.units("dimensionless", 35)
		gui.newline()
		gui.units("( = %.12f)" % (IndigoUIControls.Tonemapping.ToneMapScale.val * 10**IndigoUIControls.Tonemapping.ToneMapScaleExp.val))
	
	if IndigoTypes.Tonemapping.isT(IndigoUIControls.Tonemapping.ToneMapType.val, 'REINHARD'):
		gui.subtitle("Reinhard tonemapping")

		gui.newline("Burn")
		IndigoUIControls.Tonemapping.ToneMapBurn = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Tonemapping.ToneMapBurn.val, 0.1, 10.0, "burn to white, 0.1= a lot, 10= none")
		gui.units("dimensionless", 100)
		
		gui.newline("Pre-scale")
		IndigoUIControls.Tonemapping.ToneMapPreScale = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Tonemapping.ToneMapPreScale.val, 0,50.0, "scales the mid-grey-point of the rendered pixels, practical range 1-10")
		gui.units("dimensionless", 100)
		
		gui.newline("Post scale")
		IndigoUIControls.Tonemapping.ToneMapPostScale = Blender.Draw.Number("", IndigoEvents.evtNoEvt, gui.x, gui.y, 100, 18, IndigoUIControls.Tonemapping.ToneMapPostScale.val, 0,50.0, "scales the white-point of the rendered pixels , practical range 1-10 ")
		gui.units("dimensionless", 100)
	
	if IndigoTypes.Tonemapping.isT(IndigoUIControls.Tonemapping.ToneMapType.val, 'CAMERA'):
		gui.subtitle("Camera tonemapping")

		gui.newline("EV Adjust")
		IndigoUIControls.Tonemapping.ToneMapEV = Blender.Draw.Number("", IndigoEvents.evtNoEvt,gui.x, gui.y,105,18, IndigoUIControls.Tonemapping.ToneMapEV.val,-20,20,"Exposure Value Adjustment")
		gui.units("exposure stops", 105)

		gui.newline("Film")
		IndigoUIControls.Tonemapping.ToneMapResponseCurve = Blender.Draw.Menu(IndigoUIControls.Tonemapping.strToneMapResponseCurve, IndigoEvents.evtNoEvt,gui.x, gui.y,185,18, IndigoUIControls.Tonemapping.ToneMapResponseCurve.val, "Choose Camera Response Curve")
		
	drawExportButtons()
	return gui.y

# Draw complete GUI
def drawGUI():
	texts=Blender.Text.Get()
	shader="Indigo Shader:%t|NONE%x-1"
	for s in texts:
		shadername=s.getName()
		if(shadername[:4] == "isl_"):
			if len(IndigoGlobals.ShaderList) == 0:
				IndigoGlobals.ShaderList.append(s.getName())
			else:
				i=0
				flag=0
				for l in IndigoGlobals.ShaderList:
					if IndigoGlobals.ShaderList[i] == s.getName():
						flag=1
						break
					i+=1
				if not flag:
					IndigoGlobals.ShaderList.append(s.getName())
	
	responsecurvepath = os.path.join(utils.getResourcesPath(),"data\camera_response_functions")
	if os.path.exists(responsecurvepath):
		IndigoUIControls.Tonemapping.strToneMapResponseCurve = "Response Curve:%t "
		curvefiles = os.listdir(responsecurvepath)
		i = 0
		for cfile in curvefiles:
			IndigoUIControls.Tonemapping.strToneMapResponseCurve += "| " + cfile[:-4] + " %x" +str(i)
			IndigoUIControls.Camera.tonemapcurves[i] = cfile[:-4]
			i += 1
	else:
		IndigoUIControls.Tonemapping.strToneMapResponseCurve = "Response Curve:%t | No curves installed %x1"
	
	# TODO - factor out global button calls from 'Tab' functions
	# drawScreenButtons()
	
	if IndigoGlobals.Screen == 0:
		UIHeight = drawCamera()
	if IndigoGlobals.Screen == 1:
		UIHeight = drawEnv()
	if IndigoGlobals.Screen == 2:
		UIHeight = drawSettings(230)
	if IndigoGlobals.Screen == 3:
		UIHeight = drawSystem(230)
	if IndigoGlobals.Screen == 4:
		UIHeight = drawTonemap(230)
	if IndigoGlobals.Screen == 6:
		UIHeight = meshGUI()
	if IndigoGlobals.Screen == 5:
		UIHeight = materialGUI()
		
	#drawExportButtons()
	scrollbar.height = scrollbar.getTop() - UIHeight + 20
	scrollbar.draw()

def drawScreenButtons():
	y = int(scrollbar.getTop())
	Blender.BGL.glColor3f(0.2,0.2,0.2)
	AreaDims = Blender.Window.GetAreaSize()
	Blender.BGL.glRectf(0,0,AreaDims[0],AreaDims[1])
	
	IndigoGlobals.indigoLogo.reload()
	IndigoGlobals.indigoLogo.glLoad()
	drawImage(IndigoGlobals.indigoLogo, 0, AreaDims[1] - 512, 1.0, 1.0)
	
	# Draw the header bar
	Blender.BGL.glColor3f(0.1,0.1,0.1)
	Blender.BGL.glRectf(0,y,460,y-70)
	Blender.BGL.glColor3f(1,1,1);Blender.BGL.glRasterPos2i(10,y-25) ; Blender.Draw.Text("Blendigo  v" + blendigoVersion + "  ::  Indigo exporter module", "small")
	Blender.BGL.glRasterPos2i(370,y-25) ; Blender.Draw.Text("Press Q to close", "tiny")
	Blender.BGL.glColor3f(0.6, 0.6, 0.6) 
	Blender.BGL.glRectf(0,y-70,460,y-72)
	Blender.BGL.glColor3f(0.9, 0.9, 0.9);
	
	x = 10
	y -= 20
	w = 55
	p = 6
	Blender.Draw.Toggle("Camera", IndigoEvents.UI.openCamera, x, y-40, w, 18, (IndigoGlobals.selectedSection == 'camera'), "set camera parameters")
	x += (w+p)
	
	Blender.Draw.Toggle("Environment", IndigoEvents.UI.openEnv, x, y-40, w, 18, (IndigoGlobals.selectedSection == 'environment'), "set environment parameters")
	x += (w+p)
	
	Blender.Draw.Toggle("Renderer", IndigoEvents.UI.openRSet, x, y-40, w, 18, (IndigoGlobals.selectedSection == 'renderer'), "set rendering parameters")
	x += (w+p)
	
	Blender.Draw.Toggle("Tonemap", IndigoEvents.UI.openTmap, x, y-40, w, 18, (IndigoGlobals.selectedSection == 'tonemap'), "set tonemap parameters")
	x += (w+p)
	
	Blender.Draw.Toggle("System", IndigoEvents.UI.openSSet, x, y-40, w, 18, (IndigoGlobals.selectedSection == 'system'), "set system options")
	x += (w+p)
	
	IndigoUIControls.Mesh.meshGui = Blender.Draw.Toggle("Mesh", IndigoEvents.UI.openMesh, x, y-40, w, 18, (IndigoGlobals.selectedSection == 'mesh'), "set Mesh parameters")
	x += (w+p)
	
	IndigoUIControls.Materials.matGui = Blender.Draw.Toggle("Materials", IndigoEvents.UI.openMat, x, y-40, w, 18, (IndigoGlobals.selectedSection == 'material'), "show material editor")
	x += (w+p)

def drawExportButtons(gui=0):
	gui = guiLayout()
	y = int(scrollbar.getTop()) - 70
	
	Blender.BGL.glColor3f(0.6, 0.6, 0.6)
	Blender.BGL.glRectf(225, y, 440, y-36)

	IndigoUIControls.System.ExportMeshs = Blender.Draw.Toggle(
		"xm",
		IndigoEvents.evtNoEvt,
		230, y-31,
		20, 24,
		IndigoUIControls.System.ExportMeshs.val,
		"Export meshes? Disable to speed-up export time."
	)
	Blender.Draw.Button("RENDER", IndigoEvents.UI.evtExport, 255, y-31, 60, 24, "open file selector to export and save Indigo .igs")
	
	# Animation flyby mode (static meshes)
	IndigoUIControls.System.FlybyMode = Blender.Draw.Toggle(
		'fb',
		IndigoEvents.evtNoEvt,
		320, y-31,
		20, 24,
		IndigoUIControls.System.FlybyMode.val,
		'Export animation in flyby-mode re-using the same meshes for every frame'
	)
	Blender.Draw.Button("Render Anim", IndigoEvents.UI.evtExportAnim, 345, y-31, 90, 24, "open file selector to export and save for each frame WARNING: eats disk space")
	
	
# function that handles keyboard and mouse events
def event(evt, val):
	obj = Blender.Scene.GetCurrent().objects.active
	if(obj != None):
		try:
			try:
				if obj.getType() == "Text":
					cu=Blender.Curve.Get(obj.getData().getName())
					mat0=cu.materials[0]
				else:
					mat0 = (obj.getData(0,1).materials)[obj.activeMaterial - 1]
			except IndexError:
				mat0 = None
		except AttributeError:
			mat0 = None
	else: mat0 = None
	if obj != None and IndigoGlobals.activeObject != obj:
		IndigoGlobals.activeObject = obj
#		activeObjectMat0 = mat0
		Blender.Draw.Redraw(1)
	
	if (evt == Blender.Draw.MOUSEX) or (evt == Blender.Draw.MOUSEY):			   scrollbar.Mouse()
	if(evt == Blender.Draw.WHEELDOWNMOUSE) or (evt == Blender.Draw.DOWNARROWKEY):  scrollbar.scroll(16)
	if(evt == Blender.Draw.WHEELUPMOUSE) or (evt == Blender.Draw.UPARROWKEY):	   scrollbar.scroll(-16)
	if evt == Blender.Draw.QKEY:
		if Blender.Draw.PupMenu("Close Blendigo?%t|OK%x1") == 1:
			Blender.Draw.Exit()
		
	
	materialEvent(evt, val)


def setEnvMap(efilename):
	IndigoUIControls.Environment.EnvFile.val = efilename

def setObstacleMap(filename):
	IndigoUIControls.Camera.ObstacleMap.val = filename

def setApertureImage(filename):
	IndigoUIControls.Camera.ApertureImg.val = filename

def setIndigoPath(filename):
	IndigoUIControls.System.IndigoPath.val = filename
	readWriteSettings(1)

def setIndigoOutPath(filename):
	IndigoUIControls.System.IndigoOutPath.val = filename
	readWriteSettings(1)

# function that handles button events
def buttonEvt(evt):  
	if evt == IndigoEvents.UI.evtExport:
		readWriteSettings(1)
		if IndigoUIControls.System.DefaultExport.val == 1:
			defaultFile = os.path.join(utils.getTempDir(), "default.igs")
			exportStill(defaultFile)
		else:
			Blender.Window.FileSelector(exportStill, "Export", newFName('igs'))
	
	if evt == IndigoEvents.UI.evtExportAnim:
		#SepMatFile.val=0
		IndigoUIControls.System.ExportMeshs.val=1
		Blender.Window.FileSelector(exportAnim, "export animation", newFName('igs'))
	
	if evt == IndigoEvents.evtNoEvt:
		Blender.Draw.Redraw(1)
		readWriteSettings(1)
	
	if evt == IndigoEvents.UI.evtchangesize:
		Blender.Scene.GetCurrent().getRenderingContext().imageSizeX( IndigoUIControls.Camera.SizeX.val );
		Blender.Scene.GetCurrent().getRenderingContext().imageSizeY( IndigoUIControls.Camera.SizeY.val );
		Blender.Draw.Redraw(1)
	
	if evt == IndigoEvents.UI.openCamera:
		IndigoGlobals.selectedSection = 'camera'
		IndigoGlobals.Screen = 0
		drawGUI()
		Blender.Draw.Redraw(1)
	
	if evt == IndigoEvents.UI.openEnv:
		IndigoGlobals.selectedSection = 'environment'
		IndigoGlobals.Screen = 1
		drawGUI()
		Blender.Draw.Redraw(1)
	
	if evt == IndigoEvents.UI.openRSet:
		IndigoGlobals.selectedSection = 'renderer'
		IndigoGlobals.Screen = 2
		drawGUI()
		Blender.Draw.Redraw(1)
	
	if evt == IndigoEvents.UI.openSSet:
		IndigoGlobals.selectedSection = 'system'
		IndigoGlobals.Screen = 3
		drawGUI()
		Blender.Draw.Redraw(1)
	
	if evt == IndigoEvents.UI.openTmap:
		IndigoGlobals.selectedSection = 'tonemap'
		IndigoGlobals.Screen = 4
		drawGUI()
		Blender.Draw.Redraw(1)
	
	if evt == IndigoEvents.UI.openMesh:
		IndigoGlobals.selectedSection = 'mesh'

		try:
			obj = Blender.Object.GetSelected()[0]
			mesh = obj.getData()
			d = mesh.properties['subdivision'].convert_to_pyobject()
		except:
			d = {}
		
		IndigoGlobals.Screen = 6
		drawGUI()
		Blender.Draw.Redraw(1)
	
	if evt == IndigoEvents.UI.evtSubdivMesh:
		obj = Blender.Object.GetSelected()[0]
		mesh = obj.getData()
		try:
			d = mesh.properties['subdivision'].convert_to_pyobject()
		except:
			d = {}
		if IndigoUIControls.Mesh.meshSubdivide.val:
			meshname = obj.getData(name_only=1)
			objecttype = obj.getType()
			if(objecttype == "Curve"):
				meshname +='_cu'
			imeshname = meshname.replace("<","-").replace(">","-")
			d[imeshname] = [IndigoUIControls.Mesh.MaxNumSubdiv.val, IndigoUIControls.Mesh.SubdivPixelTreshold.val, IndigoUIControls.Mesh.SubdivCurvatureTreshold.val, IndigoUIControls.Mesh.SubdivSmooth.val]
			mesh.properties['subdivision'] = d
		else:
			meshname = obj.getData(name_only=1)
			objecttype = obj.getType()
			if(objecttype == "Curve"):
				meshname +='_cu'
			imeshname = meshname.replace("<","-").replace(">","-")
			if imeshname in d:
				del d[imeshname]
				mesh.properties['subdivision'] = d
		readWriteSettings(1)
	
	if evt == IndigoEvents.UI.evtUpdateSubdiv:
		obj = Blender.Object.GetSelected()[0]
		meshname = obj.getData(name_only=1)
		objecttype = obj.getType()
		if(objecttype == "Curve"):
			meshname +='_cu'
		imeshname = meshname.replace("<","-").replace(">","-")
		mesh = obj.getData()
		try:
			d = mesh.properties['subdivision'].convert_to_pyobject()
		except:
			d = {}
		if imeshname in d:
			d[imeshname] = [IndigoUIControls.Mesh.MaxNumSubdiv.val, IndigoUIControls.Mesh.SubdivPixelTreshold.val, IndigoUIControls.Mesh.SubdivCurvatureTreshold.val, IndigoUIControls.Mesh.SubdivSmooth.val]
			mesh = obj.getData()
			mesh.properties['subdivision'] = d
		readWriteSettings(1)
	
	if evt == IndigoEvents.UI.evtloadimg:
		Blender.Window.FileSelector(setEnvMap, "load environment map")
		readWriteSettings(1)
	
	if evt == IndigoEvents.UI.evtloadobstacle:
		Blender.Window.FileSelector(setObstacleMap,"load obstacle map")
	
	if evt == IndigoEvents.UI.evtloadaperture:
		Blender.Window.FileSelector(setApertureImage,"load aperture image")
		
	if evt == IndigoEvents.UI.evtSetIndigoOutPath:
		Blender.Window.FileSelector(setIndigoOutPath, "Select Out Indigo", newFName('png'))
	
	if evt == IndigoEvents.UI.evtSetIndigoPath:
		Blender.Window.FileSelector(setIndigoPath, "Select Indigo App Package", 'Indigo.app')
	
	if evt == IndigoEvents.UI.evtSetDofDist:
		Blender.Scene.GetCurrent().objects.camera.data.dofDist = IndigoUIControls.Camera.FocusDistance.val
		Blender.Draw.Redraw(1)
		readWriteSettings(1)
	
	if evt == IndigoEvents.UI.evtFocusSelected:
		if not setFocus("Selected"):
			IndigoUIControls.Camera.SelFocus.val=0
		
		readWriteSettings(1)
	
	if evt == IndigoEvents.UI.evtFocusCursor:
		setFocus("Cursor")
		readWriteSettings(1)
	
	if evt == IndigoEvents.UI.openMat:
		IndigoGlobals.selectedSection = 'material'
		
		IndigoGlobals.Screen = 5
		drawGUI()
		Blender.Draw.Redraw(1)
	
	materialButtonEvt(evt)

def setFocus(target='Selected'):
	currentscene = Blender.Scene.GetCurrent()
	camObj = currentscene.objects.camera
	loc1 = Blender.Mathutils.Vector(camObj.getLocation())
	
	if target == "Selected":
		try:
			selObj = Blender.Object.GetSelected()[0]
			loc2 = Blender.Mathutils.Vector(selObj.getLocation())
		except:
			Blender.Draw.PupMenu("Error%t|Select an Object")
			return False
		currentscene.objects.camera.data.dofDist = (loc1-loc2).length
		return True
	
	if target == "Cursor":
		loc2 = Blender.Mathutils.Vector(Blender.Window.GetCursorPos())
		currentscene.objects.camera.data.dofDist = (loc1-loc2).length
		return True


#===============================================================================
# Start Blendigo !!
#===============================================================================


print "  Platform             : "+PlatformInformation.type
print "  os.sep               : "+os.sep
print "  Indigo path          : "+utils.getGuiPath()

if sys.platform[:3] != 'win':
	linux = 1
	IndigoUIControls.System.ProcessPrio = Blender.Draw.Create(10)
else:
	IndigoUIControls.System.ProcessPrio = Blender.Draw.Create(1)

print(IndigoPaths)

# load settings
readWriteSettings(0)

# Register the UI
Blender.Draw.Register(drawGUI, event, buttonEvt)
